unit fConnectController;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 10/05/06 9:39a $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Interactively connect a TDBController object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/forms/fConnectController.pas $
*
* $History: fConnectController.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:00p
 * Created in $/CP Modernization/CliO/source/Common/forms
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  oDBController;

type
  TfrmConnectController = class(TForm)
    pgctrl: TPageControl;
    tbshtVistARPCBroker: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    edtBrokerServer: TEdit;
    edtBrokerPort: TEdit;
    edtAccessCode: TEdit;
    edtVerifyCode: TEdit;
    cbxCreateContext: TComboBox;
    tbshtAccess: TTabSheet;
    Label6: TLabel;
    edtFilename: TEdit;
    btnSelectAccessDB: TButton;
    tbshtCacheSQL: TTabSheet;
    Label7: TLabel;
    Label8: TLabel;
    Label12: TLabel;
    edtCacheSQLIPAddress: TEdit;
    edtCacheSQLDatabaseName: TEdit;
    edtCacheSQLPort: TEdit;
    tbshtWebService: TTabSheet;
    Label13: TLabel;
    lblConnectionType: TLabel;
    btnCancel: TButton;
    btnConnect: TButton;
    gbxMain: TGroupBox;
    Label9: TLabel;
    procedure rgTypeClick(Sender: TObject);
    procedure btnSelectAccessDBClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
  private
    fConnectionList: TList;
    fConnectionName: string;
    fConnectionIndex: integer;
    fController: TDBController;
    function getAVCodes: string;
    { Private declarations }
  public
    { Public declarations }
  end;

function ConnectController(aConnectionName: string; aDBController: TDBController): boolean;

implementation

uses
  uRegistryFunc;

type
  TConnectionType = (ctVistA, ctAccess, ctCacheSQL, ctJavaService);

const
  CONNECTIONTYPE: array[TConnectionType] of string = ('VistA RPC Broker', 'MS Access Database', 'Cache SQL', 'Java Service');

{$R *.dfm}

function ConnectController(aConnectionName: string; aDBController: TDBController): boolean;
var
  aParams: array of Variant;
  aConnType: TConnectionType;
  aRadioButton: TRadioButton;
begin
  with TfrmConnectController.Create(Application.MainForm) do
    try
      fConnectionList := TList.Create;
      for aConnType := Low(TConnectionType) to High(TConnectionType) do
        begin
          aRadioButton := TRadioButton.Create(gbxMain);
          with aRadioButton do
            begin
              Parent := gbxMain;
              Top := (Ord(aConnType) + 2) * Height;
              Left := 16;
              Tag := Ord(aConnType);
              Width := 275;
              Caption := CONNECTIONTYPE[aConnType];
              OnClick := rgTypeClick;
              fConnectionList.Add(aRadioButton);
            end;
        end;
      fConnectionName := aConnectionName;
      fController := aDBController;
      Font := Screen.MenuFont;
      gbxMain.Caption := Format(' Connection Name: %s ', [aConnectionName]);
      SetLength(aParams, 10);

      LoadConnection(fConnectionName, aParams);

      fConnectionIndex := aParams[0];
      if (fConnectionIndex > -1) and (fConnectionIndex < fConnectionList.Count) then
        TRadioButton(fConnectionList[fConnectionIndex]).Checked := True;

      case fConnectionIndex of
        -1:
          begin
            pgctrl.ActivePage := nil;
          end;
        0:
          begin
            pgctrl.ActivePage := tbshtVistARPCBroker;
            edtBrokerServer.Text := VarToStr(aParams[1]);
            edtBrokerPort.Text := VarToStr(aParams[2]);
            edtAccessCode.Text := VarToStr(aParams[3]);
            edtVerifyCode.Text := VarToStr(aParams[4]);
            cbxCreateContext.ItemIndex := cbxCreateContext.Items.IndexOf(VarToStr(aParams[5]));
          end;
        1:
          begin
            pgctrl.ActivePage := tbshtAccess;
            edtFilename.Text := VarToStr(aParams[1]);
          end;
        2:
          begin
            pgctrl.ActivePage := tbshtCacheSQL;
            edtCacheSQLIPAddress.Text := VarToStr(aParams[1]);
            edtCacheSQLPort.Text := VarToStr(aParams[2]);
            edtCacheSQLDatabaseName.Text := VarToStr(aParams[3]);
          end;
        3:
          begin
          end;
      end;
      Result := (ShowModal = mrOK);
    finally
      Free;
    end;
end;

procedure TfrmConnectController.rgTypeClick(Sender: TObject);
begin
  with TGroupBox(Sender) do
    begin
      fConnectionIndex := fConnectionList.IndexOf(Sender);
      pgctrl.ActivePageIndex := fConnectionIndex;
    end;
end;

procedure TfrmConnectController.btnSelectAccessDBClick(Sender: TObject);
begin
  with TOpenDialog.Create(self) do
    try
      Filter := 'Access Database Files|*.mdb';
      FilterIndex := 0;
      InitialDir := '.\';
      Title := Format('Open %s database', [fConnectionName]);
      if Execute then
        edtFilename.Text := Filename;
    finally
      Free;
    end;
end;

procedure TfrmConnectController.btnConnectClick(Sender: TObject);
begin
  case fConnectionIndex of
    0: fController.Initialize(fConnectionName, 0, [edtBrokerServer.Text, edtBrokerPort.Text, getAVCodes, cbxCreateContext.Text]);
    1: fController.Initialize(fConnectionName, 1, [edtFilename.Text]);
    2: fController.Initialize(fConnectionName, 2, [edtCacheSQLIPAddress.Text, edtCacheSQLPort.Text, edtCacheSQLDatabaseName.Text]);
  end;

  if fController.DBConnection.IsConnected then
    begin
      {Save it to the registry}
      case fConnectionIndex of
        0: SaveConnection(fConnectionName, [0, edtBrokerServer.Text, edtBrokerPort.Text, getAVCodes, cbxCreateContext.Text]);
        1: SaveConnection(fConnectionName, [1, edtFilename.Text]);
        2: SaveConnection(fConnectionName, [2, edtCacheSQLIPAddress.Text, edtCacheSQLPort.Text, edtCacheSQLDatabaseName.Text]);
      end;

      ModalResult := mrOK;
    end
  else
    MessageDlg('Unable to connect with these parameters', mtError, [mbOk], 0);
end;

function TfrmConnectController.getAVCodes: string;
begin
  if (edtAccessCode.Text <> '') and (edtVerifyCode.Text <> '') then
    Result := Format('%s;%s', [edtAccessCode.Text, edtVerifyCode.Text])
  else
    Result := '';
end;

end.

