unit fPersonLookup;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  oInterfaces;

type
  TfrmPersonLookup = class(TForm)
    btnSearch: TButton;
    btnOK: TButton;
    btnCancel: TButton;
    Label1: TLabel;
    edtLookup: TEdit;
    lbxSelectOne: TListBox;
    lblSelectOne: TLabel;
    procedure btnSearchClick(Sender: TObject);
    procedure lbxSelectOneClick(Sender: TObject);
    procedure edtLookupChange(Sender: TObject);
    procedure lbxSelectOneDblClick(Sender: TObject);
  private
    fPerson: IPerson;
    fPersonService: IPersonService;
    fList: IInterfaceList;
  public
    { Public declarations }
  end;

function LookupPerson(aPersonService: IPersonService; aCaption: string = ''): IPerson;

implementation

uses
  u508Dialogs;

{$R *.dfm}

function LookupPerson(aPersonService: IPersonService; aCaption: string = ''): IPerson;
begin
  with TfrmPersonLookup.Create(Application) do
    try
      TInterfaceList.Create.GetInterface(IInterfaceList, fList);
      fPerson := nil;
      fPersonService := aPersonService;
      edtLookup.Text := '';
      lbxSelectOne.Items.Clear;
      if aCaption <> '' then
        Caption := aCaption;
      if ShowModal = mrOK then
        Result := fPerson
      else
        Result := nil;
    finally
      Free;
    end;
end;

procedure TfrmPersonLookup.btnSearchClick(Sender: TObject);
var
  i: integer;
  aPerson: IPerson;
begin
  lbxSelectOne.Items.Clear;
  fPerson := nil;
  fList.Clear;
  fPersonService.ExecuteLookup(edtLookup.Text, fList);
  if fList.Count = 0 then
    MessageDlg508('No entries found matching ''' + edtLookup.Text + '''', mtInformation, [mbOk], 0)
  else
    for i := 0 to fList.Count - 1 do
      begin
        fList[i].QueryInterface(IPerson, aPerson);
        aPerson.AddToList(lbxSelectOne.Items);
      end;
  if lbxSelectOne.Items.Count > 0 then
    begin
      lbxSelectOne.ItemIndex := 0;
      btnOk.Enabled := lbxSelectOne.Items.Objects[0].GetInterface(IPerson, fPerson);
      btnSearch.Default := False;
      lbxSelectOne.SetFocus;
    end
  else
    begin
      btnOk.Enabled := False;
      edtLookup.SetFocus;
    end;
end;

procedure TfrmPersonLookup.edtLookupChange(Sender: TObject);
begin
  btnSearch.Enabled := (edtLookup.Text <> '');
  btnSearch.Default := (edtLookup.Text <> '');
end;

procedure TfrmPersonLookup.lbxSelectOneClick(Sender: TObject);
begin
  if lbxSelectOne.ItemIndex > -1 then
    btnOK.Enabled := lbxSelectOne.Items.Objects[lbxSelectOne.ItemIndex].GetInterface(IPerson, fPerson);
end;

procedure TfrmPersonLookup.lbxSelectOneDblClick(Sender: TObject);
begin
  if btnOk.Enabled then
    ModalResult := mrOK;
end;

end.

