unit fSelectNewPage;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 26 $  $Modtime: 12/05/11 2:22p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Used to add pages on the fly with filter definitions.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/forms/fSelectNewPage.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ImgList,
  ComCtrls,
  ExtCtrls,
  StdCtrls,
  ActnList,
  oCP_Exceptions,
  //oFlowsheet,
  oInterfaces;

type
  TfrmSelectNewPage = class(TForm)
    ImageList1: TImageList;
    btnCancel: TButton;
    ActionList1: TActionList;
    acUpdateComment: TAction;
    acSelectPageToActivate: TAction;
    acCancel: TAction;
    acChangeType: TAction;
    acAddPage: TAction;
    btnAction: TButton;
    pgctrl: TPageControl;
    tbshtActivate: TTabSheet;
    tbshtDeactivate: TTabSheet;
    acIsItReadyToActivate: TAction;
    acActivatePage: TAction;
    Panel1: TPanel;
    Label7: TLabel;
    edtDeactivationComment: TEdit;
    Panel2: TPanel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    lblDateTimeActivated: TLabel;
    lblDisplayName: TLabel;
    lblActivationComments: TLabel;
    acDeactivatePage: TAction;
    tbshtSelect: TTabSheet;
    pnlButtons: TPanel;
    pnlFilters: TPanel;
    lblFilters: TLabel;
    Shape1: TShape;
    pnlNameComments: TPanel;
    Label2: TLabel;
    Shape2: TShape;
    lblDisplay: TLabel;
    Label8: TLabel;
    edtComment: TEdit;
    edtDisplayName: TEdit;
    pnlHeader: TPanel;
    cbxMethod: TComboBox;
    cbxPosition: TComboBox;
    cbxLocation: TComboBox;
    cbxProduct: TComboBox;
    Label1: TLabel;
    Label9: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    lvPages: TListView;
    acSelectPageToDeactivate: TAction;
    Bevel1: TBevel;
    Shape4: TShape;
    Bevel2: TBevel;
    lblListHeader: TLabel;
    dtpCloseDate: TDateTimePicker;
    dtpCloseTime: TDateTimePicker;
    lblCloseHeader: TLabel;
    Bevel3: TBevel;
    Shape3: TShape;
    Label3: TLabel;
    Bevel4: TBevel;
    lblViewName: TLabel;
    Label4: TLabel;
    dtpDateOpened: TDateTimePicker;
    dtpTimeOpened: TDateTimePicker;
    procedure acCancelExecute(Sender: TObject);
    procedure acActivatePageExecute(Sender: TObject);
    procedure acDeactivatePageExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure acSelectPageToActivateExecute(Sender: TObject);
    procedure SelectPageToActivate(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure SelectPageToDeactivate(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure acSelectPageToDeactivateExecute(Sender: TObject);
  private
    fFlowsheet: ICPFlowsheet;
    fView: ICPFLowsheetView;
    fPage: ICPFlowsheetPage;
    fPageList: IInterfaceList;
    fViewID: string;
  end;

var
  frmSelectNewPage: TfrmSelectNewPage;

function ActivatePage(aFlowsheet: ICPFlowsheet): string;
function DeactivatePage(aPageList: IInterfaceList): boolean;

implementation

uses
  oCliOTypes,
  iCliOGrid,
  oCPTypes,
  u508Dialogs,
  //oObservation_View,
  uGlobalVars,
  XMLIntf,
  XMLDoc; {for the controllers}

{$R *.dfm}

function ActivatePage(aFlowsheet: ICPFlowsheet): string;
var
//  aPage: ICPFlowsheetPage;
  i: integer;
begin
  with TfrmSelectNewPage.Create(Application) do
    try
      if assigned(aFlowsheet) then
        aFlowsheet.QueryInterface(ICPFlowsheet, fFlowsheet)
      else
        fFlowsheet := nil;
      pnlHeader.Font.Style := [fsBold];
      pnlHeader.Caption := Format('Add Page to %s', [aFlowsheet.getName]);
      for i := 0 to fFlowsheet.PageCount - 1 do
        if fFlowsheet.Page(i).getPageType <> ptMandatory then
          with lvPages.Items.Add do
            begin
              Caption := fFlowsheet.Page(i).getFlowsheetViewName;
              SubItems.Add(Format(
                '%s (%s)',
                [fFlowsheet.Page(i).getDisplayName, CP_FLOWSHEET_PAGE_TYPE[fFlowsheet.Page(i).getPageType]]));
              SubItems.Add(aFlowsheet.Page(i).getFlowsheetViewID);
            end;
      lvPages.AlphaSort;

      MyTerminology.LoadList(cbxMethod.Items, ttMethod);
      MyTerminology.LoadList(cbxPosition.Items, ttPosition);
      MyTerminology.LoadList(cbxLocation.Items, ttLocation);
      MyTerminology.LoadList(cbxProduct.Items, ttProduct);

      cbxMethod.Items.Add('{None}');
      cbxPosition.Items.Add('{None}');
      cbxLocation.Items.Add('{None}');
      cbxProduct.Items.Add('{None}');

      cbxMethod.ItemIndex := 0;
      cbxPosition.ItemIndex := 0;
      cbxLocation.ItemIndex := 0;
      cbxProduct.ItemIndex := 0;

      dtpDateOpened.DateTime := Now;
      dtpTimeOpened.DateTime := Now;

      acSelectPageToActivate.Enabled := False;
      btnAction.Action := acSelectPageToActivate;
      pgctrl.ActivePage := tbshtSelect;
      lblListHeader.Caption := 'Available Pages';
      lvPages.OnSelectItem := SelectPageToActivate;
      lvPages.OnDblClick := acSelectPageToActivateExecute;

      ShowModal;
      //Result := (ShowModal = mrOK);
      //
      // We need to get the GUID for the new view before we get rid of the form.
      // We have to have it on the backside so we can find it and scroll to it.
      //

      result := FViewID;
    finally
      Free;
    end;
end;

function DeactivatePage(aPageList: IInterfaceList): boolean;
var
  aGridFrame: IGridFrame;
  aView: ICPFlowsheetView;
  i: integer;
begin
  with TfrmSelectNewPage.Create(Application) do
    try
      fPageList := aPageList;
      pnlHeader.Caption := 'Close Supplemental/Optional Page';
      pnlHeader.Font.Style := [fsBold];
      for i := 0 to fPageList.Count - 1 do
        if fPageList[i].QueryInterface(IGridFrame, aGridFrame) = 0 then
          if aGridFrame.getInterface.QueryInterface(ICPFlowsheetView, aView) = 0 then
            if aView.getSupplementalID <> '' then
              if (aView.getDeactivatedDateTime > 0) and (aView.getDeactivatedDateTime < Now) then
              //
              else
                with lvPages.Items.Add do
                  begin
                    Caption := aView.getName;
                    SubItems.Add(
                      aView.getDisplayName + ' (Opened: ' +
                      FormatDateTime('MMM D, YYYY hh:mm)', aView.getActivatedDateTime));
                    SubItems.Add(aView.getSupplementalID);
                  end;
      lvPages.AlphaSort;
      acSelectPageToDeactivate.Enabled := False;
      btnAction.Action := acSelectPageToDeactivate;
      lblListHeader.Caption := 'Active Pages';
      pgctrl.ActivePage := tbshtSelect;
      lvPages.OnSelectItem := SelectPageToDeactivate;
      lvPages.OnDblClick := acSelectPageToDeactivateExecute;
      Result := (ShowModal = mrOK);
    finally
      Free;
    end;
end;

procedure TfrmSelectNewPage.FormCreate(Sender: TObject);
begin
  tbshtActivate.TabVisible := False;
  tbshtDeactivate.TabVisible := False;
  tbshtSelect.TabVisible := False;
  Font := Application.MainForm.Font;
  lblFilters.Font.Style := [fsBold];
  lblDisplay.Font.Style := [fsBold];
  lblCloseHeader.Font.Style := [fsBold];
  lblListHeader.Font.Style := [fsBold];
  lblViewName.Font.Style := [fsBold];
end;

procedure TfrmSelectNewPage.SelectPageToActivate(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  i: integer;
begin
  acSelectPageToActivate.Enabled := False;
  if Selected then
    for i := 0 to fFlowsheet.PageCount - 1 do
      if fFlowsheet.Page(i).getFlowsheetViewID = Item.SubItems[1] then
        begin
          fPage := fFlowsheet.Page(i);
          acSelectPageToActivate.Enabled := True;
          Exit;
        end;
end;

procedure TfrmSelectNewPage.SelectPageToDeactivate(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  i: integer;
  aView: ICPFlowsheetView;
  aGridFrame: IGridFrame;
begin
  acSelectPageToDeactivate.Enabled := False;
  if Selected then
    for i := 0 to fPageList.Count - 1 do
      if fPageList[i].QueryInterface(IGridFrame, aGridFrame) = 0 then
        if aGridFrame.getInterface.QueryInterface(ICPFlowsheetView, aView) = 0 then
          if aView.getSupplementalID = Item.SubItems[1] then
            begin
              fView := aView;
              acSelectPageToDeactivate.Enabled := True;
              Exit;
            end;
end;

procedure TfrmSelectNewPage.acCancelExecute(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmSelectNewPage.acSelectPageToActivateExecute(Sender: TObject);
begin
  lblViewName.Caption := Format('Page Name: %s', [fPage.getDisplayName]);

  // We need to hide all of the Magic 4 stuff if this is an optional page instead of
  // a supplemental.

  if fPage.getPageType = ptOptional then
  begin
    lblFilters.Visible := false;
    Bevel1.Visible := false;
    label1.Visible := false;
    label9.Visible := false;
    label13.Visible := false;
    label14.Visible := false;
    cbxMethod.Visible := false;
    cbxPosition.Visible := false;
    cbxLocation.Visible := false;
    cbxProduct.Visible := false;
  end;

  pgctrl.ActivePage := tbshtActivate;
  btnAction.Action := acActivatePage;
  //
  // We don't want to call acIsItReadyToActivate, as it has no action associated
  // with it.  Doing so will cause an access violation.
  //acIsItReadyToActivate.Execute;
end;

procedure TfrmSelectNewPage.acSelectPageToDeactivateExecute(Sender: TObject);
begin
  pnlHeader.Caption := fView.getDisplayName;
  pgctrl.ActivePage := tbshtDeactivate;
  btnAction.Action := acDeactivatePage;
  lblDateTimeActivated.Caption :=
    FormatDateTime('YYYY-MM-DD hh:mm', fView.getActivatedDateTime) +
    Format(' by %s (%s)', [fView.getActivatedByName, fView.getActivatedByID]);
  lblDisplayName.Caption := fView.getDisplayName;
  lblActivationComments.Caption := fView.getActivatedComment;
  dtpCloseDate.DateTime := Now;
  dtpCloseTime.DateTime := Now;
end;

procedure TfrmSelectNewPage.acActivatePageExecute(Sender: TObject);
var
  aTerm: ITerm;
//  aSetID: string;
  aPosID: string;
  aLocID: string;
  aMetID: string;
  aProID: string;
  aDateTimeOpened: TDateTime;
  hh, mm, ss, ms: Word;
begin
  if (edtDisplayName.Text = '') or (edtComment.Text = '') then
    MessageDlg508('Pages must have a display name and a comment.', mtError, [mbOk], 0)
  else
    try
      with cbxMethod do
        if ItemIndex > 0 then
          if Items.Objects[ItemIndex].GetInterface(ITerm, aTerm) then
            aMetID := aTerm.getID
          else
            aMetID := '';

      with cbxPosition do
        if ItemIndex > 0 then
          if Items.Objects[ItemIndex].GetInterface(ITerm, aTerm) then
            aPosID := aTerm.getID
          else
            aPosID := '';

      with cbxLocation do
        if ItemIndex > 0 then
          if Items.Objects[ItemIndex].GetInterface(ITerm, aTerm) then
            aLocID := aTerm.getID
          else
            aLocID := '';

      with cbxProduct do
        if ItemIndex > 0 then
          if Items.Objects[ItemIndex].GetInterface(ITerm, aTerm) then
            aProID := aTerm.getID
          else
            aProID := '';

      DecodeTime(dtpTimeOpened.DateTime, hh, mm, ss, ms);
      aDateTimeOpened := trunc(dtpDateOpened.DateTime) + EncodeTime(hh, mm, 0, 0);

      fViewID := MyCPService.CreateNewSupplementalPage(
        fPage.getFlowsheetViewID, MyPatient.getDFN, aMetID, aPosID, aLocID, aProID,
        edtDisplayName.Text, MyUser.getDUZ, edtComment.Text, fPage.getPageType, aDateTimeOpened);
      if fViewID <> '' then
        ModalResult := mrOK
      else
        ModalResult := mrNone;
    except
      on E: Exception do
        MessageDlg508(E.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfrmSelectNewPage.acDeactivatePageExecute(Sender: TObject);
var
  aDateTimeClosed: TDateTime;
begin
  try
    if edtDeactivationComment.Text = '' then
      raise Exception.Create('Please enter a comment before closing');

    aDateTimeClosed := trunc(dtpCloseDate.DateTime) + Frac(dtpCloseTime.DateTime);

    if aDateTimeClosed <= fView.getActivatedDateTime then
      raise Exception.Create('Deactivation date/time must be greater than the activation date/time.');

    MyCPService.DeactivateSupplementalPage(
      fView.getSupplementalID, MyUser.getDUZ, edtDeactivationComment.Text, aDateTimeClosed);

    ModalResult := mrOK;
  except
    on E: Exception do
      MessageDlg508(E.Message, mtError, [mbOk], 0);
  end;
end;

end.

