unit oCPAccessControlListItem;

interface

uses
  Classes,
  ComCtrls,
  oInterfaces,
  oCPTypes,
  oCP_Exceptions,
  oDBController,
  uCPXML,
  XMLIntf,
  XMLDoc;

type
  TCPAccessControlListItem = class(TInterfacedObject, ICPAccessControlListItem)
    fItemID: string;
    fUserID: string;
    fAccessLevel: TCPAccessLevel;
  private
    function getItemID: string; stdcall;
    function getUserID: string; stdcall;
    function getAccessLevel: TCPAccessLevel; stdcall;

    procedure setItemID(const aValue: string); stdcall;
    procedure setUserID(const aValue: string); stdcall;
    procedure setAccessLevel(const aValue: TCPAccessLevel); stdcall;

    function AddToList(aList: TStrings; aCaption: string): integer; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;

    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    constructor Create(aXMLNode: IXMLNode); overload;
    constructor Create(aItemID, aUserID: string; aAccessLevel: TCPAccessLevel); overload;
    destructor Destroy; override;
  end;

implementation

{ TCPAccessControlListItem }

constructor TCPAccessControlListItem.Create(aXMLNode: IXMLNode);
begin
  inherited Create;
  if Assigned(aXMLNode) then
    LoadFromXML(aXMLNode)
  else
    begin
      fItemID := '';
      fUserID := '';
      fAccessLevel := alReadOnly;
    end;
end;

constructor TCPAccessControlListItem.Create(aItemID, aUserID: string; aAccessLevel: TCPAccessLevel);
begin
  inherited Create;
  fItemID := aItemID;
  fUserID := aUserID;
  fAccessLevel := aAccessLevel;
end;

function TCPAccessControlListItem.AddToList(aList: TStrings; aCaption: string): integer;
begin
  Result := aList.AddObject(aCaption, Self);
end;

function TCPAccessControlListItem.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := fItemID;
  Result.Data := Self;
end;

destructor TCPAccessControlListItem.Destroy;
begin
  inherited;
end;

function TCPAccessControlListItem.getItemID: string;
begin
  Result := fItemID;
end;

function TCPAccessControlListItem.getUserID: string;
begin
  Result := fUserID;
end;

function TCPAccessControlListItem.getAccessLevel: TCPAccessLevel;
begin
  Result := fAccessLevel;
end;

procedure TCPAccessControlListItem.setItemID(const aValue: string);
begin
  fItemID := aValue;
end;

procedure TCPAccessControlListItem.setUserID(const aValue: string);
begin
  fUserID := aValue;
end;

procedure TCPAccessControlListItem.setAccessLevel(const aValue: TCPAccessLevel);
begin
  fAccessLevel := aValue;
end;

procedure TCPAccessControlListItem.LoadFromXML(aXMLNode: IXMLNode);
begin
  fItemID := XMLValueAsStr(aXMLNode, 'ITEM_ID');
  fUserID := XMLValueAsStr(aXMLNode, 'USER_ID');
  fAccessLevel := TCPAccessLevel(XMLValueAsInt(aXMLNode, 'ACCESS_LEVEL', 0));
end;

end.

