unit oCPBackgroundTask;

interface

uses
  Classes,
  ComCtrls,
  Dialogs,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  oCPTypes,
  oCliOTypes,
  oCliO;

type
  TCPBackgroundTask = class(TInterfacedObject, ICPBackgroundTask, ICPConsoleItem)
  private
    fDBConnection: IDBConnection;
    fXMLDocument: IXMLDocument;
    fXMLRecord: IXMLNode;
    fTaskID: string;
    fTaskDesc: string;
    fSystemID: string;
    function getTaskID: string; stdcall;
    function getTaskName: string; stdcall;
    function getTaskStatus: string; stdcall;
    function getTaskRecurrence: integer; stdcall;
    function getDescription: string; stdcall;
    function getIsScheduled: boolean; stdcall;
    function getScheduledAt: TDateTime; stdcall;
    procedure Start(aStartDateTime: TDateTime); stdcall;
    procedure Stop; stdcall;

    {ICPConsoleItem}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getSystemID: string; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
  public
    constructor Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

uses
  oCP_Exceptions,
  uCPXML;

{ TCliOBackgroundTask }

constructor TCPBackgroundTask.Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fXMLDocument := NewXMLDocument;
  fTaskID := XMLValueAsStr(aXMLNode, 'TASK_ID');
  fTaskDesc := XMLValueAsStr(aXMLNode, 'TASK_NAME');
  fSystemID := XMLValueAsStr(aXMLNode, 'TASK_SYSTEM_ID');
end;

destructor TCPBackgroundTask.Destroy;
begin
  fXMLRecord := nil;
  fXMLDocument := nil;
  fDBConnection := nil;
  inherited;
end;

procedure TCPBackgroundTask.AddToList(aList: TStrings);
begin
  aList.AddObject(fTaskDesc, Self);
end;

procedure TCPBackgroundTask.AddToListView(aListView: TListView);
begin
  //
end;

function TCPBackgroundTask.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fTaskDesc, Self);
end;

function TCPBackgroundTask.getActive: boolean;
begin
  Result := True;
end;

function TCPBackgroundTask.getCanDelete: boolean;
begin
  Result := False;
end;

function TCPBackgroundTask.getCanRename: boolean;
begin
  Result := False;
end;

function TCPBackgroundTask.getComment: string;
begin
  Result := fTaskDesc;
end;

function TCPBackgroundTask.getDescription: string;
begin
  Result := fTaskDesc;
end;

function TCPBackgroundTask.getDisplayName: string;
begin
  Result := fTaskDesc;
end;

function TCPBackgroundTask.getID: string;
begin
  Result := fTaskID;
end;

function TCPBackgroundTask.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPBackgroundTask.getIsScheduled: boolean;
begin
  try
    Result := (XMLValueAsStr(fXMLRecord, 'SCHEDULED') <> '');
  except
    Result := False;
  end;
end;

function TCPBackgroundTask.getItemType: TCPItemType;
begin
  Result := ciBackgroundTask;
end;

function TCPBackgroundTask.getName: string;
begin
  Result := fTaskDesc;
end;

function TCPBackgroundTask.getScheduledAt: TDateTime;
begin
  try
    if Assigned(fXMLRecord) then
      Result := XMLValueAsDateTime(fXMLRecord, 'SCHEDULED')
    else
      Result := Now;
  except
    Result := Now;
  end;
end;

function TCPBackgroundTask.getSystemID: string;
begin
  Result := fSystemID;
end;

function TCPBackgroundTask.getTaskID: string;
begin
  Result := fTaskID;
end;

function TCPBackgroundTask.getTaskName: string;
begin
  Result := XMLValueAsStr(fXMLRecord, 'TASK_NAME');
end;

function TCPBackgroundTask.getTaskRecurrence: integer;
begin
  try
    Result := XMLValueAsInt(fXMLRecord, 'RECURRENCE', 0);
  except
    Result := 0;
  end;
end;

function TCPBackgroundTask.getTaskStatus: string;
begin
  try
    Result := XMLValueAsStr(fXMLRecord, 'MESSAGE');
  except
    Result := 'Error getting status';
  end;
end;

procedure TCPBackgroundTask.Load;
begin
  try
    fDBConnection.ExecQuery('GetBackgroundTaskStatus', [fTaskID], fXMLDocument);
    fXMLRecord := XMLDocumentFirstRecord(fXMLDocument);
    fTaskID := XMLValueAsStr(fXMLRecord, 'TASK_ID');
    fTaskDesc := XMLValueAsStr(fXMLRecord, 'TASK_NAME');
    fSystemID := XMLValueAsStr(fXMLRecord, 'TASK');
  except
    raise ECPLoadFailure.CreateFmt('Error loading Task Information for ''%s''', [fTaskID]);
  end;
end;

procedure TCPBackgroundTask.setActive(const aValue: boolean);
begin
  // Tasks are always active ???
end;

procedure TCPBackgroundTask.setComment(const aValue: string);
begin
  //
end;

procedure TCPBackgroundTask.setDisplayName(const aValue: string);
begin
  //
end;

procedure TCPBackgroundTask.setName(const aValue: string);
begin
  //
end;

procedure TCPBackgroundTask.Start(aStartDateTime: TDateTime);
begin
  fDBConnection.ExecCommand('StartBackgroundTask', [fTaskID, aStartDateTime]);
end;

procedure TCPBackgroundTask.Stop;
begin
  fDBConnection.ExecCommand('StopBackgroundTask', [fTaskID]);
end;

end.

