unit oCPCareAction;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 5/06/08 1:28p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Primary Care Action Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPCareAction.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  TCPCareAction = class(TInterfacedObject, ICPKardexAction) //, ICPConsoleItem, ICPFlowsheet)
  private
    fCPKardex: ICPKardex;
    fTerminology: ITerminology;
    fID: string;
    fTask: ITerm;
    fSchedule: ITerm;
    fPatient: IPatient;
    fStartDateTime: TDateTime;
    fStopDateTime: TDateTime;
    fStatus: TCPKardexActionStatus;
    fSpecialInstructions: string;
    fEvents: IInterfaceList;
    procedure Load;
    function LoadFromXML(aXMLNode: IXMLNode): boolean;
  public
    constructor Create(aXMLNode: IXMLNode; aPatient: IPatient; aCPKardex: ICPKardex; aTerminology: ITerminology);
    destructor Destroy; override;
  end;

implementation

uses
  oCP_Exceptions,
  uCPXML;

{------------------------------------------------------------------------------}
{------------------------------- TCPCareAction --------------------------------}
{------------------------------------------------------------------------------}

constructor TCPCareAction.Create(aXMLNode: IXMLNode; aPatient: IPatient; aCPKardex: ICPKardex; aTerminology: ITerminology);
begin
  inherited Create;
  fCPKardex := aCPKardex;
  fTerminology := aTerminology;
  fPatient := aPatient;

  TInterfaceList.Create.GetInterface(IInterfaceList, fEvents);

  if Assigned(aXMLNode) then
    LoadFromXML(aXMLNode)
  else
    begin
      fID := NewGUID;
      fTask := nil;
      fSchedule := nil;
      fStartDateTime := Now;
      fStopDateTime := Now + 14;
      fStatus := asActive;
      fSpecialInstructions := '';
    end;
end;

destructor TCPCareAction.Destroy;
begin
  fEvents.Clear;
  inherited;
end;

procedure TCPCareAction.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aPage: ICPFlowsheetPage;
begin
  {
  try
    fIsLoaded := False;
    fPages.Clear;

    aXMLDoc := NewXMLDocument;
    if not fDBConnection.ExecQuery('GetFlowsheet', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetFlowsheet'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    if Assigned(aXMLRec) then
      LoadFromXML(aXMLRec)
    else
      raise ECPLoadFailure.CreateFmt('No record returned for flowsheet ''%s''', [fID]);

    if not fDBConnection.ExecQuery('GetFlowsheetPageList', [fID], aXMLDoc) then
      raise ECPLoadFailure.CreateFmt('Query ''GetFlowsheetPageList'' failed for ID ''%s''', [fID]);

    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while Assigned(aXMLRec) do
      try
        aPage := NewCPFlowsheetPage(aXMLRec);
        fPages.Add(aPage);
      finally
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;

    fIsLoaded := True;
  except
    on E: ECPException do
      raise;
    on E: Exception do
      raise ECPException.CreateFmt('Unknown CP Expection ''%s''', [e.Message]);
  end;
  }
end;

function TCPCareAction.LoadFromXML(aXMLNode: IXMLNode): Boolean;
begin
  Result := True; {Assume the best here}
  with aXMLNode do
    try
      fID := XMLValueAsStr(aXMLNode, 'ID');
      fTask := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'TASK_ID'));
      fSchedule := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'SCHEDULE_ID'));
      fStartDateTime := XMLValueAsDateTime(aXMLNode, 'START_DATE_TIME');
      fStopDateTime := XMLValueAsDateTime(aXMLNode, 'STOP_DATE_TIME');
      fStatus := TCPKardexActionStatus(XMLValueAsInt(aXMLNode, 'STATUS'));
      fSpecialInstructions := XMLValueAsStr(aXMLNode, 'SPECIAL_INSTRUCTIONS');
    except
      Result := False;
    end;
end;

end.

