unit oCPEmailMessage;

interface

uses
  Classes,
  oInterfaces,
  SysUtils;

type
  TCPEMailMessage = class(TInterfacedObject, ICPEMailMessage)
  private
    fDBConnection: IDBConnection;

    fSubject: string;
    fRecipients: TStringList;
    fText: TStringList;

    procedure SetSubject(const aValue: string); stdcall;
    procedure AddRecipient(const aValue: string); stdcall;
    procedure AddText(const aValue: string); overload; stdcall;
    procedure AddText(const aValue: TStrings); overload; stdcall;
    procedure AddList(const aValue: TStrings); stdcall;

    procedure Send; stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

{ TCPEMailMessage }

constructor TCPEMailMessage.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fRecipients := TStringList.Create;
  fText := TStringList.Create;
end;

destructor TCPEMailMessage.Destroy;
begin
  FreeAndNil(fText);
  FreeAndNil(fRecipients);
  inherited Destroy;
end;

procedure TCPEMailMessage.AddList(const aValue: TStrings);
var
  i: integer;
begin
  for i := 0 to aValue.Count - 1 do
    fText.Add(Format('     %4d. %s', [i + 1, aValue[i]]));
end;

procedure TCPEMailMessage.AddRecipient(const aValue: string);
begin
  fRecipients.Add(aValue);
end;

procedure TCPEMailMessage.AddText(const aValue: string);
begin
  fText.Add(aValue);
end;

procedure TCPEMailMessage.AddText(const aValue: TStrings);
begin
  fText.AddStrings(aValue);
end;

procedure TCPEMailMessage.Send;
var
  aParams: array of Variant;
  i, j: integer;
  zztmp: string;
begin
  SetLength(aParams, fRecipients.Count + fText.Count + 1);
  aParams[0] := 'SUB:' + fSubject;
  j := 1;

  for i := 0 to fRecipients.Count - 1 do
    begin
      aParams[j] := 'TO:' + fRecipients[i];
      inc(j);
    end;

  for i := 0 to fText.Count - 1 do
    begin
      aParams[j] := 'TEXT:' + fText[i];
      inc(j);
    end;

  {Debug phase - remove later}
  zztmp := 'Ready To Fire This E-Mail' + #13;
  for i := Low(aParams) to High(aParams) do
    zztmp := zztmp + #13 + aParams[i];
  //MessageDlg(zztmp, mtInformation, [mbOk], 0);
  {End Debug}

  fDBConnection.ExecCommand('SendEMailMessage', aParams);
end;

procedure TCPEMailMessage.SetSubject(const aValue: string);
begin
  fSubject := aValue;
end;

end.

