unit oCPExport;

interface

uses
  XMLDoc,
  XMLIntf,
  SysUtils,
  Dialogs,
  oInterfaces,
  uCPXML,
  Classes;

type
  TCPExport = class(TObject)
  private
    fDBConnection: IDBConnection;
    fXMLDocument: IXMLDocument;
    fHeader: IXMLNode;
    fTables: IXMLNode;
    fFileKey: string;
    function getTableNode(aTableName: string): IXMLNode;
    function getTableKeyCount(aTableName: string): integer;
    function getTableKeyName(aTableName: string; aIndex: integer): string;

    procedure Clear;
    procedure AddTable(aTableName: string; aQueryName: string; aParams: array of Variant); stdcall;
    procedure AddPrimaryKeys(aTableName: string; aPrimaryKeys: array of Variant); stdcall;
    procedure Save(aFilename: string; aHeaderFilename: string = ''); stdcall;
    procedure Display(aList: TStrings); stdcall;
    procedure SetDescription(aValue: string); stdcall;
    procedure SetFramework(aValue: string); stdcall;
    procedure SetVersion(aValue: integer); stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

{ TCPExport }

constructor TCPExport.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  Clear;
end;

destructor TCPExport.Destroy;
begin
  fDBConnection := nil;
  fXMLDocument := nil;
  inherited;
end;

function TCPExport.getTableKeyCount(aTableName: string): integer;
begin
  Result := getTableNode(aTableName).Attributes['PK_CNT'];
end;

function TCPExport.getTableKeyName(aTableName: string; aIndex: integer): string;
begin
  Result := getTableNode(aTableName).Attributes[Format('PK_%0.2d', [aIndex])];
end;

function TCPExport.getTableNode(aTableName: string): IXMLNode;
begin
  Result := fTables.ChildNodes.First;
  while Assigned(Result) do
    if Result.Attributes['NAME'] = aTableName then
      Exit
    else
      Result := Result.NextSibling;
end;

procedure TCPExport.AddPrimaryKeys(aTableName: string; aPrimaryKeys: array of Variant);
var
  i: integer;
begin
  getTableNode(aTableName).Attributes['PK_CNT'] := Length(aPrimaryKeys);

  for i := Low(aPrimaryKeys) to High(aPrimaryKeys) do
    getTableNode(aTableName).Attributes[Format('PK_%0.2d', [i + 1])] := aPrimaryKeys[i];
end;

procedure TCPExport.AddTable(aTableName: string; aQueryName: string; aParams: array of Variant);
var
  i: integer;
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aTableRoot: IXMLNode;
begin
  if getTableNode(aTableName) <> nil then
    raise Exception.Create('Export Table Already Exists');

  aTableRoot := fTables.AddChild('TABLE');
  aTableRoot.Attributes['NAME'] := aTableName;

  aXMLDoc := NewXMLDocument;
  fDBConnection.ExecQuery(aQueryName, aParams, aXMLDoc);
  aXMLDoc.Active := True;

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aTableRoot.ChildNodes.Add(aXMLRec.CloneNode(True));
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;
end;

procedure TCPExport.Clear;
begin
  fXMLDocument := NewXMLDocument;
  fXMLDocument.DocumentElement := fXMLDocument.AddChild('EXPORT');
  fHeader := fXMLDocument.DocumentElement.AddChild('HEADER');
  fHeader.ChildValues['DATE_CREATED'] := Now;
  fTables := fXMLDocument.DocumentElement.AddChild('TABLES');
end;

procedure TCPExport.Display(aList: TStrings);
begin
  aList.Text := FormatXMLData(fXMLDocument.XML.Text);
end;

procedure TCPExport.Save(aFilename: string; aHeaderFilename: string = '');
begin
  fXMLDocument.SaveToFile(aFilename);
  if aHeaderFilename <> '' then
    with NewXMLDocument do
      begin
        Active := True;
        DocumentElement := fHeader.CloneNode(True);
        DocumentElement.ChildValues['SOURCE'] := ExtractFileName(aFilename);
        SaveToFile(aHeaderFilename);
      end;
end;

procedure TCPExport.SetDescription(aValue: string);
begin
  fHeader.ChildValues['DESCRIPTION'] := aValue;
end;

procedure TCPExport.SetFramework(aValue: string);
begin
  fHeader.ChildValues['FRAMEWORK'] := aValue;
end;

procedure TCPExport.SetVersion(aValue: integer);
begin
  fHeader.ChildValues['VERSION'] := aValue;
end;

end.

