unit oCPFlowsheetView;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 15 $  $Modtime: 10/19/11 2:28p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Defines the logic for getting and displaying collections
*                     of observations
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oCPFlowsheetView.pas $
================================================================================
}
interface

uses
  Windows,
  Classes,
  ComCtrls,
  SysUtils,
  Graphics,
  oCliO,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf,
  XMLDoc;

type
  TCPFlowsheetView = class(TInterfacedObject, ICPConsoleItem, ICPConsoleCopy, ICPFlowsheetView, ICliOReportItem)
  protected
    fIsLoaded: boolean;
    [weak] fCPService: ICPService;
    fTerminology: ITerminology;
    fViewTerms: IInterfaceList;
    fID: string;
    fSupplementalID: string;
    fActive: boolean;
    fComment: string;
    fName: string;
    fDisplayName: string;
    fInterval: integer;
    fViewStyle: TObservationViewStyle;
    fAllowPivot: boolean;
    fNationalTitle: boolean;

    fIncludeTotals: boolean;
    fIncludeCounts: boolean;
    fIncludeAverages: boolean;

    {Supplemental Page Items}
    fDefaultSetID: string;
    fDefaultMethod: ITerm;
    fDefaultPosition: ITerm;
    fDefaultLocation: ITerm;
    fDefaultProduct: ITerm;
    fPageType: TCPFlowsheetPageType;
    fActivatedDateTime: TDateTime;
    fActivatedByID: string;
    fActivatedByName: string;
    fActivatedComment: string;
    fDeactivatedDateTime: TDateTime;
    fDeactivatedByID: string;
    fDeactivatedByName: string;
    fDeactivatedComment: string;

    {ICPConsoleItem interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setService(const aCPService: ICPService); stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    procedure CopyFrom(aID: string); stdcall;
    procedure Load; stdcall;
    procedure ForcedLoad; stdcall;
    procedure Save; stdcall;

    {ICliOReportItem}
    procedure AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean = False; aList: ICliOList = nil); stdcall;

    {ICPFlowsheetView interface}
    function getDefaultMethodTerm: string; stdcall;
    function getDefaultPositionTerm: string; stdcall;
    function getDefaultLocationTerm: string; stdcall;
    function getDefaultProductTerm: string; stdcall;

    function getDisplayInterval: integer; stdcall;
    function getViewStyle: TObservationViewStyle; stdcall;
    function getAllowPivot: boolean; stdcall;
    function getIncludeTotals: boolean; stdcall;
    function getIncludeCounts: boolean; stdcall;
    function getIncludeAverages: boolean; stdcall;
    function getIsNationalTitle: boolean; stdcall;

    {Supplemental Page Stuff}
    function getDefaultSetID: string; stdcall;
    function getDefaultMethod: ITerm; stdcall;
    function getDefaultPosition: ITerm; stdcall;
    function getDefaultLocation: ITerm; stdcall;
    function getDefaultProduct: ITerm; stdcall;
    function getPageType: TCPFlowsheetPageType; stdcall;
    function getActivatedDateTime: TDateTime; stdcall;
    function getActivatedByID: string; stdcall;
    function getActivatedByName: string; stdcall;
    function getActivatedComment: string; stdcall;
    function getDeactivatedDateTime: TDateTime; stdcall;
    function getDeactivatedByID: string; stdcall;
    function getDeactivatedByName: string; stdcall;
    function getDeactivatedComment: string; stdcall;
    function getSupplementalID: string; stdcall;
    function getSupplementalTitle: string; stdcall;

    function getIndex(aList: TStrings): integer; stdcall;

    function getViewTermByIndex(aIndex: integer): ICPFlowsheetViewTerm; stdcall;
    function getViewTermByOrder(aTermOrder: integer): ICPFlowsheetViewTerm; stdcall;
    function getViewTermCount: integer; stdcall;
    function getViewTermIndex(aItem: IInterface): integer; stdcall;

    procedure setDisplayInterval(const aValue: integer); stdcall;
    procedure setViewStyle(const aValue: TObservationViewStyle); stdcall;
    procedure setAllowPivot(const aValue: boolean); stdcall;
    procedure setIncludeTotals(const aValue: boolean); stdcall;
    procedure setIncludeCounts(const aValue: boolean); stdcall;
    procedure setIncludeAverages(const aValue: boolean); stdcall;

    procedure InsertViewTerm(aViewTerm: ICPFlowsheetViewTerm); stdcall;
    procedure DeleteViewTerm(aViewTerm: ICPFlowsheetViewTerm); stdcall;

    function LoadFromXML(aXMLNode: IXMLNode): boolean;
    procedure zzDumpToFile(aFilename: string); stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCPFlowsheetView(aXMLNode: IXMLNode; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheetView;

implementation

uses
  Dialogs,
  uCPXML,
  oCP_Exceptions,
  oCPFlowsheetViewTerm,
  ActiveX;

function NewCPFlowsheetView(aXMLNode: IXMLNode; aCPService: ICPService; aTerminology: ITerminology): TCPFlowsheetView;
begin
  Result := TCPFlowsheetView.Create;
  with Result do
    try
      fTerminology := aTerminology;
      fCPService := aCPService;
      if Assigned(aXMLNode) then
        LoadFromXML(aXMLNode)
      else
        fCPService.ExecuteDBCommand('AddFlowsheetView', [fID, fName, fDisplayName, fInterval, fViewStyle, fComment, 0]);
    except
      FreeAndNil(Result);
    end;
end;

procedure TCPFlowsheetView.CopyFrom(aID: string);
var
  aOriginalID: string;
  aOriginalName: string;
begin
  aOriginalID := fID;
  aOriginalName := fName;

  fID := aID;
  try
    ForcedLoad;
    fID := aOriginalID;
    fName := aOriginalName;
    fNationalTitle := False;
    Save;
  except
    raise ECPCommandFail.Create('Error creating copy');
  end;
end;

constructor TCPFlowsheetView.Create;
begin
  inherited Create;
  fID := NewGUID;
  fActive := False;
  fViewTerms := TInterfaceList.Create;
  fName := 'NEW FLOWSHEET VIEW';
  fDisplayName := 'New Flowsheet View';
  fComment := 'New View';
  fInterval := 1440;
  fViewStyle := vsTerminology;
  fPageType := ptMandatory;
  fNationalTitle := False;
  fIsLoaded := False;
end;

procedure TCPFlowsheetView.DeleteViewTerm(aViewTerm: ICPFlowsheetViewTerm);
begin
  fViewTerms.Remove(aViewTerm);
end;

destructor TCPFlowsheetView.Destroy;
begin
  fViewTerms.Clear;
  fViewTerms := nil;

  fCPService := nil;
  fTerminology := nil;
  fDefaultMethod := nil;
  fDefaultPosition := nil;
  fDefaultLocation := nil;
  fDefaultProduct := nil;
  inherited;
end;

procedure TCPFlowsheetView.ForcedLoad;
begin
  fIsLoaded := False;
  Load;
end;

procedure TCPFlowsheetView.zzDumpToFile(aFilename: string);
var
  i: integer;
  tt: TTermType;
//  s: string;
  aTerm: ITerm;
  aTermName: string;
const
  afmt = '    %-20s %-10s %-25s %-10s %-10s';
  afmt2 = '    %-0.20s %-0.10s %-0.25s %-0.10s %-0.10s';
  aLine = '-------------------------------------------------------------------------------------------------';
  aLine2 = '=================================================================================================';
begin
  with TStringList.Create do
    try
      Add('TObservationView Display');
      Add(Format('%0.84s', [aLine2]));
      Add(Format('ID:...............%s', [fID]));
      Add(Format('Comment:..........%s', [fComment]));
      Add(Format('Name:.............%s', [fName]));
      Add(Format('Display Name:.....%s', [fDisplayName]));
      Add(Format('Interval:.........%d', [fInterval]));
      Add(Format('View Style:.......%s', [OBSERVATION_VIEW_STYLE[fViewStyle]]));
      Add(Format('Allow Pivot:......%s', [CP_YN_TEXT[fAllowPivot]]));
      Add(Format('Term Count:.......%d', [getViewTermCount]));
      Add(' ');

      for i := 0 to getViewTermCount - 1 do
        with getViewTermByIndex(i) do
          begin
            Add(Format('    %-0.80s', [aLine2]));
            Add(Format('    Display Order:    %d (Index: %d)', [getDisplayOrder, i]));
            Add(Format('    Display Name:     %s', [getDisplayName]));
            Add(Format('    Display Width:    %d', [getDisplayWidth]));
            Add(Format('    Diaplay Only:     %s', [CP_YN_TEXT[getDisplayOnly]]));
            Add(Format('    Include Average:  %s', [CP_YN_TEXT[getDisplayAverage]]));
            Add(Format('    Include Count:    %s', [CP_YN_TEXT[getDisplayCount]]));
            Add(Format('    Include Total:    %s', [CP_YN_TEXT[getDisplayTotal]]));
            Add(' ');
            Add(Format(aFmt, ['Attribute', 'ID', 'Term', 'Allowed', 'Mandatory']));
            Add(Format(aFmt2, [aLine, aLine, aline, aLine, aLine]));
            for tt := Low(TTermType) to High(TTermType) do
              begin
                aTerm := fTerminology.GetTerm(getAttributeID(tt));
                if Assigned(aTerm) then
                  aTermName := aTerm.getTerm
                else
                  aTermName := '';
                Add(
                  Format(aFmt,
                  [QUALNAME[tt], getAttributeID(tt), aTermName, CP_YN_TEXT[getAttributeAllowed(tt)], CP_YN_TEXT[getAttributeMandatory(tt)]]));
              end;
          end;
      if fViewTerms.Count > 0 then
        Add(Format('    %-0.80s', [aLine2]));
      //SaveToFile(aFilename);
    finally
      Clear;
      Free;
    end;
end;

{ TObservation_View }

procedure TCPFlowsheetView.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, self);
end;

procedure TCPFlowsheetView.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

procedure TCPFlowsheetView.AddToReport(aReportText: TStrings; aMargin: integer; aIncludeDetail: boolean; aList: ICliOList);
var
  j: integer;
  aTerm: ITerm;
  aViewTerm: ICPFlowsheetViewTerm;
  aTotal: Double;
  aCount: Integer;
  aMin: Double;
  aMax: Double;
  aVal: Double;
  aDefaultUnit: ITerm;
  aObs: IObservation;
  aReportItem: ICliOReportItem;
  aValue: string;
  aFmt: string;
begin
  aMin := 0;
  aMax := 0;

  aReportText.Add('');
  aReportText.Add(StringOfChar('-', aMargin));
  aReportText.Add(StringOfChar(' ', (aMargin - Length(getDisplayName)) div 2) + getDisplayName);
  if getDefaultSetID <> '' then
    begin
      aReportText.Add('Date Opened:.....' + FormatDateTime('mmm d, yyyy hh:mm', getActivatedDateTime));
      if getDeactivatedByID <> '' then
        aReportText.Add('Date Closed:.....' + FormatDateTime('mmm d, yyyy hh:mm', getDeactivatedDateTime))
      else
        aReportText.Add('Date Closed:.....');
      aReportText.Add('  Method:........' + getDefaultMethodTerm);
      aReportText.Add('  Position:......' + getDefaultPositionTerm);
      aReportText.Add('  Location:......' + getDefaultLocationTerm);
      aReportText.Add('  Product:.......' + getDefaultProductTerm);
      if aIncludeDetail then
        begin {We can add the audit log here later}
          aReportText.Add('  Comment:.......' + getActivatedComment);
        end;
    end;
  aReportText.Add(StringOfChar('-', aMargin));

  for j := 0 to getViewTermCount - 1 do
    begin
      aViewTerm := getViewTermByIndex(j);
      aTerm := fTerminology.GetTerm(aViewTerm.getAttributeID(ttObservation));
      aTotal := 0;
      aCount := 0;

      if aViewTerm.getDisplayOnly then
        aReportText.Add(Format('[%s (Totals Only)]', [aViewTerm.getDisplayName]))
      else
        begin
          aReportText.Add(Format('[%s]', [aViewTerm.getDisplayName]));
          if aTerm.getDataType = dtNumeric then
            aReportText.Add(
              Format('  %-20s%-8s%-15s%' + IntToStr(aMargin - 53) + 's',
              ['Observed Date/Time', 'Initial', 'Qualifiers', 'Value']))
          else
            aReportText.Add(
              Format('  %-20s%-8s%-15s%-' + IntToStr(aMargin - 53) + 's',
              ['Observed Date/Time', 'Initial', 'Qualifiers', 'Value']));
        end;

      aDefaultUnit := fTerminology.GetTerm(aViewTerm.getAttributeID(ttUnit));
      aObs := aList.FirstObservation;
      while Assigned(aObs) do
        begin
          {
            blj 23 Sept 2011 - Remedy ticket 525716
            The view wasn't checking to see if the observation was IN the view if
            it was supplemental, thus causing a train wreck when multiple copies
            of the same view were on a report.  So, we just need to do a quick
            check first.
          }
          if getPageType = ptSupplemental then
            if not aObs.IsPartOfSet(getDefaultSetID) then
            begin
              aObs := aList.NextObservation(aObs);
              continue;
            end;
          
          if aObs.getStatus = osVerified then
            if aViewTerm.ContainsObservation(aObs) then
              begin
                if not aViewTerm.getDisplayOnly then
                  if aObs.QueryInterface(ICliOReportItem, aReportItem) = 0 then
                    aReportItem.AddToReport(aReportText, aMargin, aIncludeDetail);

                {This is where I need to return to use the picklist ranking value}
                if aObs.getObsTerm.getDataType in [dtNumeric] then
                  begin
                    aVal := fTerminology.GetConvertedValue(
                      aObs.getNumericValue,
                      aObs.getObsUnit,
                      aDefaultUnit);

                    if aCount = 0 then
                      begin
                        aMin := aVal;
                        aMax := aVal;
                      end
                    else
                      begin
                        if aVal < aMin then
                          aMin := aVal;
                        if aVal > aMax then
                          aMax := aVal;
                      end;

                    aTotal := aTotal + aVal;
                    inc(aCount);
                  end
                else
                  inc(aCount);
              end;
          aObs := aList.NextObservation(aObs);
        end;

      aFmt := '%' + IntToStr(aMargin - 22) + 's  %12s %-7s';

      if aViewTerm.getDisplayTotal or aViewTerm.getDisplayCount or aViewTerm.getDisplayAverage or
        aViewTerm.getDisplayMinValue or aViewTerm.getDisplayMaxValue then
        aReportText.Add(Format(aFmt, ['', StringOfChar('-', 12), '']));

      if aViewTerm.getDisplayTotal then
        begin
          aValue := Format('%12.2f', [aTotal]);
          if Assigned(aDefaultUnit) then
            aReportText.Add(Format(aFmt, ['Total:', aValue, '(' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format(aFmt, ['Total:', aValue, '']))
        end;

      if aViewTerm.getDisplayCount then
        aReportText.Add(Format(aFmt, ['Count:', IntToStr(aCount), '']));

      if aViewTerm.getDisplayAverage then
        begin
          if aCount > 0 then
            aValue := Format('%12.2f', [aTotal / aCount])
          else
            aValue := '***';

          if Assigned(aDefaultUnit) then
            aReportText.Add(Format(aFmt, ['Average:', aValue, '(' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format(aFmt, ['Average:', aValue, '']));
        end;

      if aViewTerm.getDisplayMinValue then
        begin
          if aCount > 0 then
            aValue := Format('%12.2f', [aMin])
          else
            aValue := '***';

          if Assigned(aDefaultUnit) then
            aReportText.Add(Format(aFmt, ['Minimum:', aValue, '(' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format(aFmt, ['Minimum:', aValue, '']));
        end;

      if aViewTerm.getDisplayMaxValue then
        begin
          if aCount > 0 then
            aValue := Format('%12.2f', [aMax])
          else
            aValue := '***';

          if Assigned(aDefaultUnit) then
            aReportText.Add(Format(aFmt, ['Maximum:', aValue, '(' + aDefaultUnit.getAbbreviation + ')']))
          else
            aReportText.Add(Format(aFmt, ['Maximum:', aValue, '']));
        end;

      aReportText.Add('');
    end;
end;

function TCPFlowsheetView.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

function TCPFlowsheetView.getViewTermByIndex(aIndex: integer): ICPFlowsheetViewTerm;
begin
  try
    fViewTerms[aIndex].QueryInterface(ICPFlowsheetViewTerm, Result);
  except
    Result := nil;
  end;
end;

function TCPFlowsheetView.getViewTermByOrder(aTermOrder: integer): ICPFlowsheetViewTerm;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to fViewTerms.Count - 1 do
    if fViewTerms[i].QueryInterface(ICPFlowsheetViewTerm, Result) = 0 then
      if Result.getDisplayOrder = aTermOrder then
        Exit;
  Result := nil;
end;

function TCPFlowsheetView.getViewTermCount: integer;
begin
  Result := fViewTerms.Count;
end;

procedure TCPFlowsheetView.Load;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aViewTerm: ICPFlowsheetViewTerm;
begin
  if fIsLoaded and not (FindCmdLineSwitch('maintenance')) then
    Exit;

  fViewTerms.Clear;

  aXMLDoc := NewXMLDocument;

  if fSupplementalID <> '' then
    begin
      if not fCPService.ExecuteDBQuery('GetFlowsheetSupplementalPage', [fSupplementalID], aXMLDoc) then
        raise ECPLoadFailure.CreateFmt('Error loading supplemental page ''%s'' defintion.', [fID]);
    end
  else
    begin
      if not fCPService.ExecuteDBQuery('GetFlowsheetView', [fID], aXMLDoc) then
        raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' defintion.', [fID]);
    end;

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  if Assigned(aXMLRec) then
    LoadFromXML(aXMLRec)
  else
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' defintion.', [fID]);

  {Load the terms}
  if not fCPService.ExecuteDBQuery('GetFlowsheetViewTerms', [fID], aXMLDoc) then
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' terms.', [fID]);

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aViewTerm := NewCPFlowsheetViewTerm(aXMLRec, fDefaultSetID);
      fViewTerms.Add(aViewTerm);
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    except
      raise ECPLoadFailure.Create('Error loading view term');
    end;

  if not fCPService.ExecuteDBQuery('GetFlowsheetViewFilters', [fID], aXMLDoc) then
    raise ECPLoadFailure.CreateFmt('Error loading view ''%s'' filters.', [fID]);

  aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLRec) do
    try
      aViewTerm := getViewTermByOrder(XMLValueAsInt(aXMLRec, 'TERM_ORDER'));
      aViewTerm.setAttribute(
        TTermType(XMLValueAsInt(aXMLRec, 'FILTER_TYPE')),
        True,
        XMLValueAsInt(aXMLRec, 'FILTER_USAGE', 0) = 1,
        XMLValueAsStr(aXMLRec, 'FILTER_TERM'));
    finally
      aXMLRec := XMLDocumentNextRecord(aXMLRec);
    end;

  fIsLoaded := True;
end;

function TCPFlowsheetView.LoadFromXML(aXMLNode: IXMLNode): boolean;
begin
  try
    {Common items for views and supp pages}
    fID := XMLValueAsStr(aXMLNode, 'VIEW_ID');
    fSupplementalID := XMLValueAsStr(aXMLNode, 'ID');
    fDefaultSetID := XMLValueAsStr(aXMLNode, 'SET_ID');

    fName := XMLValueAsStr(aXMLNode, 'NAME');
    fInterval := XMLValueAsInt(aXMLNode, 'TIME_INTERVAL');
    fViewStyle := TObservationViewStyle(XMLValueAsInt(aXMLNode, 'X_AXIS', 0));
    fAllowPivot := (XMLValueAsInt(aXMLNode, 'ALLOW_PIVOT') = 1);
    fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE') = 1);
    fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
    fDisplayName := XMLValueAsStr(aXMLNode, 'DISPLAY_NAME');
    fNationalTitle := (XMLValueAsInt(aXMLNode, 'NATIONAL_TITLE') = 1);

    if fSupplementalID <> '' then
      begin
        fDefaultMethod := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_METHOD_ID'));
        fDefaultPosition := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_POSTITION_ID'));
        fDefaultLocation := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_LOCATION_ID'));
        fDefaultProduct := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'DEFAULT_PRODUCT_ID'));
        fPageType := TCPFlowsheetPageType(XMLValueAsInt(aXMLNode, 'ACTIVATED_AS_TYPE', 0));
        fActivatedDateTime := XMLValueAsDateTime(aXMLNode, 'ACTIVATED_DATE_TIME');
        fActivatedByID := XMLValueAsStr(aXMLNode, 'ACTIVATED_BY_ID');
        fActivatedByName := XMLValueAsStr(aXMLNode, 'ACTIVATED_BY_NAME');
        fActivatedComment := XMLValueAsStr(aXMLNode, 'ACTIVATED_COMMENT');
        fDeactivatedDateTime := XMLValueAsDateTime(aXMLNode, 'DEACTIVATED_DATE_TIME');
        fDeactivatedByID := XMLValueAsStr(aXMLNode, 'DEACTIVATED_BY_ID');
        fDeactivatedByName := XMLValueAsStr(aXMLNode, 'DEACTIVATED_BY_NAME');
        fDeactivatedComment := XMLValueAsStr(aXMLNode, 'DEACTIVATED_COMMENT');
      end;

    Result := True;
  except
    Result := False;
  end;
end;

procedure TCPFlowsheetView.Save;
var
  i: integer;
  aViewTerm: ICPFlowsheetViewTerm;
  tt: TTermType;
begin
  if not fCPService.ExecuteDBCommand('UpdateFlowsheetView',
    [fID, fDisplayName, fComment, fInterval, fViewStyle, fAllowPivot, fActive, fNationalTitle]) then
    raise ECPSaveFailure.CreateFmt('Error updating view ID ''%s''', [fID]);

  if not fCPService.ExecuteDBCommand('DeleteFlowsheetViewTerms', [fID]) then
    raise ECPSaveFailure.CreateFmt('Error deleting terms for view ID ''%s''', [fID]);

  if not fCPService.ExecuteDBCommand('DeleteFlowsheetViewFilters', [fID]) then
    raise ECPSaveFailure.CreateFmt('Error deleting filters for view ID ''%s''', [fID]);

  for i := 0 to fViewTerms.Count - 1 do
    begin
      aViewTerm := getViewTermByIndex(i);
      with aViewTerm do
        begin
          if not fCPService.ExecuteDBCommand('AddFlowsheetViewTerm',
            [fID, getDisplayOrder, getAttributeID(ttObservation), getDisplayName, getDisplayWidth,
            getDisplayTotal, getDisplayCount, getDisplayAverage, getDisplayOnly, getDisplayColumns,
              getDisplayInCellTotal, getDisplayMinValue, getDisplayMaxValue, getRequiredTerm, getDisplayInDropdown]) then
            raise ECPSaveFailure.CreateFmt('Error adding term %s to view.', [aViewTerm.getDisplayName]);

          for tt := Low(TTermType) to High(TTermType) do
            if aViewTerm.getAttributeAllowed(tt) then
              if not fCPService.ExecuteDBCommand('AddFlowsheetViewFilter',
                [fID, getDisplayOrder, tt, getAttributeID(tt),
                CP_YN[getAttributeMandatory(tt)]]) then
                raise ECPSaveFailure.CreateFmt('Error adding Filter %s to view.', [TERMTYPE[tt]]);
        end;
    end;
end;

function TCPFlowsheetView.getViewTermIndex(aItem: IInterface): integer;
var
  aObservation: IObservation;
  aViewTerm: ICPFlowsheetViewTerm;
begin
  if aItem.QueryInterface(IObservation, aObservation) = 0 then {It is a valid observation}
    {Check the observation against the view primary and secondary items}
    //if aObservation.getHasAttribute(fPrimaryIDMatch) and aObservation.getHasAttribute(fSecondaryIDMatch) then
    for Result := 0 to fViewTerms.Count - 1 do {Walk down the IViewTerms}
      begin
        if fViewTerms[Result].QueryInterface(ICPFlowsheetViewTerm, aViewTerm) = 0 then
          if aViewTerm.ContainsObservation(aObservation) then
            Exit; {if you are here, everything matches up, bail with the index}
      end;
  Result := -1; {Something went south, no index to return}
end;

{
function TObservationView.getViewType: TObservationViewType;
begin
  Result := fViewType;
end;
}

procedure TCPFlowsheetView.InsertViewTerm(aViewTerm: ICPFlowsheetViewTerm);
begin
  fViewTerms.Add(aViewTerm);
end;

{ Getters and Setters }

function TCPFlowsheetView.getActivatedByID: string;
begin
  Result := fActivatedByID;
end;

function TCPFlowsheetView.getActivatedByName: string;
begin
  Result := fActivatedByName;
end;

function TCPFlowsheetView.getActivatedComment: string;
begin
  Result := fActivatedComment;
end;

function TCPFlowsheetView.getActivatedDateTime: TDateTime;
begin
  Result := fActivatedDateTime;
end;

function TCPFlowsheetView.getActive: Boolean;
begin
  Result := (fActive or fNationalTitle);
end;

function TCPFlowsheetView.getCanDelete: boolean;
begin
  Result := False;
end;

function TCPFlowsheetView.getCanRename: boolean;
begin
  Result := (not fNationalTitle);
end;

function TCPFlowsheetView.getComment: string;
begin
  Result := fComment;
end;

function TCPFlowsheetView.getName: string;
begin
  Result := fName
end;

function TCPFlowsheetView.getIsLoaded: boolean;
begin
  Result := False;
end;

function TCPFlowsheetView.getIsNationalTitle: boolean;
begin
  Result := fNationalTitle;
end;

function TCPFlowsheetView.getDisplayName: string;
begin
  if fDisplayName = '' then
    Result := fName
  else
    Result := fDisplayName;
end;

function TCPFlowsheetView.getID: string;
begin
  Result := fID;
end;

function TCPFlowsheetView.getItemType: TCPItemType;
begin
  Result := ciFlowsheetView;
  {
  case fViewType of
    vtGrid: Result := ciView;
    vtReport: Result := ciReportView;
  else
    Result := ciView;
  end;
  }
end;

function TCPFlowsheetView.getDeactivatedByID: string;
begin
  Result := fDeactivatedByID;
end;

function TCPFlowsheetView.getDeactivatedByName: string;
begin
  Result := fDeactivatedByName;
end;

function TCPFlowsheetView.getDeactivatedComment: string;
begin
  Result := fDeactivatedComment;
end;

function TCPFlowsheetView.getDeactivatedDateTime: TDateTime;
begin
  Result := fDeactivatedDateTime;
end;

function TCPFlowsheetView.getDefaultLocation: ITerm;
begin
  Result := fDefaultLocation;
end;

function TCPFlowsheetView.getDefaultLocationTerm: string;
begin
  if Assigned(fDefaultLocation) then
    Result := fDefaultLocation.getTerm
  else
    Result := '<None>';
end;

function TCPFlowsheetView.getDefaultMethod: ITerm;
begin
  Result := fDefaultMethod;
end;

function TCPFlowsheetView.getDefaultMethodTerm: string;
begin
  if Assigned(fDefaultMethod) then
    Result := fDefaultMethod.getTerm
  else
    Result := '<None>';
end;

function TCPFlowsheetView.getDefaultPosition: ITerm;
begin
  Result := fDefaultPosition;
end;

function TCPFlowsheetView.getDefaultPositionTerm: string;
begin
  if Assigned(fDefaultPosition) then
    Result := fDefaultPosition.getTerm
  else
    Result := '<None>';
end;

function TCPFlowsheetView.getDefaultProduct: ITerm;
begin
  Result := fDefaultProduct;
end;

function TCPFlowsheetView.getDefaultProductTerm: string;
begin
  if Assigned(fDefaultProduct) then
    Result := fDefaultProduct.getTerm
  else
    Result := '<None>';
end;

function TCPFlowsheetView.getDefaultSetID: string;
begin
  Result := fDefaultSetID;
end;

function TCPFlowsheetView.getDisplayInterval: integer;
begin
  Result := fInterval;
end;

function TCPFlowsheetView.getAllowPivot: boolean;
begin
  Result := fAllowPivot;
end;

function TCPFlowsheetView.getViewStyle: TObservationViewStyle;
begin
  Result := fViewStyle;
end;

function TCPFlowsheetView.getIncludeAverages: boolean;
begin
  Result := fIncludeAverages;
end;

function TCPFlowsheetView.getIncludeCounts: boolean;
begin
  Result := fIncludeCounts;
end;

function TCPFlowsheetView.getIncludeTotals: boolean;
begin
  Result := fIncludeTotals;
end;

function TCPFlowsheetView.getIndex(aList: TStrings): integer;
begin
  Result := aList.IndexOfObject(Self);
end;

function TCPFlowsheetView.getPageType: TCPFlowsheetPageType;
begin
  Result := fPageType;
end;

function TCPFlowsheetView.getSupplementalID: string;
begin
  Result := fSupplementalID;
end;

function TCPFlowsheetView.getSupplementalTitle: string;
begin
  if (fDeactivatedDateTime > 0) and (fDeactivatedDateTime < Now) then
    Result := Format('%s (Closed %s, Comment: %s)',
      [getDisplayName,
      FormatDateTime('MM/DD/YYYY hh:mm', fDeactivatedDateTime),
        fDeactivatedComment])
  else
    Result := Format(
      '%s (Added %s as %s) Comment: %s',
      [getDisplayName,
      FormatDateTime('MM/DD/YYYY hh:mm', fActivatedDateTime),
        CP_FLOWSHEET_PAGE_TYPE[fPageType],
        fActivatedComment]);
end;

procedure TCPFlowsheetView.setIncludeAverages(const aValue: boolean);
begin
  fIncludeAverages := aValue;
end;

procedure TCPFlowsheetView.setIncludeCounts(const aValue: boolean);
begin
  fIncludeCounts := aValue;
end;

procedure TCPFlowsheetView.setIncludeTotals(const aValue: boolean);
begin
  fIncludeTotals := aValue;
end;

procedure TCPFlowsheetView.setName(const aValue: string);
begin
  fName := aValue;
end;

procedure TCPFlowsheetView.setService(const aCPService: ICPService);
begin
  fCPService := aCPService;
end;

procedure TCPFlowsheetView.setViewStyle(const aValue: TObservationViewStyle);
begin
  fViewStyle := aValue;
end;

procedure TCPFlowsheetView.setAllowPivot(const aValue: boolean);
begin
  fAllowPivot := aValue;
end;

procedure TCPFlowsheetView.setComment(const aValue: string);
begin
  fComment := aValue;
end;

procedure TCPFlowsheetView.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TCPFlowsheetView.setDisplayInterval(const aValue: integer);
begin
  fInterval := aValue;
end;

procedure TCPFlowsheetView.setDisplayName(const aValue: string);
begin
  fDisplayName := aValue;
end;

end.

