unit oCPGatewayConfig;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 10 $  $Modtime: 5/22/09 1:11p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Default object to manage the Gateway configuration in the
*                     Windows registry.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPGatewayConfig.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oInterfaces,
  oCPTypes;

type
  TCPGatewayConfig = class(TInterfacedObject, ICPConsoleItem, ICPGatewayConfig)
  protected
    fDBConnection: IDBConnection;
    fAccessCode: string;
    fVerifyCode: string;
    fDaysToKeepData: Integer;
    fName: string;
    fID: string;
    fLogFileDirectory: string;

    fBrokerServerIP: string;
    fBrokerServerPort: integer;
    fDataRetrievalType: TDataRetrievalType;
    fPollingInterval: Integer; // In tenths of a minute.

    fLogLevel: Integer;

    fGatewayServerIP: string;
    fGatewayServerPort: integer;
    fGatewayServerHint: string;
    fSharedDirectory: string;

    {ICPConsole Interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: Boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    function Activate(const aDBConnection: IDBConnection): Boolean;
    function Deactivate(const aDBConnection: IDBConnection): Boolean;
    function LoadActions(aListView: TListView): Integer;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;

    function getReference: TObject; stdcall;

    function getBrokerServerIP: string; stdcall;
    function getBrokerServerPort: integer; stdcall;
    function getAccessCode: string; stdcall;
    function getVerifyCode: string; stdcall;
    function getIsGatewayInstalled: boolean; stdcall;

    function getGatewayServerIP: string; stdcall;
    function getGatewayServerPort: integer; stdcall;
    function getGatewayServerHint: string; stdcall;
    function getDaysToKeepData: integer; stdcall;
    function getLogFileDirectory: string; stdcall;
    function getLogLevel: Integer; stdcall;
    function getSharedDirectory: string; stdcall;
    function getEncryptedAccess: string; stdcall;
    function getEncryptedVerify: string; stdcall;
    function getDataRetrievalType: TDataRetrievalType; stdcall;
    function getPollingInterval: integer; stdcall;
    procedure Save; stdcall;
    procedure Load; stdcall;

    procedure setAccessCode(const aValue: string); stdcall;
    procedure setVerifyCode(const aValue: string); stdcall;
    procedure setBrokerServerIP(const aValue: string); stdcall;
    procedure setBrokerServerPort(const aValue: integer); stdcall;

    procedure setDaysToKeepData(const aValue: integer); stdcall;
    procedure setLogFileDirectory(const aValue: string); stdcall;
    procedure setGatewayServerIP(const aValue: string); stdcall;
    procedure setGatewayServerPort(const AValue: integer); stdcall;
    procedure setGatewayServerHint(const aValue: string); stdcall;
    procedure setLogLevel(const AValue: Integer); stdcall;
    procedure setSharedDirectory(const aValue: string); stdcall;
    procedure setEncryptedAccess(const AValue: string); stdcall;
    procedure setEncryptedVerify(const AValue: string); stdcall;
    procedure setDataRetrievalType(const AValue: TDataRetrievalType); stdcall;
    procedure setPollingInterval(const AValue: integer); stdcall;

    function getDBConnection: IDBConnection; stdcall;

    property EncryptedAccess: string read fAccessCode write fAccessCode;
    property EncryptedVerify: string read fVerifyCode write fVerifyCode;
  end;

implementation

uses
  uServiceFunc,
  SysUtils,
  WinSvc,
  Windows,
  Registry,
  XWBHash, //2FA
  oCPParameterFile;

const
  REG_KEY_ROOT = '\Software\VistA\Clinical Procedures\CPGateway';
  REG_VALUE_ACCESS = 'Access Code';
  REG_VALUE_VERIFY = 'Verify Code';
  REG_VALUE_VISTASERVER = 'Server';
  REG_VALUE_VISTAPORT = 'Port';
  DATA_RETENTION = 'GATEWAY_DATA_RETENTION';
  LOG_FILE_DIR = 'GATEWAY_LOG_FILE_DIR';
  BROKER_PORT = 'GATEWAY_BROKER_PORT';
  LOG_LEVEL = 'GATEWAY_LOG_LEVEL';
  GATEWAY_SERVER_PORT = 'GATEWAY_NOTIFY_PORT';
  GATEWAY_SERVER_IP = 'GATEWAY_SERVER_IP';
  GATEWAY_SERVER_HINT = 'GATEWAY_SERVER_HINT';
  SHARED_DIR = 'GATEWAY_SHARED_DIR';
  DATA_RETRIEVAL_TYPE = 'DATA_RETRIEVAL_TYPE';
  POLLING_INTERVAL = 'POLLING_INTERVAL';
  CP_GATEWAY = 'MD PARAMETERS';

  { TCPGatewayConfig }

procedure TCPGatewayConfig.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPGatewayConfig.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

function TCPGatewayConfig.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

constructor TCPGatewayConfig.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fID := 'CP Gateway Configuration';
  fName := 'CP Gateway Configuration';
end;

destructor TCPGatewayConfig.Destroy;
begin
  fDBConnection := nil;
  inherited;
end;

function TCPGatewayConfig.Activate(const aDBConnection: IDBConnection): Boolean;
begin
  result := GatewayStart;
end;

function TCPGatewayConfig.Deactivate(const aDBConnection: IDBConnection): Boolean;
begin
  result :=GatewayStop;
end;

function TCPGatewayConfig.getAccessCode: string;
begin
  if Length(fAccessCode) > 0 then
    Result := Decrypt(fAccessCode)
  else
    Result := '';
end;

function TCPGatewayConfig.getActive: Boolean;
begin
  Result := true;
end;

function TCPGatewayConfig.getCanDelete: boolean;
begin
  Result := True;
end;

function TCPGatewayConfig.getCanRename: boolean;
begin
  Result := False;
end;

function TCPGatewayConfig.getComment: string;
begin
  Result := Format('''%s'' ', [getDisplayName]) +
    'holds the configuration settings for the CP Gateway Service. If changes are made ' +
    'the service must be shutdown and restarted for the new settings to take effect. ' +
    'Extreme care should be taken when modifiying these settings.';
end;

function TCPGatewayConfig.getDataRetrievalType: TDataRetrievalType;
begin
  result := fDataRetrievalType;
end;

function TCPGatewayConfig.getDaysToKeepData: Integer;
begin
  Result := fDaysToKeepData;
end;

function TCPGatewayConfig.getDBConnection: IDBConnection;
begin
  result := FDBConnection;
end;

function TCPGatewayConfig.getName: string;
begin
  Result := fName;
end;

function TCPGatewayConfig.getDisplayName: string;
begin
  Result := fName;
end;

function TCPGatewayConfig.getEncryptedAccess: string;
begin
  result := fAccessCode;
end;

function TCPGatewayConfig.getEncryptedVerify: string;
begin
  result := fVerifyCode;
end;

function TCPGatewayConfig.getID: string;
begin
  result := '';
end;

function TCPGatewayConfig.getIsGatewayInstalled: boolean;
begin
  Result := GatewayInstalled;
end;

function TCPGatewayConfig.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPGatewayConfig.getItemType: TCPItemType;
begin
  Result := ciParameters;
end;

function TCPGatewayConfig.getLogFileDirectory: string;
begin
  Result := IncludeTrailingPathDelimiter(fLogFileDirectory);
end;

function TCPGatewayConfig.getBrokerServerPort: integer; stdcall;
begin
  Result := fBrokerServerPort;
end;

function TCPGatewayConfig.getBrokerServerIP: string;
begin
  Result := fBrokerServerIP;
end;

function TCPGatewayConfig.getSharedDirectory: string;
begin
  Result := fSharedDirectory;
end;

function TCPGatewayConfig.getVerifyCode: string;
begin
  if Length(fVerifyCode) > 0 then
    Result := Decrypt(fVerifyCode)
  else
    Result := '';
end;

function TCPGatewayConfig.LoadActions(aListView: TListView): Integer;
begin
  //
  result := -1;
end;

procedure TCPGatewayConfig.Save; stdcall;
var
  Params: ICPParameterFile;
  Reg: TRegistry;
begin
  //We need to save the Access and Verify Code locally.  Otherwise, we won't be
  // able to connect.

  if getIsGatewayInstalled then
    begin
      reg := TRegistry.Create;
      try
        reg.RootKey := HKEY_LOCAL_MACHINE;
        reg.OpenKey(REG_KEY_ROOT, true);

        reg.WriteString(REG_VALUE_ACCESS, fAccessCode);
        reg.WriteString(REG_VALUE_VERIFY, fVerifyCode);
        reg.WriteString(REG_VALUE_VISTASERVER, fBrokerServerIP);
        reg.WriteInteger(REG_VALUE_VISTAPORT, fBrokerServerPort);
      finally
        reg.CloseKey;
        FreeAndNil(reg);
      end;
    end;

  if not assigned(fDBConnection) then
    raise Exception.Create('DB Connection not assigned in TCPGatewayConfig.Save');
  Params := TCPParameterFile.Create(fDBConnection);
  try
    Params.WriteString(pkSystem, GATEWAY_SERVER_IP, fGatewayServerIP);
    Params.WriteInteger(pkSystem, GATEWAY_SERVER_PORT, fGatewayServerPort);
    Params.WriteString(pkSystem, GATEWAY_SERVER_HINT, fGatewayServerHint);

    Params.WriteInteger(pkSystem, LOG_LEVEL, fLogLevel);
    Params.WriteString(pkSystem, LOG_FILE_DIR, fLogFileDirectory);

    Params.WriteInteger(pkSystem, DATA_RETENTION, fDaysToKeepData);
    Params.WriteString(pkSystem, SHARED_DIR, fSharedDirectory);
    Params.WriteInteger(pkSystem, DATA_RETRIEVAL_TYPE, ord(fDataRetrievalType));
    Params.WriteInteger(pkSystem, POLLING_INTERVAL, fPollingInterval);
  finally
    Params := nil;
  end;
end;

procedure TCPGatewayConfig.Load; stdcall;
var
  Params: ICPParameterFile;
  reg: TRegistry;
begin
  if not assigned(FDBConnection) then
    raise Exception.Create('DB Connection not assigned in TCPGatewayConfig.Load');
  Params := TCPParameterFile.Create(FDBConnection);
  try
    if getIsGatewayInstalled then
      begin
        reg := TRegistry.Create;
        try
          reg.RootKey := HKEY_LOCAL_MACHINE;
          reg.OpenKey(REG_KEY_ROOT, True);

          fAccessCode := reg.ReadString(REG_VALUE_ACCESS);
          fVerifyCode := reg.ReadString(REG_VALUE_VERIFY);
          fBrokerServerIP := reg.ReadString(REG_VALUE_VISTASERVER);
          try
            {
            The TRegistry object will quite happily return a blank string
              if a string doesn't currently exist in the registry.  However,
              if an integer doesn't exist, it will throw an exception instead
              of returning 0 or some such.

            Nice and consistent.
            }
            fBrokerServerPort := reg.ReadInteger(REG_VALUE_VISTAPORT);
          except
            fBrokerServerPort := 0;
          end;

        finally
          reg.CloseKey;
          FreeAndNil(reg);
        end;
      end;

    fGatewayServerIP := Params.ReadString(pkSystem, GATEWAY_SERVER_IP, '');
    fGatewayServerPort := Params.ReadInteger(pkSystem, GATEWAY_SERVER_PORT, 8888);
    fGatewayServerHint := Params.ReadString(pkSystem, GATEWAY_SERVER_HINT, '');
    fLogLevel := Params.ReadInteger(pkSystem, LOG_LEVEL, 1);
    fLogFileDirectory := Params.ReadString(pkSystem, LOG_FILE_DIR, '');

    fDaysToKeepData := Params.ReadInteger(pkSystem, DATA_RETENTION, 14);
    fSharedDirectory := Params.ReadString(pkSystem, SHARED_DIR, '');
    fDataRetrievalType := TDataRetrievalType(Params.ReadInteger(pkSystem, DATA_RETRIEVAL_TYPE, 0));
    fPollingInterval := Params.ReadInteger(pkSystem, POLLING_INTERVAL, 50);
  finally
    Params := nil;
  end;
end;

procedure TCPGatewayConfig.setAccessCode(const aValue: string);
begin
  if Length(aValue) > 0 then
    fAccessCode := Encrypt(aValue)
  else
    fAccessCode := '';
end;

procedure TCPGatewayConfig.setDataRetrievalType(const AValue: TDataRetrievalType);
begin
  fDataRetrievalType := AValue;
end;

procedure TCPGatewayConfig.setDaysToKeepData(const aValue: integer);
begin
  fDaysToKeepData := aValue;
end;

procedure TCPGatewayConfig.setDisplayName(const aValue: string);
begin
  //
end;

procedure TCPGatewayConfig.setEncryptedAccess(const AValue: string);
begin
  fAccessCode := AValue;
end;

procedure TCPGatewayConfig.setEncryptedVerify(const AValue: string);
begin
  fVerifyCode := AValue;
end;

procedure TCPGatewayConfig.setLogFileDirectory(const aValue: string);
begin
  fLogFileDirectory := aValue;
end;

procedure TCPGatewayConfig.setBrokerServerPort(const aValue: integer);
begin
  fBrokerServerPort := aValue;
end;

procedure TCPGatewayConfig.setComment(const aValue: string);
begin
  //
end;

procedure TCPGatewayConfig.setBrokerServerIP(const aValue: string);
begin
  fBrokerServerIP := aValue;
end;

procedure TCPGatewayConfig.setSharedDirectory(const aValue: string);
begin
  fSharedDirectory := aValue;
end;

procedure TCPGatewayConfig.setVerifyCode(const aValue: string);
begin
  if Length(aValue) > 0 then
    fVerifyCode := Encrypt(aValue)
  else
    fVerifyCode := '';
end;

{getters and setters}

procedure TCPGatewayConfig.setActive(const aValue: boolean); stdcall;
begin
  //
end;

function TCPGatewayConfig.getGatewayServerHint: string;
begin
  Result := fGatewayServerHint;
end;

function TCPGatewayConfig.getGatewayServerIP: string;
begin
  result := fGatewayServerIP;
end;

function TCPGatewayConfig.getGatewayServerPort: Integer; stdcall;
begin
  result := fGatewayServerPort;
end;

function TCPGatewayConfig.getPollingInterval: integer;
begin
  result := fPollingInterval;
end;

function TCPGatewayConfig.getReference: TObject;
begin
  result := self;
end;

procedure TCPGatewayConfig.setName(const aValue: string);
begin
  //
end;

procedure TCPGatewayConfig.setGatewayServerHint(const aValue: string);
begin
  fGatewayServerHint := aValue;
end;

procedure TCPGatewayConfig.setGatewayServerIP(const aValue: string);
begin
  fGatewayServerIP := aValue;
end;

procedure TCPGatewayConfig.setGatewayServerPort(const AValue: Integer); stdcall;
begin
  fGatewayServerPort := AValue;
end;

procedure TCPGatewayConfig.setPollingInterval(const AValue: integer);
begin
  fPollingInterval := AValue;
end;

function TCPGatewayConfig.getLogLevel: Integer; stdcall;
begin
  result := FLogLevel;
end;

procedure TCPGatewayConfig.setLogLevel(const AValue: Integer); stdcall;
begin
  FLogLevel := AValue;
end;

end.

