unit oCPGatewayTest;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 3 $  $Modtime: 5/22/09 1:20p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Used to test Gateway Connectivity
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPGatewayTest.pas $
*
* $History: oCPGatewayTest.pas $
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 5/22/09    Time: 2:17p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Build Pre-275:  
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 11/07/07   Time: 10:27a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:05p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  SysUtils,
  Classes;

type
  ECPGatewayConnectionError = class (Exception)
  end;
  
  TCPGatewayProgressNotification = procedure (aMsg: string) of object;
  TCPGatewayCompletionNotification = procedure (aMsg: string) of object;
  TCPGatewayTest = class (TThread)
  private
    fAccess: string;
    fOnCompletionNotification: TCPGatewayCompletionNotification;
    fOnProgressNotification: TCPGatewayProgressNotification;
    fPort: Integer;
    fReturnMessage: TStringList;
    fServer: string;
    fVerify: string;
    fXFerDirectory: string;
    procedure OnDefaultCompletionNotification(aMsg: string);
    procedure OnDefaultProgressNotification(aMsg: string);
  protected
    procedure Execute; override;
  public
    constructor Create(aServer: string; aPort: integer; aAccess, aVerfiy: string; aXFerDirectory: string);
    function getIsStillRunning: Boolean;
    function getResults: string;
    procedure setOnCompletionNotification(aMethod: TCPGatewayCompletionNotification);
    procedure setOnProgressNotification(aMethod: TCPGatewayProgressNotification);
  end;
  
implementation

uses
  TRPCB,
  Windows;

const
  RPC_NAME = 'MD GATEWAY';
  RPC_CONTEXT = 'MD GUI MANAGER';
  DIR_NAME = 'CP Gateway/Imaging BGP transfer directory';

{ TCPGatewayTest }

{
**************************************************** TCPGatewayTest ****************************************************
}
constructor TCPGatewayTest.Create(aServer: string; aPort: integer; aAccess, aVerfiy: string; aXFerDirectory: string);
begin
  inherited Create(false);
  fOnProgressNotification := OnDefaultProgressNotification;
  fOnCompletionNotification := OnDefaultCompletionNotification;
  fServer := aServer;
  fPort := aPort;
  fAccess := aAccess;
  fVerify := aVerfiy;
  fXFerDirectory := aXFerDirectory;
  fReturnMessage := TStringList.Create;
end;

procedure TCPGatewayTest.Execute;
var
  i: Integer;
  testPath: string;
  testFile: string;
  testData: TStringList;
  
  const
    XFERDIR = '';
  
begin
  with TRPCBroker.Create(nil) do
    try
      fOnProgressNotification('Starting Connectivity Tests');
      testData := TStringList.Create;
      ShowErrorMsgs := semRaise;
      Server := fServer;
      ListenerPort := fPort;
      AccessVerifyCodes := Format('%s;%s', [fAccess, fVerify]);
      try
        fOnProgressNotification('Connecting Broker');
        Connected := True;
        fOnProgressNotification('Creating Context');
        if not CreateContext(RPC_CONTEXT) then
          raise ECPGatewayConnectionError.CreateFmt('Service account has no access to ''%s'' context.', [RPC_CONTEXT]);
        RemoteProcedure := RPC_NAME;
        Param[0].Value := 'TSTCONN';
        Param[0].PType := Literal;
        fOnProgressNotification('Calling Test RPC');
        lstCall(testData);
        {Now lets test the shared directory}
        fOnProgressNotification('Testing access to the ' + DIR_NAME);
        i := 0;
  
        testPath := IncludeTrailingPathDelimiter(fXFerDirectory);
  
        if not DirectoryExists(testPath) then
          raise Exception.CreateFmt('Unable to access %s: ''%s''', [DIR_NAME, testPath]);
  
        testFile := Format('%stest.000', [TestPath, i]);
  
        try
          while FileExists(testFile) do
            begin
              inc(i);
              testFile := Format('%stest.%0.3d', [TestPath, i]);
            end;
        except
          raise Exception.CreateFmt('Unable to access %s: %s', [DIR_NAME, testPath]);
        end;
  
        {OK, the file does not exist, lets write to it}
        fOnProgressNotification('Testing write access to '+DIR_NAME);
        //testData.SaveToFile(testFile);
        fOnProgressNotification('Testing read access to '+DIR_NAME);
        testData.LoadFromFile(testFile);
        fOnProgressNotification('Testing delete access to '+DIR_NAME);
        if not DeleteFile(PChar(testFile)) then
          raise Exception.CreateFmt('Unable to delete test file ''%s''', [testFile]);
        fReturnMessage.Add('Successful connection!');
      except
        on E: Exception do
          begin
            fReturnMessage.Add('Connection Failed!');
            fReturnMessage.Add('');
            fReturnMessage.Add(e.Message);
          end;
      end;
    finally
      fOnProgressNotification('Done');
      fOnCompletionNotification(fReturnMessage.Text);
      FreeAndNil(testData);
      Free;
    end;
  
  Terminate;
end;

function TCPGatewayTest.getIsStillRunning: Boolean;
begin
  Result := (not Terminated);
end;

function TCPGatewayTest.getResults: string;
begin
  Result := fReturnMessage.Text;
end;

procedure TCPGatewayTest.OnDefaultCompletionNotification(aMsg: string);
begin
  // Default to prevent null pointers
end;

procedure TCPGatewayTest.OnDefaultProgressNotification(aMsg: string);
begin
  // Default to prevent null pointers
end;

procedure TCPGatewayTest.setOnCompletionNotification(aMethod: TCPGatewayCompletionNotification);
begin
  //fOnCompletionNotification := aMethod;
end;

procedure TCPGatewayTest.setOnProgressNotification(aMethod: TCPGatewayProgressNotification);
begin
  fOnProgressNotification := aMethod;
end;

end.

