unit oCPImport;

interface

uses
  XMLDoc,
  XMLIntf,
  SysUtils,
  Dialogs,
  oInterfaces,
  uCPXML,
  Classes;

type
  TCPImport = class(TObject)
  private
    fDBConnection: IDBConnection;
    fXMLDocument: IXMLDocument;
    fHeader: IXMLNode;
    fTables: IXMLNode;
    fFileKey: string;
    procedure Execute(aFilename: string); stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

{ TCPImport }

constructor TCPImport.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
end;

destructor TCPImport.Destroy;
begin
  fDBConnection := nil;
  inherited;
end;

procedure TCPImport.Execute(aFilename: string);
var
  aTable: IXMLNode;
  aRecord: IXMLNode;
  aTableName: string;
  aPrimaryKeys: array of variant;
  aFieldList: array of string;
  aValueList: array of variant;
  i, j: integer;
  aParams: TStringList;
begin
  fXMLDocument := NewXMLDocument;
  fXMLDocument.LoadFromFile(aFilename);
  aParams := TStringList.Create;
  fTables := fXMLDocument.DocumentElement.ChildNodes.FindNode('TABLES');
  aTable := fTables.ChildNodes.First;
  while Assigned(aTable) do
    try
      ShowMessage('TABLE TO IMPORT:' + #13 + aTable.XML);
      j := aTable.Attributes['PK_CNT'];
      SetLength(aPrimaryKeys, j);
      for i := 1 to j do
        aPrimaryKeys[i - 1] := aTable.Attributes[Format('PK_%0.2d', [i])];
      i := Length(aPrimaryKeys);
      aRecord := aTable.ChildNodes.First;
      while Assigned(aRecord) do
        try
          aParams.Clear;
          aParams.Add(aTable.Attributes['NAME']);
          for i := Low(aPrimaryKeys) to High(aPrimaryKeys) do
            begin
              aParams.Add(aPrimaryKeys[i]);
              aParams.Add(aRecord.ChildValues[aPrimaryKeys[i]]);
            end;
          ShowMessage('Values to determine update/insert' + #13 + aParams.Text);
        finally
          aRecord := aRecord.NextSibling;
        end;
    finally
      aTable := aTable.NextSibling;
    end;
  FreeAndNil(aParams);
end;

end.

