unit oCPParameters;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 17 $  $Modtime: 7/21/09 3:21p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  CP Parameters
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPParameters.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oCPTypes,
  oCPParameterFile,
  oInterfaces;

type
  TCPParameters = class(TInterfacedObject, ICPConsoleItem, ICPParameters)
  private
    fDBConnection: IDBConnection;
    fAllowExternalAttachments: Boolean;
    fAllowCachedQueries: Boolean;
    fByPassCRC: Boolean;
    fCPHomePage: string;
    fName: string;
    fOfflineMessage: string;
    fOffLine: Boolean;
    fVersions: TStringList;
    fBatchLoadSize: integer;
    fUpdateFramework: string;
    fUpdateMaster: string;
    fUpdateSource: string;
    fUnverifiedRetention: integer;
    fInstrumentDataRetention: integer;
    fImagingXferDirectory: string;
    fRegisteredApplicationVersion: string;
    fCurrentApplicationVersion: string;
    fCurrentApplicationCompileDateTime: string;
    fCurrentApplicationCRCValue: string;
    fNotificationList: array[TCPNotificationList] of string;
    fTerminologyVersion: integer;
    fTerminologyDescription: string;
    fTerminologyCacheSettings: string;
    {ICPConsole}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;

    {ICPParameters}
    function getAllowExternalAttachments: boolean; stdcall;
    function getByPassCRC: boolean; stdcall;
    function getAllowCachedQueries: boolean; stdcall;
    function getCPHomePage: string; stdcall;
    function getOffLine: Boolean; stdcall;
    function getOfflineMessage: string; stdcall;
    function getBatchLoadSize: integer; stdcall;
    function getUnverifiedRetention: integer; stdcall;
    function getInstrumentDataRetention: integer; stdcall;
    function getImagingXferDirectory: string; stdcall;
    function getRegisteredApplicationVersion: string; stdcall;
    function getCurrentApplicationVersion: string; stdcall;
    function getCurrentApplicationCompileDateTime: string; stdcall;
    function getCurrentApplicationCRCValue: string; stdcall;
    function getApplicationRegistered: boolean; stdcall;
    function getNotificationList(const aList: TCPNotificationList): string; stdcall;

    {implementation settings - managed only via roll-n-scroll}
    function getUseKardex(aLocationName: string): boolean; stdcall;
    function getUseTIUNoteWriter(aLocationName: string): boolean; stdcall;

    {Terminology Framework is set via KIDS only}
    function getUpdateFramework: string; stdcall;
    function getUpdateMaster: string; stdcall;
    function getUpdateSource: string; stdcall;

    function getTerminologyVersion: integer; stdcall;
    function getTerminologyDescription: string; stdcall;
    function getTerminologyCacheConnectionString: string; stdcall;
    function getTerminologyCached: boolean; stdcall;
    function getTerminologyPath: string; stdcall;

    function getVersionCRCValue(aVersion: string): string; stdcall;
    function getVersionCompatible(aVersion: string): boolean; stdcall;

    procedure setAllowExternalAttachments(const aValue: boolean); stdcall;
    procedure setByPassCRC(const aValue: boolean); stdcall;
    procedure setAllowCachedQueries(const aValue: boolean); stdcall;
    procedure setCPHomePage(const aValue: string); stdcall;
    procedure setOfflineMessage(const aValue: string); stdcall;
    procedure setOffLine(const aValue: boolean); stdcall;
    procedure setBatchLoadSize(const aValue: integer); stdcall;
    procedure setUnverifedRetention(const aValue: integer); stdcall;
    procedure setInstrumentDataRetention(const aValue: integer); stdcall;
    procedure setImagingXferDirectory(const aValue: string); stdcall;
    procedure setNotificationList(const aList: TCPNotificationList; const aValue: string); stdcall;

    procedure setTerminologyVersion(const aValue: integer); stdcall;
    procedure setTerminologyDescription(const aValue: string); stdcall;
    procedure setTerminologyCacheConnectionString(const aValue: string); stdcall;
    procedure setTerminologyCached(const aValue: boolean); stdcall;
    procedure setTerminologyPath(const aValue: string); stdcall;

    procedure Save; stdcall;
    procedure SaveList(const aList: TCPNotificationList); stdcall;
  public
    constructor Create(aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

implementation

uses
  Forms, {TApplication object}
  oVersionInformation,
  MFunStr,
  SysUtils;

const
  BoolStr: array[boolean] of string = ('0', '1');

  { TCPParameters }

procedure TCPParameters.AddToList(aList: TStrings);
begin
  aList.AddObject(fName, Self);
end;

procedure TCPParameters.AddToListView(aListView: TListView);
begin
  aListView.Items.Add.Data := Self;
end;

function TCPParameters.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fName, Self);
end;

constructor TCPParameters.Create(aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fName := 'CP Parameters';
  fVersions := TStringList.Create;
  fBatchLoadSize := 20;
end;

destructor TCPParameters.Destroy;
begin
  FreeAndNil(fVersions);
  fDBConnection := nil;
  inherited;
end;

function TCPParameters.getActive: boolean;
begin
  Result := True;
end;

function TCPParameters.getAllowCachedQueries: boolean;
begin
  Result := fAllowCachedQueries;
end;

function TCPParameters.getAllowExternalAttachments: Boolean;
begin
  Result := fAllowExternalAttachments;
end;

function TCPParameters.getApplicationRegistered: boolean;
begin
  Result := (fRegisteredApplicationVersion = fCurrentApplicationVersion);
end;

procedure TCPParameters.setActive(const aValue: boolean);
begin
  //
end;

procedure TCPParameters.setAllowCachedQueries(const aValue: boolean);
begin
  fAllowCachedQueries := aValue;
end;

procedure TCPParameters.setAllowExternalAttachments(const aValue: boolean);
begin
  fAllowExternalAttachments := aValue;
end;

function TCPParameters.getBatchLoadSize: integer;
begin
  Result := fBatchLoadSize;
end;

function TCPParameters.getByPassCRC: Boolean;
begin
  Result := fByPassCRC;
end;

procedure TCPParameters.setBatchLoadSize(const aValue: integer);
begin
  fBatchLoadSize := aValue;
end;

procedure TCPParameters.setByPassCRC(const aValue: boolean);
begin
  fByPassCRC := aValue;
end;

function TCPParameters.getCanDelete: boolean;
begin
  Result := False;
end;

function TCPParameters.getCanRename: boolean;
begin
  Result := False;
end;

function TCPParameters.getComment: string;
begin
  Result := 'Common parameter values for the Clinical Procedures Suite of applications.';
end;

function TCPParameters.getCPHomePage: string;
begin
  Result := fCPHomePage;
end;

function TCPParameters.getCurrentApplicationCompileDateTime: string;
begin
  Result := fCurrentApplicationCompileDateTime;
end;

function TCPParameters.getCurrentApplicationCRCValue: string;
begin
  Result := fCurrentApplicationCRCValue;
end;

function TCPParameters.getCurrentApplicationVersion: string;
begin
  Result := fCurrentApplicationVersion;
end;

function TCPParameters.getRegisteredApplicationVersion: string;
begin
  Result := fRegisteredApplicationVersion;
end;

function TCPParameters.getDisplayName: string;
begin
  Result := 'Common Clinical Procedure parameter values.';
end;

function TCPParameters.getID: string;
begin
  Result := 'MD MANAGER';
end;

function TCPParameters.getImagingXferDirectory: string;
begin
  Result := fImagingXferDirectory;
end;

function TCPParameters.getInstrumentDataRetention: integer;
begin
  Result := fInstrumentDataRetention;
end;

function TCPParameters.getIsLoaded: boolean;
begin
  Result := True;
end;

function TCPParameters.getItemType: TCPItemType;
begin
  Result := ciParameters;
end;

function TCPParameters.getName: string;
begin
  Result := fName;
end;

function TCPParameters.getNotificationList(const aList: TCPNotificationList): string;
begin
  Result := fNotificationList[aList];
end;

procedure TCPParameters.setComment(const aValue: string);
begin
  //
end;

procedure TCPParameters.setCPHomePage(const aValue: string);
begin
  fCPHomePage := aValue;
end;

procedure TCPParameters.setDisplayName(const aValue: string);
begin
  //
end;

procedure TCPParameters.setImagingXferDirectory(const aValue: string);
begin
  fImagingXferDirectory := aValue;
end;

procedure TCPParameters.setInstrumentDataRetention(const aValue: integer);
begin
  fInstrumentDataRetention := aValue;
end;

procedure TCPParameters.setName(const aValue: string);
begin
  //
end;

procedure TCPParameters.setNotificationList(const aList: TCPNotificationList; const aValue: string);
begin
  fNotificationList[aList] := aValue;
end;

function TCPParameters.getOfflineMessage: string;
begin
  Result := fOfflineMessage;
end;

function TCPParameters.getUseTIUNoteWriter(aLocationName: string): boolean;
begin
  result := true;

{
  No longer needed now that we're live.
}
//  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
//    try
//      if ReadString(pkSystem, 'USE_NOTEWRITER') = '1' then
//        Result := True
//      else if ReadString(pkDivision, 'USE_NOTEWRITER') = '1' then
//        Result := True
//      else if ReadString(Format('LOC.%s', [aLocationName]), 'USE_NOTEWRITER') = '1' then
//        Result := True
//      else
//        Result := False;
//    except
//      Result := False;
//    end;
end;

function TCPParameters.getTerminologyCacheConnectionString: string;
begin
  Result := fTerminologyCacheSettings;
end;

function TCPParameters.getTerminologyCached: boolean;
begin
  Result := (StrToIntDef(Copy(fTerminologyCacheSettings, 1, 1), 0) = 1);
end;

function TCPParameters.getTerminologyDescription: string;
begin
  Result := fTerminologyDescription;
end;

function TCPParameters.getTerminologyPath: string;
begin
  Result := Piece(fTerminologyCacheSettings, ';', 2);
end;

function TCPParameters.getUnverifiedRetention: integer;
begin
  Result := fUnverifiedRetention;
end;

function TCPParameters.getUpdateFramework: string;
begin
  Result := fUpdateFramework;
end;

function TCPParameters.getUpdateMaster: string;
begin
  Result := fUpdateMaster;
end;

function TCPParameters.getUpdateSource: string;
begin
  Result := fUpdateSource;
end;

function TCPParameters.getUseKardex(aLocationName: string): boolean;
begin
{
Until we get Kardex working:
}

  result := false;

//  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
//    try
//      if ReadString(pkSystem, 'USE_KARDEX') = '1' then
//        Result := True
//      else if ReadString(pkDivision, 'USE_KARDEX') = '1' then
//        Result := True
//      else if ReadString(Format('LOC.%s', [aLocationName]), 'USE_KARDEX') = '1' then
//        Result := True
//      else
//        Result := False;
//    except
//      Result := False;
//    end;
end;

function TCPParameters.getTerminologyVersion: integer;
begin
  Result := fTerminologyVersion;
end;

procedure TCPParameters.setOfflineMessage(const aValue: string);
begin
  fOfflineMessage := aValue;
end;

procedure TCPParameters.setTerminologyCacheConnectionString(const aValue: string);
begin
  fTerminologyCacheSettings := aValue;
end;

procedure TCPParameters.setTerminologyCached(const aValue: boolean);
begin
  if aValue then
    fTerminologyCacheSettings := '1;' + Piece(fTerminologyCacheSettings, ';', 2)
  else
    fTerminologyCacheSettings := '0;' + Piece(fTerminologyCacheSettings, ';', 2)
end;

procedure TCPParameters.setTerminologyDescription(const aValue: string);
begin
  fTerminologyDescription := aValue;
end;

procedure TCPParameters.setTerminologyPath(const aValue: string);
begin
  fTerminologyCacheSettings := Piece(fTerminologyCacheSettings, ';', 1) + ';' + aValue;
end;

procedure TCPParameters.setTerminologyVersion(const aValue: integer);
begin
  fTerminologyVersion := aValue;
end;

procedure TCPParameters.setUnverifedRetention(const aValue: integer);
begin
  fUnverifiedRetention := aValue;
end;

function TCPParameters.getOffLine: Boolean;
begin
  Result := fOffLine;
end;

procedure TCPParameters.setOffLine(const aValue: boolean);
begin
  fOffLine := aValue;
end;

function TCPParameters.getVersionCompatible(aVersion: string): Boolean;
begin
  //Result := (Piece(ReadString(pkSystem, Format('VERSION_%s', [aVersion])), ';', 1) = '1');
  result := false;
end;

function TCPParameters.getVersionCRCValue(aVersion: string): string;
begin
  //Result := Piece(ReadString(pkSystem, Format('VERSION_%s', [aVersion])), ';', 2);
  result := '';
end;

procedure TCPParameters.Load;
var
  aList: TCPNotificationList;
begin
  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
    try
      for aList := Low(TCPNotificationList) to High(TCPNotificationList) do
        fNotificationList[aList] := ReadString(pkSystem, CP_NOTIFICATION_PARAM[aList]);

      fAllowExternalAttachments := ReadBoolean(pkSystem, 'ALLOW_EXTERNAL_ATTACHEMENTS');
      fAllowCachedQueries := ReadBoolean(pkSystem, 'ALLOW_CACHED_QUERIES');
      fByPassCRC := ReadBoolean(pkSystem, 'BYPASS_CRC');
      fCPHomePage := ReadString(pkSystem, 'HOMEPAGE');
      fOfflineMessage := ReadString(pkSystem, 'OFFLINE_MESSAGE');
      fOffLine := ReadBoolean(pkSystem, 'OFFLINE');
      fBatchLoadSize := ReadInteger(pkSystem, 'BATCH_LOAD_SIZE', 20);
      fUnverifiedRetention := ReadInteger(pkSystem, 'UNVERIFIED_OBS_RETENTION', 0);
      fInstrumentDataRetention := ReadInteger(pkSystem, 'INSTRUMENT_DATA_RETENTION', 0);
      fImagingXferDirectory := ReadString(pkSystem, 'IMAGING_XFER_DIRECTORY');

      fRegisteredApplicationVersion := ReadString(pkSystem,
        'VERSION_' + UpperCase(ChangeFileExt(ExtractFileName(Application.ExeName), '')), '0.0.0.0');

      with TVersionInformation.Create(application.ExeName) do
        try
          fCurrentApplicationVersion := getFileVersion;
          fCurrentApplicationCompileDateTime := getCompiledDateTime;
          fCurrentApplicationCRCValue := getCRC32;
        finally
          Free;
        end;

      fUpdateFramework := ReadString(pkSystem, 'UPDATE_FRAMEWORK');
      fUpdateMaster := ReadString(pkSystem, 'UPDATE_MASTER');
      fUpdateSource := ReadString(pkSystem, 'UPDATE_SOURCE');

      fTerminologyVersion := ReadInteger(pkSystem, 'TERMINOLOGY_VERSION');
      fTerminologyDescription := ReadString(pkSystem, 'TERMINOLOGY_DESCRIPTION');
      fTerminologyCacheSettings := ReadString(pkSystem, 'TERMINOLOGY_CACHE_SETTINGS');

      {Legacy parameter override - used only until new CPUser is written and M-Code is scrubbed}
      setApplicationKey('MD ALLOW EXTERNAL ATTACHMENTS');
      fAllowExternalAttachments := ReadBoolean(pkSystem, '1');

      setApplicationKey('MD CRC BYPASS');
      fByPassCRC := ReadBoolean(pkSystem, '1');

      setApplicationKey('MD DAYS FOR INSTRUMENT DATA');
      fInstrumentDataRetention := ReadInteger(pkSystem, '1');

      setApplicationKey('MD IMAGING XFER');
      fImagingXferDirectory := ReadString(pkSystem, '1');

      setApplicationKey('MD WEBLINK');
      fCPHomePage := ReadString(pkSystem, '1');

      setApplicationKey('MD ONLINE');
      fOffLine := (not ReadBoolean(pkSystem, '1'));
    except
      raise;
    end;
end;

procedure TCPParameters.Save;
begin
  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
    try
      WriteBoolean(pkSystem, 'ALLOW_EXTERNAL_ATTACHEMENTS', fAllowExternalAttachments);
      WriteBoolean(pkSystem, 'ALLOW_CACHED_QUERIES', fAllowCachedQueries);
      WriteBoolean(pkSystem, 'BYPASS_CRC', fByPassCRC);
      WriteString(pkSystem, 'HOMEPAGE', fCPHomePage);
      WriteBoolean(pkSystem, 'OFFLINE', fOffline);
      WriteString(pkSystem, 'OFFLINE_MESSAGE', fOfflineMessage);
      WriteInteger(pkSystem, 'BATCH_LOAD_SIZE', fBatchLoadSize);
      WriteInteger(pkSystem, 'TERMINOLOGY_VERSION', fTerminologyVersion);
      WriteString(pkSystem, 'TERMINOLOGY_DESCRIPTION', fTerminologyDescription);
      WriteString(pkSystem, 'TERMINOLOGY_CACHE_SETTINGS', fTerminologyCacheSettings);
      WriteInteger(pkSystem, 'UNVERIFIED_OBS_RETENTION', fUnverifiedRetention);
      WriteInteger(pkSystem, 'INSTRUMENT_DATA_RETENTION', fInstrumentDataRetention);
      WriteString(pkSystem, 'IMAGING_TRANSFER_DIRECTORY', fImagingXferDirectory);

      {Legacy parameter override - used only until new CPUser is written and M-Code is scrubbed}

      setApplicationKey('MD ALLOW EXTERNAL ATTACHMENTS');
      WriteBoolean(pkSystem, '1', fAllowExternalAttachments);

      setApplicationKey('MD CRC BYPASS');
      WriteBoolean(pkSystem, '1', fByPassCRC);

      setApplicationKey('MD DAYS FOR INSTRUMENT DATA');
      WriteInteger(pkSystem, '1', fInstrumentDataRetention);

      setApplicationKey('MD IMAGING XFER');
      WriteString(pkSystem, '1', fImagingXferDirectory);

      setApplicationKey('MD WEBLINK');
      WriteString(pkSystem, '1', fCPHomePage);

      setApplicationKey('MD ONLINE');
      WriteBoolean(pkSystem, '1', (not fOffLine));
    except
      raise;
    end;
end;

procedure TCPParameters.SaveList(const aList: TCPNotificationList);
begin
  with NewCPParameterFile(fDBConnection, 'MD PARAMETERS') do
    try
      WriteString(pkSystem, CP_NOTIFICATION_PARAM[aList], fNotificationList[aList]);
    except
      raise;
    end;
end;

end.

