unit oCPTypes;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 9 $  $Modtime: 11/25/08 12:49p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Contains all types and constants for CP.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCPTypes.pas $
================================================================================
}
interface

uses
  SysUtils;

type
  TCPItemType = (ciProcedure, ciInstrument, ciParameters, ciMappingTable, ciFlowsheetView, ciFlowsheet, ciBackgroundTask, ciShift, ciFlowsheetTotal, ciSchedule);
  TCPProcessingApplication = (paUnknown, paDefault, paHemodialysis);
  TCPProcessedResult = (prFinal, prMultiple, prCummulative);
  TCPFlowsheetPageType = (ptMandatory, ptOptional, ptSupplemental);
  TCPParamKey = (pkSystem, pkDivision, pkUser);
  TCPBackgroundTaskRecurrence = (trNone, trHourly, trDaily, trWeekly, trMonthly, trYearly);
  TCPLocationType = (ltUnknown, ltClinic, ltModule, ltWard, ltOther, ltNonClinicStop, ltFileArea, ltImaging, ltOperatingRoom);
  TDataRetrievalType = (drtNotify, drtPoll);
  TCPLookAndFeel = (lfCPToolBar, lfClassicCPRS);
  TCPAccessLevel = (alReadOnly, alReadWrite, alReadWriteDelete, alFullControl);

  TCPScheduleType = (stContinuous, stPRN, stOneTime, stStat, stNow);
  TCPScheduleTypePRN = (prnNot, prnAnd, prnAndNTE, prnOr);

  TCPKardexActionStatus = (asActive, asExpired, asDiscontinued, asHeld);
  TCPKardexEventStatus = (esPending, esCompleted, esHeld, esRefused);

  TCPNotificationList = (nlAdministrators, nlHL7Errors, nlTerminologyErrors, nlFlowsheetErrors);

const
  CONSOLE_ITEM_HEADER: array[TCPItemType] of string = ('Procedure', 'Instrument', 'Parameters', 'Mapping Table', 'Flowsheet View', 'Flowsheet', 'Background Task', 'Shift', 'Flowsheet Total', 'Schedule');

  CONSOLE_ITEM_NAME: array[TCPItemType] of string = ('Procedures', 'Instruments', 'Parameters', 'Mapping Tables', 'Flowsheet Views', 'Flowsheets', 'Background Tasks', 'Shifts', 'Flowsheet Totals', 'Schedule');

  CP_PROCESSING_APPLICATION: array[TCPProcessingApplication] of string = ('Unknown', 'Default', 'Hemodialysis');

  CP_PROCESSED_RESULT: array[TCPProcessedResult] of string = ('Final Result', 'Multiple Results', 'Cummulative Results');

  CP_YN: array[boolean] of integer = (0, 1);

  CP_YN_TEXT: array[boolean] of string = ('False', 'True');

  CP_PARAM_KEY_NAME: array[TCPParamKey] of string = ('SYS', 'DIV', 'USR');

  CP_BACKGROUND_TASK_RECURRENCE: array[TCPBackgroundTaskRecurrence] of string = ('None', 'Hourly', 'Daily', 'Weekly', 'Monthly', 'Yearly');

  CP_LOCATION_TYPE: array[TCPLocationType] of string = ('Unknown', 'Clinic', 'Module', 'Ward', 'Other Location', 'Non-Clinic Stop', 'File Area', 'Imaging', 'Operating Room');

  CP_LOCATION_TYPE_ABBV: array[TCPLocationType] of string = ('U', 'C', 'M', 'W', 'Z', 'N', 'F', 'I', 'OR');

  CP_FLOWSHEET_PAGE_TYPE: array[TCPFlowsheetPageType] of string = ('Mandatory', 'Optional', 'Supplemental');

  CP_LOOK_AND_FEEL: array[TCPLookAndFeel] of string = ('CP Toolbars', 'Classic CPRS Tabs');

  CP_ACCESS_LEVEL: array[TCPAccessLevel] of string = ('Read Only', 'Read+Write', 'Read+Write+Delete', 'Full Control');

  CP_KARDEX_ACTION_STATUS: array[TCPKardexActionStatus] of string = ('Active', 'Expired', 'Discontinued', 'Held');

  CP_KARDEX_EVENT_STATUS: array[TCPKardexEventStatus] of string = ('Due', 'Completed', 'Held', 'Refused');

  CP_SCHEDULE_TYPE: array[TCPScheduleType] of string = ('Continuous', 'PRN', 'One-Time', 'Stat', 'Now');

  CP_SCHEDULE_TYPE_PRN: array[TCPScheduleTypePRN] of string = ('No PRN', 'and PRN', 'and PRN/NTE', 'or PRN');

  CP_NOTIFICATION_LIST: array[TCPNotificationList] of string = ('CP Administrators', 'HL7 Errors', 'Terminology Errors', 'Flowsheet Errors');

  CP_NOTIFICATION_PARAM: array[TCPNotificationList] of string = ('NOTIFICATION_CPADMIN', 'NOTIFICATION_HL7_ERRORS', 'NOTIFICATION_TERMINOLOGY_ERRORS', 'NOTIFICATION_FLOWSHEET_ERRORS');

implementation

end.

