unit oCliOAlarm;

interface

uses
  Classes,
  ComCtrls, {TListView}
  SysUtils,
  oCliO,
  oCliOTypes,
  oInterfaces,
  XMLIntf;

type
  TCliOAlarm = class(TInterfacedObject, ICliOAlarm)
  private
    fDBConnection: IDBConnection;
    fTerminology: ITerminology;
    fAlarmList: ICliOAlarmList;
    fID: string;
    fPatientID: string;
    fTerm: ITerm;
    fUnit: ITerm;
    fAlarm: string;
    fAlarmValue: variant;
    fAlarmType: TCliOAlarmType;
    fMinValue: double;
    fMaxValue: double;
    fPickListValues: TStringList;
    fActive: boolean;
    fActivated: boolean;
    fActivationDateTime: TDateTime;
    fActivationByID: string;
    fActivationComment: string;
    fDeactivationDateTime: TDateTime;
    fDeactivationByID: string;
    fDeactivationComment: string;

    function getID: string; stdcall;
    function getPatientID: string; stdcall;
    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getAlarmType: TCliOAlarmType; stdcall;
    function getMinValue: double; stdcall;
    function getMaxValue: double; stdcall;
    function getPickListValues: string; stdcall;
    function getActive: boolean; stdcall;
    function getActivated: boolean; stdcall;
    function getActivationDateTime: TDateTime; stdcall;
    function getActivationByID: string; stdcall;
    function getActivationComment: string; stdcall;
    function getDeactivationDateTime: TDateTime; stdcall;
    function getDeactivationByID: string; stdcall;
    function getDeactivationComment: string; stdcall;

    procedure setPatientID(const aValue: string); stdcall;
    procedure setTerm(const aValue: ITerm); stdcall;
    procedure setUnit(const aValue: ITerm); stdcall;
    procedure setAlarmType(const aValue: TCliOAlarmType); stdcall;
    procedure setMinValue(const aValue: double); stdcall;
    procedure setMaxValue(const aValue: double); stdcall;
    procedure setPickListValues(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setActivationDateTime(const aValue: TDateTime); stdcall;
    procedure setActivationByID(const aValue: string); stdcall;
    procedure setActivationComment(const aValue: string); stdcall;
    procedure setDeactivationDateTime(const aValue: TDateTime); stdcall;
    procedure setDeactivationByID(const aValue: string); stdcall;
    procedure setDeactivationComment(const aValue: string); stdcall;

    function GetAlarm: string; stdcall;
    function GetAlarmSettings: string; stdcall;
    function GetAlarmValue: variant; stdcall;
    procedure Clear; stdcall;

    function Execute(aValue: Variant; aUnit: ITerm = nil): boolean; stdcall;
    procedure Deactivate(aUser: IUser; aComment: string); stdcall;
    procedure Update(aUser: IUser); stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;

    procedure LoadFromXML(aXMLNode: IXMLNode);
  public
    function getAlarmList: ICliOAlarmList; stdcall;
    procedure setAlarmList(const aValue: ICliOAlarmList); stdcall;

    constructor Create(aDBConnection: IDBConnection; aTerminology: ITerminology; aXMLNode: IXMLNode);
    destructor Destroy; override;
  end;

implementation

uses
  StrUTils,
  uCPXML;

{ TCliOAlarm }

constructor TCliOAlarm.Create(aDBConnection: IDBConnection; aTerminology: ITerminology; aXMLNode: IXMLNode);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fTerminology := aTerminology;
  fPickListValues := TStringList.Create;
  if Assigned(aXMLNode) then
    LoadFromXML(aXMLNode);
end;

destructor TCliOAlarm.Destroy;
begin
  fDBConnection := nil;
  FreeAndNil(fPickListValues);
  inherited;
end;

function TCliOAlarm.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  Result.Caption := fTerm.getTerm;
  Result.Data := Self;
end;

procedure TCliOAlarm.Clear;
begin
  fDBConnection.ExecCommand('PatientAlarmClear', [fID, 0]);
  fActivated := False;
  fAlarmList.OnCliOAlarmClear(Self as ICliOAlarm);
end;

function TCliOAlarm.Execute(aValue: Variant; aUnit: ITerm): boolean;
var
  aUnitAbbv: string;
begin
  try
    fAlarmValue := aValue;
    if Assigned(fUnit) then
      aUnitAbbv := fUnit.getAbbreviation
    else
      aUnitAbbv := '';

    if Assigned(aUnit) then
      begin
        aUnitAbbv := aUnit.getAbbreviation;
        if Assigned(aUnit) then
          if aUnit.getID <> fUnit.getID then
            aValue := fTerminology.GetConvertedValue(aValue, aUnit, fUnit)
          else
        else
          raise ETermConversionError.Create('Unit Specification Required');
      end
    else
      aUnitAbbv := '';

    case fAlarmType of
      atWithinRange: Result := (aValue >= fMinValue) and (aValue <= fMaxValue);
      atNotWithinRange: Result := (aValue < fMinValue) or (aValue > fMaxValue);
      atGreaterThan: Result := (aValue > fMaxValue);
      atLessThan: Result := (aValue < fMinValue);
      atBooleanTrue: Result := (aValue = 1);
      atBooleanFalse: Result := (aValue <> 1);
      atEqualsPickList: Result := (Pos(aValue, fPickListValues.CommaText) > 0);
    else
      Result := False;
    end;

    if Result then
      begin
        if not fActivated then
          fDBConnection.ExecCommand('PatientAlarmSet', [fID, 1]);

        if fAlarmType = atEqualsPickList then
          fAlarmValue := fTerminology.GetTerm(fAlarmValue).getTerm;

        fAlarm := Format('Value %s%s for %s is %s',
          [fAlarmValue, aUnitAbbv, fTerm.getTerm, GetAlarmSettings]);
      end;
  except
    on E: ETermConversionError do
      begin
        fAlarmValue := aValue;
        fAlarm := 'Unable to convert value to ' + fUnit.getTerm;
        Result := True; {Need to raise the error regardless and fire the alarm}
      end;
    on E: Exception do
      begin
        fAlarmValue := aValue;
        fAlarm := E.Message;
        Result := True;
      end;
  end;

  if Result then
    fAlarmList.OnCliOAlarmSet(Self as ICliOAlarm);
end;

function TCliOAlarm.getActivated: boolean;
begin
  Result := fActivated;
end;

function TCliOAlarm.getActivationByID: string;
begin
  Result := fActivationByID;
end;

function TCliOAlarm.getActivationComment: string;
begin
  Result := fActivationComment;
end;

function TCliOAlarm.getActivationDateTime: TDateTime;
begin
  Result := fActivationDateTime;
end;

function TCliOAlarm.getActive: boolean;
begin
  Result := fActive;
end;

function TCliOAlarm.GetAlarm: string;
begin
  Result := fAlarm;
end;

function TCliOAlarm.getAlarmList: ICliOAlarmList;
begin
  Result := fAlarmList;
end;

function TCliOAlarm.GetAlarmSettings: string;
var
  i: integer;
  aUnit: string;
begin
  if Assigned(fUnit) then
    aUnit := fUnit.getAbbreviation
  else
    aUnit := '';

  case fAlarmType of
    atWithinRange: Result := Format('BETWEEN %s%s and %s%s', [FloatToStr(fMinValue), aUnit, FloatToStr(fMaxValue), aUnit]);
    atNotWithinRange: Result := Format('NOT BETWEEN %s%s and %s%s', [FloatToStr(fMinValue), aUnit, FloatToStr(fMaxValue), aUnit]);
    atLessThan: Result := Format('LESS THAN %s%s', [FloatToStr(fMinValue), aUnit]);
    atGreaterThan: Result := Format('GREATER THAN %s%s', [FloatToStr(fMaxValue), aUnit]);
    atEqualsPickList:
      begin
        Result := StringReplace(fPickListValues.CommaText, ',', ' or ', [rfReplaceAll]);
        for i := 0 to fPickListValues.Count - 1 do
          Result := StringReplace(Result, fPickListValues[i], '''' + fTerminology.GetTerm(fPickListValues[i]).getTerm + '''', [rfReplaceAll]);
        Result := Format('EQUALS %s', [Result]);
      end;
  else
    Result := 'Alarm not properly set,';
  end;
end;

function TCliOAlarm.getAlarmType: TCliOAlarmType;
begin
  Result := fAlarmType;
end;

function TCliOAlarm.GetAlarmValue: variant;
begin
  Result := fAlarmValue;
end;

function TCliOAlarm.getDeactivationByID: string;
begin
  Result := fDeactivationByID;
end;

function TCliOAlarm.getDeactivationComment: string;
begin
  Result := fDeactivationComment;
end;

function TCliOAlarm.getDeactivationDateTime: TDateTime;
begin
  Result := fDeactivationDateTime;
end;

function TCliOAlarm.getID: string;
begin
  Result := fID;
end;

function TCliOAlarm.getMaxValue: double;
begin
  Result := fMaxValue;
end;

function TCliOAlarm.getMinValue: double;
begin
  Result := fMinValue;
end;

function TCliOAlarm.getPatientID: string;
begin
  Result := fPatientID;
end;

function TCliOAlarm.getPickListValues: string;
begin
  Result := fPickListValues.CommaText;
end;

function TCliOAlarm.getTerm: ITerm;
begin
  Result := fTerm;
end;

function TCliOAlarm.getUnit: ITerm;
begin
  Result := fUnit;
end;

procedure TCliOAlarm.LoadFromXML(aXMLNode: IXMLNode);
begin
  fID := XMLValueAsStr(aXMLNode, 'ID');
  fPatientID := XMLValueAsStr(aXMLNode, 'PATIENT_ID');
  fTerm := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'TERM_ID'));
  fUnit := fTerminology.GetTerm(XMLValueAsStr(aXMLNode, 'UNIT_ID'));
  fAlarmType := TCliOAlarmType(XMLValueAsInt(aXMLNode, 'ALARM_TYPE'));
  fMinValue := XMLValueAsDouble(aXMLNode, 'MIN_VALUE');
  fMaxValue := XMLValueAsDouble(aXMLNode, 'MAX_VALUE');
  fPickListValues.CommaText := XMLValueAsStr(aXMLNode, 'PICKLIST_IDS');
  fActivated := (XMLValueAsInt(aXMLNode, 'ALARM_SET') = 1);
  fActive := (XMLValueAsInt(aXMLNode, 'ACTIVE', 0) = 1);
  fActivationDateTime := XMLValueAsDateTime(aXMLNode, 'ACTIVATED_DATE_TIME');
  fActivationByID := XMLValueAsStr(aXMLNode, 'ACTIVATED_BY_ID');
  fActivationComment := XMLValueAsStr(aXMLNode, 'ACTIVATED_COMMENT');
  fDeactivationDateTime := XMLValueAsDateTime(aXMLNode, 'DEACTIVATED_DATE_TIME');
  fDeactivationByID := XMLValueAsStr(aXMLNode, 'DEACTIVATED_BY_ID');
  fDeactivationComment := XMLValueAsStr(aXMLNode, 'DEACTIVATED_COMMENT');
end;

procedure TCliOAlarm.setActivationByID(const aValue: string);
begin
  fActivationByID := aValue;
end;

procedure TCliOAlarm.setActivationComment(const aValue: string);
begin
  fActivationComment := aValue;
end;

procedure TCliOAlarm.setActivationDateTime(const aValue: TDateTime);
begin
  fActivationDateTime := aValue;
end;

procedure TCliOAlarm.setActive(const aValue: boolean);
begin
  fActive := aValue;
end;

procedure TCliOAlarm.setAlarmList(const aValue: ICliOAlarmList);
begin
  fAlarmList := aValue;
end;

procedure TCliOAlarm.setAlarmType(const aValue: TCliOAlarmType);
begin
  fAlarmType := aValue;
end;

procedure TCliOAlarm.setDeactivationByID(const aValue: string);
begin
  fDeactivationByID := aValue;
end;

procedure TCliOAlarm.setDeactivationComment(const aValue: string);
begin
  fDeactivationComment := aValue;
end;

procedure TCliOAlarm.setDeactivationDateTime(const aValue: TDateTime);
begin
  fDeactivationDateTime := aValue;
end;

procedure TCliOAlarm.setMaxValue(const aValue: double);
begin
  fMaxValue := aValue;
end;

procedure TCliOAlarm.setMinValue(const aValue: double);
begin
  fMinValue := aValue;
end;

procedure TCliOAlarm.setPatientID(const aValue: string);
begin
  fPatientID := aValue;
end;

procedure TCliOAlarm.setPickListValues(const aValue: string);
begin
  fPickListValues.CommaText := aValue;
end;

procedure TCliOAlarm.setTerm(const aValue: ITerm);
begin
  fTerm := aValue;
end;

procedure TCliOAlarm.setUnit(const aValue: ITerm);
begin
  fUnit := aValue;
end;

procedure TCliOAlarm.Deactivate(aUser: IUser; aComment: string);
begin
  fDBConnection.ExecCommand('PatientAlarmDeactivate',
    [fID, Now, aUser.getDUZ, 0, 0, aComment]);
end;

procedure TCliOAlarm.Update(aUser: IUser);
var
  aTerm: string;
  aUnit: string;
begin
  try
    if fID <> '' then
      Deactivate(aUser, Format('Alarm Updated by %s.', [aUser.getName]));

    fID := NewGUID;

    aTerm := fTerm.getID;

    if Assigned(fUnit) then
      aUnit := fUnit.getID
    else
      aUnit := '';

    {
    if fPickListValues.Count > 0 then
      aValue := fPickListValues.CommaText
    else
      aValue := '';
    }

    fDBConnection.ExecCommand(
      'AddPatientAlarm',
      [fID, fPatientID, aTerm, aUnit,
      fAlarmType, fMinValue, fMaxValue, fPickListValues.CommaText,
        1, Now, aUser.getDUZ, fActivationComment]);
  except
    raise Exception.Create('Error filing alarm settings.');
  end;
end;

end.

