unit oCliOAlarmList;

interface

uses
  Classes,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oCliO,
  oCliOTypes,
  oInterfaces,
  uCPXML;

type
  TCliOAlarmList = class(TInterfacedObject, ICliOAlarmList)
  protected
    fDBConnection: IDBConnection;
    fTerminology: ITerminology;
    fList: IInterfacelist;
    fPatientID: string;

    fOnCliOAlarmSet: TOnCLiOAlarmSet;
    fOnCliOAlarmClear: TOnCliOAlarmClear;

    function getOnCliOAlarmSet: TOnCliOAlarmSet; stdcall;
    procedure setOnCliOAlarmSet(const aOnCliOAlarmSet: TOnCliOAlarmSet); stdcall;
    procedure fOnCliOAlarmExecute(aAlarm: IInterface);

    function getOnCliOAlarmClear: TOnCliOAlarmClear; stdcall;
    procedure setOnCliOAlarmClear(const aCliOAlarm: TOnCliOAlarmClear); stdcall;
    procedure fOnCliOAlarmClearExecute(aAlarm: IInterface);

    function getAlarmCount: integer; stdcall;
    function getAlarm(aIndex: integer): ICliOAlarm; overload; stdcall;
    function getAlarm(aID: string): ICliOAlarm; overload; stdcall;
    function getNewAlarm: ICliOAlarm; overload; stdcall;
    function getPatientID: string; stdcall;
    function getActivatedAlarmCount: integer; stdcall;

    procedure setPatientID(aValue: string); stdcall;
    procedure Load(aPatientID: string); stdcall;
    procedure Refresh; stdcall;
    procedure Execute(aValue: variant; aTerm: ITerm; aUnit: ITerm = nil); stdcall;
  public
    constructor Create(aDBConnection: IDBConnection; aTerminology: ITerminology);
    destructor Destroy; override;

    property OnCliOAlarmSet: TOnCliOAlarmSet read getOnCliOAlarmSet write setOnCliOAlarmSet;
    property OnCliOAlarmClear: TOnCliOAlarmClear read getOnCliOAlarmClear write setOnCliOAlarmClear;
  end;

implementation

uses
  oCliOAlarm;

{ TCliOAlarmList }

constructor TCliOAlarmList.Create(aDBConnection: IDBConnection; aTerminology: ITerminology);
begin
  inherited Create;
  fOnCliOAlarmSet := fOnCliOAlarmExecute;
  fOnCliOAlarmClear := fOnCliOAlarmClearExecute;
  TInterfaceList.Create.GetInterface(IInterfaceList, fList);
  fDBConnection := aDBConnection;
  fTerminology := aTerminology;
end;

destructor TCliOAlarmList.Destroy;
begin
  fList.Clear;
  fDBConnection := nil;
  inherited;
end;

procedure TCliOAlarmList.Execute(aValue: variant; aTerm, aUnit: ITerm);
var
  aCliOAlarm: ICliOAlarm;
  i: integer;
begin
  for i := 0 to fList.Count - 1 do
    if fList[i].QueryInterface(ICliOAlarm, aCliOAlarm) = 0 then
      if aCliOAlarm.getTerm.getID = aTerm.getID then
        aCliOAlarm.Execute(aValue, aUnit);
end;

procedure TCliOAlarmList.fOnCliOAlarmClearExecute(aAlarm: IInterface);
begin
  //
end;

procedure TCliOAlarmList.fOnCliOAlarmExecute(aAlarm: IInterface);
begin
  //
end;

function TCliOAlarmList.getAlarm(aIndex: integer): ICliOAlarm;
begin
  try
    fList[aIndex].QueryInterface(ICliOAlarm, Result);
  except
    Result := nil;
  end;
end;

function TCliOAlarmList.getActivatedAlarmCount: integer;
var
  i: integer;
begin
  Result := 0;
  for i := 0 to fList.Count - 1 do
    if (fList[i] as ICliOAlarm).getActivated then
      inc(Result);
end;

function TCliOAlarmList.getAlarm(aID: string): ICliOAlarm;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to fList.Count - 1 do
    if (fList[i] as ICliOAlarm).getID = aID then
      fList[i].QueryInterface(ICliOAlarm, Result);
end;

function TCliOAlarmList.getAlarmCount: integer;
begin
  Result := fList.Count;
end;

function TCliOAlarmList.getNewAlarm: ICliOAlarm;
begin
  if TCliOAlarm.Create(fDBConnection, fTerminology, nil).GetInterface(ICliOAlarm, Result) then
    begin
      Result.setPatientID(fPatientID);
      fList.Add(Result);
    end;
end;

function TCliOAlarmList.getOnCliOAlarmSet: TOnCliOAlarmSet;
begin
  Result := fOnCliOAlarmSet;
end;

function TCliOAlarmList.getOnCliOAlarmClear: TOnCliOAlarmClear;
begin
  Result := fOnCliOAlarmClear;
end;

function TCliOAlarmList.getPatientID: string;
begin
  Result := fPatientID;
end;

procedure TCliOAlarmList.Load(aPatientID: string);
begin
  fPatientID := aPatientID;
  Refresh;
end;

procedure TCliOAlarmList.Refresh;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aAlarm: ICliOAlarm;
begin
  fList.Clear;
  aXMLDoc := NewXMLDocument;

  try
    fDBConnection.ExecQuery('GetPatientAlarms', [fPatientID, 1], aXMLDoc);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while Assigned(aXMLRec) do
      begin
        with TCliOAlarm.Create(fDBConnection, fTerminology, aXMLRec) do
          begin
            setAlarmList(Self as ICliOAlarmList);
            GetInterface(ICliOAlarm, aAlarm);
            fList.Add(aAlarm);
          end;
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;
  except
    raise Exception.Create('Error loading patient alarms');
  end;
end;

procedure TCliOAlarmList.setOnCliOAlarmSet(const aOnCliOAlarmSet: TOnCliOAlarmSet);
begin
  fOnCliOAlarmSet := aOnCliOAlarmSet;
end;

procedure TCliOAlarmList.setOnCliOAlarmClear(const aCliOAlarm: TOnCliOAlarmClear);
begin
  fOnCliOAlarmClear := aCliOAlarm;
end;

procedure TCliOAlarmList.setPatientID(aValue: string);
begin
  Load(aValue);
end;

end.

