unit oCliOFilerSet;

interface

uses
  Classes,
  oCliO,
  oCliOTypes,
  oCP_Exceptions,
  oInterfaces,
  SysUtils,
  XMLDoc,
  XMLIntf;

type
  TCliOFilerSetField = (
    sfEnteredDateTime,
    sfEnteredByID,
    sfPublic,
    sfComment);

  TCliOFilerSet = class(TInterfacedObject, ICliOFilerSet, ICliOFilerXML)
  private
    fNode: IXMLNode;

    function getStringValue(aField: TCliOFilerSetField): string;
    function getIntegerValue(aField: TCliOFilerSetField): integer;

    procedure setStringValue(aField: TCliOFilerSetField; aValue: string);
    procedure setIntegerValue(aField: TCliOFilerSetField; aValue: integer);

    {ICliOFilerXML interface}
    function getXML: WideString; stdcall;

    {ICliOFilerSet interface}
    function getID: string; stdcall;
    function getEnteredByID: string; stdcall;
    function getEnteredDateTime: TDateTime; stdcall;
    function getIsPublic: boolean; stdcall;
    function getComment: string; stdcall;

    procedure setEnteredByID(const aValue: string); stdcall;
    procedure setEnteredDateTime(const aValue: TDateTime); stdcall;
    procedure setIsPublic(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
  public
    constructor Create(aXMLRoot: IXMLNode);
  end;

implementation

uses
  uCPXML;

const
  XMLTAGS: array[TCliOFilerSetField] of string = (
    'ENTERED_DATE_TIME',
    'ENTERED_BY_ID',
    'PUBLIC',
    'COMMENT');

{ TCliOFilerSet }

constructor TCliOFilerSet.Create(aXMLRoot: IXMLNode);
begin
  inherited Create;
  fNode := aXMLRoot.AddChild('SET');
  fNode.Attributes['ID'] := NewGUID;
end;

function TCliOFilerSet.getIntegerValue(aField: TCliOFilerSetField): integer;
begin
  Result := XMLValueAsInt(fNode, XMLTAGS[aField]);
end;

function TCliOFilerSet.getStringValue(aField: TCliOFilerSetField): string;
begin
  Result := XMLValueAsStr(fNode, XMLTAGS[aField]);
end;

procedure TCliOFilerSet.setIntegerValue(aField: TCliOFilerSetField; aValue: integer);
begin
  fNode.ChildValues[XMLTAGS[aField]] := aValue;
end;

procedure TCliOFilerSet.setStringValue(aField: TCliOFilerSetField; aValue: string);
begin
  fNode.ChildValues[XMLTAGS[aField]] := aValue;
end;

function TCliOFilerSet.getComment: string;
begin
  Result := getStringValue(sfComment);
end;

function TCliOFilerSet.getEnteredByID: string;
begin
  Result := getStringValue(sfEnteredByID);
end;

function TCliOFilerSet.getEnteredDateTime: TDateTime;
begin
  Result := XMLValueAsDateTime(fNode, XMLTAGS[sfEnteredDateTime]);
end;

function TCliOFilerSet.getIsPublic: boolean;
begin
  Result := (getIntegerValue(sfPublic) = 1);
end;

function TCliOFilerSet.getID: string;
begin
  Result := fNode.Attributes['ID'];
end;

function TCliOFilerSet.getXML: WideString;
begin
  Result := fNode.XML;
end;

procedure TCliOFilerSet.setComment(const aValue: string);
begin
  setStringValue(sfComment, aValue);
end;

procedure TCliOFilerSet.setEnteredByID(const aValue: string);
begin
  setStringValue(sfEnteredByID, aValue);
end;

procedure TCliOFilerSet.setEnteredDateTime(const aValue: TDateTime);
begin
  setStringValue(sfEnteredDateTime, DateTimeToXMLString(aValue));
end;

procedure TCliOFilerSet.setIsPublic(const aValue: boolean);
begin
  if aValue then
    setIntegerValue(sfPublic, 1)
  else
    setIntegerValue(sfPublic, 0);
end;

end.

