unit oCliOHL7Log;

interface

uses
  Classes,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  oCliO;

type
  TCliOHL7Log = class(TInterfacedObject, ICliOHL7Log)
  protected
    fDBConnection: IDBConnection;
    fEntries: IInterfaceList;

    function FirstLogEntry: ICliOHL7LogEntry; stdcall;
    function NextLogEntry(aCliOHL7LogEntry: ICliOHL7LogEntry): ICliOHL7LogEntry; stdcall;
    function LogEntry(aLogID: string): ICliOHL7LogEntry; overload; stdcall;
    function LogEntry(aIndex: integer): ICliOHL7LogEntry; overload; stdcall;
    function LogEntryCount: Integer; stdcall;

    procedure LoadMessageText(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings); stdcall;
    procedure LoadMessageErrors(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings); stdcall;
    procedure ResubmitEntry(aCliOHL7LogEntry: ICliOHL7LogEntry; aPatientID, aInstrumentID: string); stdcall;
    procedure DiscardEntry(aCliOHL7LogEntry: ICliOHL7LogEntry); stdcall;

    procedure Refresh; stdcall;
    procedure Clear; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCliOHL7Log(aDBConnection: IDBConnection): TCliOHL7Log;

implementation

uses
  uCPXML,
  oCliOHL7LogEntry;

function NewCliOHL7Log(aDBConnection: IDBConnection): TCliOHL7Log;
begin
  Result := TCliOHL7Log.Create;
  with Result do
    try
      fDBConnection := aDBConnection;
    except
      Result := nil;
    end;
end;

{ TCliOHL7Log }

procedure TCliOHL7Log.Clear;
begin
  fEntries.Clear;
end;

constructor TCliOHL7Log.Create;
begin
  inherited;
  TInterfaceList.Create.GetInterface(IInterfaceList, fEntries);
end;

destructor TCliOHL7Log.Destroy;
begin
  fEntries.Clear;
  fEntries := nil;
  inherited;
end;

procedure TCliOHL7Log.DiscardEntry(aCliOHL7LogEntry: ICliOHL7LogEntry);
begin
  //
end;

function TCliOHL7Log.FirstLogEntry: ICliOHL7LogEntry;
begin
  if fEntries.Count > 0 then
    fEntries[0].QueryInterface(ICliOHL7LogEntry, Result)
  else
    Result := nil;
end;

function TCliOHL7Log.LogEntry(aLogID: string): ICliOHL7LogEntry;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to fEntries.Count - 1 do
    with (fEntries[i] as ICliOHL7LogEntry) do
      if getLogID = aLogID then
        begin
          QueryInterface(ICliOHL7LogEntry, Result);
          Exit;
        end;
end;

procedure TCliOHL7Log.LoadMessageErrors(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  i: integer;
begin
  aList.Clear;
  NewXMLDocument.QueryInterface(IXMLDocument, aXMLDoc);
  if fDBConnection.ExecQuery('GetHL7MonitorReasons', [aCliOHL7LogEntry.getLogID], aXMLDoc) then
    begin
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLRec) do
        try
          aList.add(
            XMLValueAsStr(aXMLRec, 'DATE_TIME_ENTERED') +
            '  ' +
            XMLValueAsStr(aXMLRec, 'REASON'));
        finally
          aXMLRec := XMLDocumentNextRecord(aXMLRec);
        end
    end
  else
    aList.Add(Format('Message %s not found', [aCliOHL7LogEntry.getLogID]));

  for i := aList.Count - 1 downto 0 do
    if aList.Strings[i] = '' then
      aList.Delete(i);
end;

procedure TCliOHL7Log.LoadMessageText(aCliOHL7LogEntry: ICliOHL7LogEntry; aList: TStrings);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  i: integer;
begin
  aList.Clear;
  NewXMLDocument.QueryInterface(IXMLDocument, aXMLDoc);
  if fDBConnection.ExecQuery('GetHL7Message', [aCliOHL7LogEntry.getLogID], aXMLDoc) then
    begin
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      if Assigned(aXMLRec) then
        aList.Text := XMLValueAsStr(aXMLRec, 'HL7_TEXT')
      else
        aList.Add(Format('Message %s not found', [aCliOHL7LogEntry.getLogID]));
    end;
  for i := aList.Count - 1 downto 0 do
    if aList.Strings[i] = '' then
      aList.Delete(i);
end;

function TCliOHL7Log.LogEntry(aIndex: integer): ICliOHL7LogEntry;
begin
  try
    fEntries[aIndex].QueryInterface(ICliOHL7LogEntry, Result);
  except
    Result := nil;
  end;
end;

function TCliOHL7Log.LogEntryCount: Integer;
begin
  Result := fEntries.Count;
end;

function TCliOHL7Log.NextLogEntry(aCliOHL7LogEntry: ICliOHL7LogEntry): ICliOHL7LogEntry;
var
  i: integer;
begin
  i := fEntries.IndexOf(aCliOHL7LogEntry) + 1;
  if (i > 0) and (i < fEntries.Count) then
    fEntries[i].QueryInterface(ICliOHL7LogEntry, Result)
  else
    Result := nil;
end;

procedure TCliOHL7Log.Refresh;
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aLogEntry: ICliOHL7LogEntry;
begin
  fEntries.Clear;

  aXMLDoc := NewXMLDocument;
  if fDBConnection.ExecQuery('GetHL7MonitorErrors', [], aXMLDoc) then
    begin
      aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
      while Assigned(aXMLRec) do
        try
          if NewCliOHL7LogEntry(aXMLRec).GetInterface(ICliOHL7LogEntry, aLogEntry) then
            fEntries.Add(aLogEntry);
        finally
          aXMLRec := XMLDocumentNextRecord(aXMLRec);
        end;
    end;
end;

procedure TCliOHL7Log.ResubmitEntry(aCliOHL7LogEntry: ICliOHL7LogEntry; aPatientID, aInstrumentID: string);
begin
  //
end;

end.

