unit oCliOHL7LogEntry;

interface

uses
  Classes,
  SysUtils,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  oCliO,
  oCliOTypes;

type
  TCliOHL7LogEntry = class(TInterfacedObject, ICliOHL7LogEntry)
  private
    fLogID: string;
    fMessageDateTime: TDateTime;
    fStatus: TCliOHL7LogStatus;
    fMappingTableID: string;
    fPatientID: string;
    fPatientName: string;
    fPatientSSN: string;
    fPatientDOB: TDateTime;
    fPatientSex: string;
    fHL7PatientID: string;
    fHL7DeviceID: string;
    fHL7LocationID: string;
    fHL7ErrorType: TCliOHL7LogErrorType;
    function getLogID: string; stdcall;
    function getMessageDateTime: TDateTIme; stdcall;
    function getStatus: TCliOHL7LogStatus; stdcall;
    function getMappingTableID: string; stdcall;
    function getPatientID: string; stdcall;
    function getPatientName: string; stdcall;
    function getPatientSSN: string; stdcall;
    function getPatientDOB: TDateTime; stdcall;
    function getPatientSex: string; stdcall;
    function getHL7PatientID: string; stdcall;
    function getHL7DeviceID: string; stdcall;
    function getHL7DeviceLocation: string; stdcall;
    function getHL7Error: TCliOHL7LogErrorType; stdcall;
    function getHL7ErrorText: string; stdcall;

    procedure setPatient(aPatient: IPatient); stdcall;
    procedure setMappingTableID(aMappingTableID: string); stdcall;
  public
    constructor Create;
    destructor Destroy; override;
  end;

function NewCliOHL7LogEntry(aXMLNode: IXMLNode): TCliOHL7LogEntry;

implementation

uses
  uCPXML;

function NewCliOHL7LogEntry(aXMLNode: IXMLNode): TCliOHL7LogEntry;
begin
  Result := TCliOHL7LogEntry.Create;
  with Result do
    try
      fLogID := XMLValueAsStr(aXMLNode, 'MESSAGE_ID');
      fMessageDateTime := XMLValueAsDateTime(aXMLNode, 'MESSAGE_DATE_TIME');
      fStatus := TCliOHL7LogStatus(XMLValueAsInt(aXMLNode, 'STATUS', 0));
      fMappingTableID := XMLValueAsStr(aXMLNode, 'MAPPING_TABLE_ID');
      fPatientID := XMLValueAsStr(aXMLNode, 'PATIENT_ID');
      fPatientName := XMLValueAsStr(aXMLNode, 'PATIENT_NAME');
      fPatientSSN := XMLValueAsStr(aXMLNode, 'PATIENT_SSN');
      fPatientDOB := XMLValueAsDateTime(aXMLNode, 'PATIENT_DOB');
      fPatientSex := XMLValueAsStr(aXMLNode, 'PATIENT_SEX');
      fHL7PatientID := XMLValueAsStr(aXMLNode, 'REPORTED_PATIENT');
      fHL7DeviceID := XMLValueAsStr(aXMLNode, 'REPORTED_INSTRUMENT');
      fHL7LocationID := XMLValueAsStr(aXMLNode, 'REPORTED_LOCATION');

      if fMappingTableID = '' then
        fHL7ErrorType := etNoInstrument
      else if fPatientID = '' then
        fHL7ErrorType := etNoPatient
      else
        fHL7ErrorType := etOBXSegment;
    except
      FreeAndNil(Result);
    end;
end;

{ TCliOHL7LogEntry }

constructor TCliOHL7LogEntry.Create;
begin
  inherited;
end;

destructor TCliOHL7LogEntry.Destroy;
begin
  inherited;
end;

function TCliOHL7LogEntry.getHL7DeviceID: string;
begin
  Result := fHL7DeviceID;
end;

function TCliOHL7LogEntry.getHL7DeviceLocation: string;
begin
  Result := fHL7LocationID;
end;

function TCliOHL7LogEntry.getHL7Error: TCliOHL7LogErrorType;
begin
  Result := fHL7ErrorType;
end;

function TCliOHL7LogEntry.getHL7ErrorText: string;
begin
  Result := CLIO_HL7_LOG_ERROR_TYPE[fHL7ErrorType];
end;

function TCliOHL7LogEntry.getHL7PatientID: string;
begin
  Result := fHL7PatientID;
end;

function TCliOHL7LogEntry.getLogID: string;
begin
  Result := fLogID;
end;

function TCliOHL7LogEntry.getMappingTableID: string;
begin
  Result := fMappingTableID;
end;

function TCliOHL7LogEntry.getMessageDateTime: TDateTIme;
begin
  Result := fMessageDateTime;
end;

function TCliOHL7LogEntry.getPatientDOB: TDateTime;
begin
  Result := fPatientDOB;
end;

function TCliOHL7LogEntry.getPatientID: string;
begin
  Result := fPatientID;
end;

function TCliOHL7LogEntry.getPatientName: string;
begin
  if fPatientID <> '' then
    Result := Format(
      '%s (%s) %s %s',
      [fPatientName, fPatientSSN, fPatientSex, FormatDateTime('m/d/yyyy', fPatientDOB)])
  else
    Result := '*Unknown*';
end;

function TCliOHL7LogEntry.getPatientSex: string;
begin
  Result := fPatientSex;
end;

function TCliOHL7LogEntry.getPatientSSN: string;
begin
  Result := fPatientSSN;
end;

function TCliOHL7LogEntry.getStatus: TCliOHL7LogStatus;
begin
  Result := fStatus;
end;

procedure TCliOHL7LogEntry.setMappingTableID(aMappingTableID: string);
begin
  fMappingTableID := aMappingTableID;
end;

procedure TCliOHL7LogEntry.setPatient(aPatient: IPatient);
begin
  fPatientID := aPatient.getDFN;
  fPatientName := aPatient.getName;
  fPatientSSN := aPatient.getSSN;
  fPatientSex := aPatient.getSex;
  fPatientDOB := aPatient.getDOB;
end;

end.

