unit oCliOReportItem;

interface

uses
  oCliOReportWriter,
  Classes,
  SysUtils,
  Variants,
  Graphics,
  XMLDoc,
  XMLIntf;

type
  TCliOReportItem = class(TInterfacedObject, ICliOReportItem)
  private
    fXMLNode: IXMLNode;
    fFontStyles: TFontStyles;

    function getValue(aColumn: integer): string; stdcall;
    function getLine: string; stdcall;
    function getFontStyle: TFontStyles; stdcall;

    procedure setValue(aColumn: integer; aValue: string); stdcall;
    procedure setLine(aValue: string); stdcall;
    procedure setFontBold(aValue: boolean); stdcall;
    procedure setFontItalic(aValue: boolean); stdcall;
    procedure setFontUnderline(aValue: boolean); stdcall;
    procedure setFontStrikeThrough(aValue: boolean); stdcall;
  public
    constructor Create(aXMLRoot: IXMLNode);
    destructor Destroy; override;
  end;

implementation

{ TCliOReportItem }

constructor TCliOReportItem.Create(aXMLRoot: IXMLNode);
begin
  inherited Create;
  fXMLNode := aXMLRoot.AddChild('ITEM');
  fFontStyles := [];
end;

destructor TCliOReportItem.Destroy;
begin
  inherited;
end;

function TCliOReportItem.getFontStyle: TFontStyles;
begin
  Result := fFontStyles;
end;

function TCliOReportItem.getLine: string;
begin
  Result := VarToStrDef(fXMLNode.ChildValues['LINE'], '');
end;

function TCliOReportItem.getValue(aColumn: integer): string;
begin
  Result := VarToStrDef(fXMLNode.ChildValues[Format('C%0.2d', [aColumn])], '');
end;

procedure TCliOReportItem.setFontBold(aValue: boolean);
begin
  if aValue then
    fFontStyles := fFontStyles + [fsBold]
  else
    fFontStyles := fFontStyles - [fsBold];
end;

procedure TCliOReportItem.setFontItalic(aValue: boolean);
begin
  if aValue then
    fFontStyles := fFontStyles + [fsItalic]
  else
    fFontStyles := fFontStyles - [fsItalic];
end;

procedure TCliOReportItem.setFontStrikeThrough(aValue: boolean);
begin
  if aValue then
    fFontStyles := fFontStyles + [fsStrikeOut]
  else
    fFontStyles := fFontStyles - [fsStrikeOut];
end;

procedure TCliOReportItem.setFontUnderline(aValue: boolean);
begin
  if aValue then
    fFontStyles := fFontStyles + [fsUnderline]
  else
    fFontStyles := fFontStyles - [fsUnderline];
end;

procedure TCliOReportItem.setLine(aValue: string);
begin
  fXMLNode.ChildValues['LINE'] := aValue;
end;

procedure TCliOReportItem.setValue(aColumn: integer; aValue: string);
begin
  fXMLNode.ChildValues[Format('C%0.2d', [aColumn])] := aValue;
end;

end.

