unit oCliOTypes;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 7 $  $Modtime: 12/29/10 12:33p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Contains all types and constants for CliO.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oCliOTypes.pas $
================================================================================
}
interface

type
  {Event Handlers}
  TOnCLiOAlarmSet = procedure(aAlarm: IInterface) of object;

  TOnCliOAlarmClear = procedure(aAlarm: IInterface) of object;

  {Ordinal}
  TTermType = (ttUnknown, ttObservation, ttUnit, ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor, ttTask, ttSchedule, ttValue, ttConcept);

  TTermDataType = (dtUndefined, dtComplex, dtNumeric, dtPickList, dtBoolean, dtString, dtFunction);

  TTermDisplay = (tdTerm, tdAbbv, tdDisplayName, tdID);

  TTermChildValueType = (cvExtract, cvDelimited, cvCalculation);

  TTermValueType = (vtUnspecified, vtTemperature, vtLength, vtVolume, vtRate, vtTime, vtMass, vtScale);

  TObservationRecordStatus = (osUnverified, osVerified, osArchived, osPurged, osCorrected, osRescinded);

  TObservationRecordStatusSet = set of TObservationRecordStatus;

  TObservationViewPrompt = (vpNotAllowed, vpAllowed, vpAllowedWithDefault, vpMandatoryItem);

  TObservationViewStyle = (vsTerminology, vsDateTime);

  //TObservationViewType = (vtGrid, vtReport);

  TObservationAuditType = (atStatusChange, atVerified, atPurged, atArchived, atCorrection, atRescission);

  TControllerType = (ctTerminology, ctObservation, ctVistA);

  //TCPConsoleItemType = (ciProcedure, ciInstrument, ciGateway, ciParameters, ciMappingTable, ciView, ciFlowsheet, ciTermDefinition, ciBackgroundTask, ciGatewayAdmin);

  TConversionType = (ctCustom, ctDivision, ctMultiplication, ctSubtraction, ctAddition);

  TConversionMethod = (cmFahrenheit2Celsius, cmFahrenheight2Kelvin, cmCelsius2Fahrenheit, cmCelsius2Kelvin, cmKelvin2Fahrenheit, cmKelvin2Celsius);

  //TFlowsheetPageType = (ptMandatory, ptOptional, ptSupplemental);

  TFilerRecordType = (rtBattery, rtObservation);

  TObservationRange = (orUnknown, orNormal, orOutOfBoundsLow, orOutOfBoundsHigh, orLow, orHigh);

  TCliOHL7LogStatus = (lsUnknown, lsEntered, lsAwaitingProcessing, lsError, lsProcessed);

  TCliOHL7LogErrorType = (etNoPatient, etNoInstrument, etNoMappingTable, etOBXSegment);

  TCliOAlarmType = (atWithinRange, atNotWithinRange, atGreaterThan, atLessThan, atEqualsPickList, atBooleanTrue, atBooleanFalse);

  {Sets}
  TTermQualifiers = set of TTermType;

const
  CLIO_DATE_TIME_FORMAT = 'YYYY-MM-DD HH:MM:SS';

  BOOLEAN_AS_INTEGER: array[Boolean] of integer = (0, 1);

  TERMTYPE: array[TTermType] of string = ('Unknown', 'Observation Name', 'Unit of Measure', 'Method of Observation', 'Body Position', 'Anatomical Location', 'Quality of Measurement', 'Product', 'Null Reason', 'Task', 'Schedule', 'Coded Data Value', 'Concept');

  QUALNAME: array[TTermType] of string = ('Unknown', 'Observation', 'Unit', 'Method', 'Position', 'Location', 'Quality', 'Product', 'Null Reason', 'Task', 'Schedule', 'Coded Data', 'Concept');

  TERMTYPE_XMLTAG: array[TTermType] of string = ('UNKNOWN', 'TERM_ID', 'UNIT_ID', 'METHOD_ID', 'POSITION_ID', 'LOCATION_ID', 'QUALITY_ID', 'PRODUCT_ID', 'NULL_FLAVOR_ID', 'TASK', 'SCHEDULE', 'VALUE', 'CONCEPT');

  TERMTYPE_ABBREV: array[TTermType] of string = ('UNK', 'TRM', 'UNT', 'MTH', 'POS', 'LOC', 'QUA', 'PRO', 'NUL', 'TSK', 'SCH', 'VAL', 'CNC');

  TERMCHILDVALUETYPE: array[TTermChildValueType] of string = ('Extract', 'Delimited', 'Calculation');

  TERMVALUETYPE: array[TTermValueType] of string = ('Unspecified', 'Temperature', 'Length', 'Volume', 'Rate', 'Time', 'Mass', 'Scale');

  TERMQUALIFIERS: TTermQualifiers = [ttUnit, ttMethod, ttPosition, ttLocation, ttQuality, ttProduct];//, ttNullFlavor]; // Nothing used it and it just took up real-estate - drp 12-29-2010

  TERMFILTERS: TTermQualifiers = [ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor];

  TERMDEFINITIONPAIRS: TTermQualifiers = [ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor, ttConcept];

  TERMVALUEFORMAT: array[ttUnit..ttQuality] of string = ('%unit', '%method', '%position', '%location', '%quality');

  TERMDATATYPE: array[TTermDataType] of string = ('Qualifier Value', 'Complex', 'Numeric', 'PickList', 'Boolean', 'String', 'Function');

  TERMACTIVE: array[Boolean] of integer = (0, 1);

  TERMACTIVETEXT: array[Boolean] of string = ('No', 'Yes');

  CONTROLLER_NAME: array[TControllerType] of string = ('Terminology', 'Observation', 'VistA');

  //CONSOLE_ITEM_HEADER: array[TCPConsoleItemType] of string = ('Procedure', 'Instrument', 'Gateway', 'Parameters', 'Mapping Table', 'Observation View', 'Flowsheet', 'Terminology Definition', 'Background Tasks', 'Gateway Administrator');

  //CONSOLE_ITEM_NAME: array[TCPConsoleItemType] of string = ('Procedures', 'Instruments', 'Gateway Configuration', 'Parameters', 'Mapping Tables', 'Observation Views', 'Flowsheets', 'Terminology Definition', 'Background Tasks', 'Gateway Administrator');

  CONVERSION_METHOD_NAME: array[TConversionMethod] of string = ('Fahrenheit to Celsius', 'Fahrenheit to Kelvin', 'Celsius to Fahrenheit', 'Celsius to Kelvin', 'Kelvin to Fahrenheight', 'Kelvin to Celsius');

  CONVERSION_TYPE_NAME: array[TConversionType] of string = ('Custom', 'Division', 'Multiplication', 'Subtraction', 'Addition');

  //FLOWSHEET_PAGE_TYPE: array[TFlowsheetPageType] of string = ('Mandatory', 'Optional', 'Supplemental');

  OBSERVATION_RANGE: array[TObservationRange] of string = ('Unknown', 'Normal', 'Out Of Bounds Low', 'Out Of Bounds High', 'Low', 'High');

  OBSERVATION_RANGE_ORDER: array[0..2] of TObservationRange = (orLow, orNormal, orHigh);

  OBSERVATION_STATUS: array[TObservationRecordStatus] of string = ('Unverified', 'Verified', 'Archived', 'Purged', 'Corrected', 'Rescinded');

  OBSERVATION_AUDIT_TYPE: array[TObservationAuditType] of string = ('Status Change', 'Verification', 'Purged', 'Archived', 'Correction', 'Rescission');

  OBSERVATION_VIEW_STYLE: array[TObservationViewStyle] of string = ('Terminology List', 'Date/Time List');

  //OBSERVATION_VIEW_TYPE: array[TObservationViewType] of string = ('Flowsheet View', 'Report View');

  CLIO_HL7_LOG_STATUS: array[TCliOHL7LogStatus] of string = ('Unknown', 'Entered', 'Awaiting Processing', 'Error', 'Processed');

  CLIO_HL7_LOG_ERROR_TYPE: array[TCliOHL7LogErrorType] of string = ('No Patient', 'No Instrument', 'No Mapping Table', 'OBX Segment Error');

implementation

end.

