unit oDBController_CPConsole;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 29 $  $Modtime: 11/02/11 4:31p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  CPConsole Controller Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oDBController_CPConsole.pas $
================================================================================
}
interface

uses
  Classes,
  ComCtrls,
  oInterfaces,
  oCPTypes,
  oCliOTypes,
  oCP_Exceptions,
  oDBController,
  SysUtils,
  XMLIntf,
  XMLDoc;

type
  TDBController_CPConsole = class(TDBController, ICPService)
  private
    fPersonService: IPersonService;
    fInstruments: IInterfaceList;
    fProcedures: IInterfaceList;
    fBackgroundTasks: IInterfaceList;
    fFlowsheets: IInterfaceList;
    fFlowsheetTotals: IInterfaceList;
    fFlowsheetViews: IInterfaceList;
    fShifts: IInterfaceList;
    fSchedules: IInterfaceList;
    fTerminology: ITerminology;
    fACL: ICPAccessControlList;
    fAccessParameters: ICPParameterFile;

    {blj - 25 June 2010: Per request from Clin3 Manager, we're gonna keep users from
    adding new ConsoleItems if there's already a console item with that name in the list.}

    procedure CheckForDuplicateName(AList: IInterfaceList; ANewName: string);

  public
    constructor Create(aTerminology: ITerminology; aPersonService: IPersonService);
    destructor Destroy; override;

    function ExecuteDBCommand(aName: string; aParams: array of variant): boolean; stdcall;
    function ExecuteDBQuery(aName: string; aParams: array of variant; aXMLDocument: IXMLDocument): boolean; stdcall;

    procedure DBTransactionBegin; stdcall;
    procedure DBTransactionCommit; stdcall;
    procedure DBTransactionRollback; stdcall;

    function CPInstrument(aIndex: integer): ICPInstrument; overload; stdcall;
    function CPInstrument(aID: string): ICPInstrument; overload; stdcall;
    function CPInstrumentCount: integer; stdcall;

    function CPProcedure(aIndex: integer): ICPProcedure; overload; stdcall;
    function CPProcedure(aID: string): ICPProcedure; overload; stdcall;
    function CPProcedureCount: integer; stdcall;

    function CPBackgroundTask(aIndex: integer): ICPBackgroundTask; overload; stdcall;
    function CPBackgroundTask(aID: string): ICPBackgroundTask; overload; stdcall;
    function CPBackgroundTaskCount: integer; stdcall;

    function CPFlowsheet(aIndex: integer): ICPFlowsheet; overload; stdcall;
    function CPFlowsheet(aID: string): ICPFlowsheet; overload; stdcall;
    function CPFlowsheetCount: integer; stdcall;

    function CPShift(aIndex: integer): ICPShift; overload; stdcall;
    function CPShift(aID: string): ICPShift; overload; stdcall;
    function CPShiftCount: integer; stdcall;

    function CPSchedule(aIndex: integer): ICPSchedule; overload; stdcall;
    function CPSchedule(aID: string): ICPSchedule; overload; stdcall;
    function CPScheduleCount: integer; stdcall;

    function CPFlowsheetTotal(aIndex: integer): ICPFlowsheetTotal; overload; stdcall;
    function CPFlowsheetTotal(aID: string): ICPFlowsheetTotal; overload; stdcall;
    function CPFlowsheetTotalCount: integer; stdcall;

    function CPFlowsheetView(aIndex: integer): ICPFlowsheetView; overload; stdcall;
    function CPFlowsheetView(aID: string): ICPFlowsheetView; overload; stdcall;
    function CPFlowsheetViewCount: integer; stdcall;

    procedure Load; stdcall;
    procedure LoadACL; stdcall;
    procedure LoadItem(aItem: ICPConsoleItem; aForceLoad: boolean = False); stdcall;
    procedure LoadList(aList: TStrings; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;
    procedure LoadList(aListView: TListView; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;
    procedure LoadList(aTreeNode: TTreeNode; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;

    procedure LoadActiveSupplementalViews(aPatientID: string; aList: IInterfaceList); stdcall;
    procedure LoadAllSupplementalViews(aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aList: IInterfaceList); stdcall;

    {HL7 interface functions}
    procedure ResubmitHL7Message(aMessageID: string); stdcall;
    procedure DiscardHL7Message(aMessageID: string); stdcall;
    function MatchHL7MessageToPatient(aMessageID: string; aNewPTID: string): boolean; stdcall;
    function MatchHL7MessageToMappingTable(aMessageID: string; aNewMappingTableSourceID: string): boolean; stdcall;

    function NewCPEmailMessage: ICPEMailMessage; stdcall;

    {These need to move to a new interface that inherits the above items but is called ICPServiceManager}
    function NewConsoleItem(aItemName: string; aCPItemType: TCPItemType; aUser: IUser): ICPConsoleItem; stdcall;

    function CreateNewCPFlowsheetPage(aFlowsheet: ICPFlowsheet): ICPFlowsheetPage; stdcall;
    function CreateNewCPFlowsheetViewTerm(aView: ICPFlowsheetView): ICPFlowsheetViewTerm; stdcall;
    function CreateNewSet(aUserID, aComment: string): string; stdcall;
    function CreateNewSupplementalPage(aViewID, aPatientID, aMetID, aPosID, aLocID, aProID, aDisplayName, aUserID, aComment: string; aPageType: TCPFlowsheetPageType; aDateTimeOpened: TDateTime): string; stdcall;
    function DeactivateSupplementalPage(aID, aUserID, aDeactivationComment: string; aDateTimeClosed: TDateTime): boolean; stdcall;
    //function DeactivateSupplementalPage(aID, aUserID, aDeactivationComment: string): boolean; stdcall;

    procedure DeleteConsoleItem(aCPConsoleItem: ICPConsoleItem); stdcall;
    procedure RenameConsoleItem(aCPConsoleItem: ICPConsoleItem; aNewName: string); stdcall;
    procedure SaveConsoleItem(aItem: IInterface); stdcall;

    procedure ACLUpdate(aItem: ICPConsoleItem; aPerson: IPerson; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure ACLUpdate(aItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure ACLDelete(aItem: ICPConsoleItem; aPerson: IPerson); overload; stdcall;
    procedure ACLDelete(aItem: ICPAccessControlListItem); overload; stdcall;
    procedure ACLLoadList(aItem: ICPConsoleItem; aList: TStrings); stdcall;
    function ACLAccessLevel(aItem: ICPConsoleItem; aPerson: IPerson): TCPAccessLevel; stdcall;


    function getAccessParameters: ICPParameterFile;
  end;

implementation

uses
  //Dialogs,
  uCPXML,
  oCPShift,
  oCPFlowsheet,
  oCPFlowsheetTotal,
  oCPFlowsheetPage,
  oCPFlowsheetView,
  oCPFlowsheetViewTerm,
  oCPInstrument,
  oCPProcedure,
  oCPAccessControlListItem,
  oCPSchedule,
  oCPEMailMessage;

{ TDBController_CPConsole }

const USER_ACCESS = 'MD CONSOLE VISIBILITY';

constructor TDBController_CPConsole.Create(aTerminology: ITerminology; aPersonService: IPersonService);
begin
  inherited Create;
  fTerminology := aTerminology;
  fPersonService := aPersonService;
  TInterfaceList.Create.GetInterface(IInterfaceList, fInstruments);
  TInterfaceList.Create.GetInterface(IInterfaceList, fProcedures);
  TInterfaceList.Create.GetInterface(IInterfaceList, fBackgroundTasks);
  TInterfaceList.Create.GetInterface(IInterfaceList, fFlowsheets);
  TInterfaceList.Create.GetInterface(IInterfaceList, fFlowsheetTotals);
  TInterfaceList.Create.GetInterface(IInterfaceList, fFlowsheetViews);
  TInterfaceList.Create.GetInterface(IInterfaceList, fShifts);
  TInterfaceList.Create.GetInterface(IInterfaceList, fSchedules);


end;

destructor TDBController_CPConsole.Destroy;
begin
  fInstruments.Clear;
  fProcedures.Clear;
  fBackgroundTasks.Clear;
  fFlowsheets.Clear;
  fShifts.Clear;
  fFlowsheetTotals.Clear;
  fSchedules.Clear;
  fTerminology := nil;
  fPersonService := nil;
  inherited;
end;

function TDBController_CPConsole.NewConsoleItem(aItemName: string; aCPItemType: TCPItemType; aUser: IUser): ICPConsoleItem;
begin
  try
    case aCPItemType of
      ciProcedure:
        begin
          CheckForDuplicateName(fProcedures, aItemName);
          if NewCPProcedure(nil, Self).GetInterface(ICPConsoleItem, Result) then
            begin
              fProcedures.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end;
        end;

      ciInstrument:
        begin
          CheckForDuplicateName(fInstruments, aItemName);
          if NewCPInstrument(nil, Self).GetInterface(ICPConsoleItem, Result) then
            begin
              fInstruments.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end;
        end;

      ciFlowsheet:
        begin
          CheckForDuplicateName(fFlowsheets, aItemName);
          if NewCPFlowsheet(nil, Self, fTerminology).GetInterface(ICPConsoleItem, Result) then
            begin
              fFlowsheets.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end
          else
            raise ECPConsoleCreate.Create('Error creating new Flowsheet.');
        end;

      ciFlowsheetTotal:
        begin
          CheckForDuplicateName(fFlowsheetTotals, aItemName);        
          if NewCPFlowsheetTotal(nil, Self, fTerminology).GetInterface(ICPConsoleItem, Result) then
            begin
              fFlowsheetTotals.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end
          else
            raise ECPConsoleCreate.Create('Error creating new Flowsheet Total.');
        end;

      ciFlowsheetView:
        begin
          CheckForDuplicateName(fFlowsheetViews, aItemName);        
          if NewCPFlowsheetView(nil, Self, fTerminology).GetInterface(ICPConsoleItem, Result) then
            begin
              fFlowsheetViews.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end;
        end;

      ciShift:
        begin
          CheckForDuplicateName(fShifts, aItemName);        
          if NewCPShift(nil, Self).GetInterface(ICPConsoleItem, Result) then
            begin
              fShifts.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end
          else
            raise ECPConsoleCreate.Create('Error creating new Shift.');
        end;

      ciSchedule:
        begin
          CheckForDuplicateName(fSchedules, aItemName);        
          if NewCPSchedule(nil, Self).GetInterface(ICPConsoleItem, Result) then
            begin
              fSchedules.Add(Result);
              RenameConsoleItem(Result, aItemName);
              ACLUpdate(Result, aUser, alFullControl);
            end
          else
            raise ECPConsoleCreate.Create('Error creating new Schedule.');
        end;

    else
      raise ECPConsoleCreate.CreateFmt(
        'Error, cannot create system item ''%s''.',
        [CONSOLE_ITEM_HEADER[aCPItemType]]);
    end;
  except
    on E: Exception do
      begin
        Result := nil;
        raise;
      end;
  end;
end;

function TDBController_CPConsole.NewCPEmailMessage: ICPEMailMessage;
begin
  TCPEMailMessage.Create(DBConnection).GetInterface(ICPEMailMessage, Result);
end;

function TDBController_CPConsole.CreateNewSet(aUserID, aComment: string): string;
begin
  try
    Result := NewGUID;
    if not DBConnection.ExecCommand('AddSet',
      [Result, Now, aUserID, 1, aComment]) then
      raise Exception.CreateFmt('Error creating Set [%s].', [Result]);
  except
    on E: Exception do
      begin
        Result := '';
        raise ECPSaveFailure.Create(E.message);
      end;
  end;
end;

function TDBController_CPConsole.CreateNewSupplementalPage(aViewID, aPatientID,
  aMetID, aPosID, aLocID, aProID, aDisplayName, aUserID, aComment: string;
  aPageType: TCPFlowsheetPageType; aDateTimeOpened: TDateTime): string;
var
  aNewID: string;
  aSetID: string;
begin
  try
    DBConnection.BeginTransaction;
    if aPageType = ptSupplemental then
      aSetID := CreateNewSet(aUserID, aComment)
    else
      aSetID := '';
    aNewID := NewGUID;

    if not DBConnection.ExecCommand('AddSupplementalPage',
      [aNewID, aViewID, aPatientID, aMetID, aPosID, aLocID, aProID,
      aDisplayName, 1, aSetID, aDateTimeOpened, aUserID, aComment, aPageType]) then
      raise Exception.CreateFmt('Error creating supplemental page [%s].', [aDisplayName]);
    DBConnection.CommitTransaction;
    result := aNewID;
  except
    on E: Exception do
      begin
        DBConnection.RollbackTransaction;
        raise ECPSaveFailure.Create(E.message);
      end;
  end;
end;

function TDBController_CPConsole.DeactivateSupplementalPage(aID, aUserID, aDeactivationComment: string; aDateTimeClosed: TDateTime): boolean;
begin
  try
    DBConnection.BeginTransaction;
    if not DBConnection.ExecCommand('UpdateSupplementalPage',
      [aID, aDateTimeClosed, aUserID, aDeactivationComment, 0]) then
      raise Exception.CreateFmt('Error closing page [%s].', [aID]);
    DBConnection.CommitTransaction;
    result := true;
  except
    on E: Exception do
      begin
        DBConnection.RollbackTransaction;
        raise ECPSaveFailure.Create(E.message);
      end;
  end;
end;

{These two creates are needed because Page and ViewTerms aren't ICPConsole items}

function TDBController_CPConsole.CreateNewCPFlowsheetPage(aFlowsheet: ICPFlowsheet): ICPFlowsheetPage;
begin
  NewCPFlowsheetPage(nil, aFlowsheet, Self).GetInterface(ICPFlowsheetPage, Result);
end;

function TDBController_CPConsole.CreateNewCPFlowsheetViewTerm(aView: ICPFlowsheetView): ICPFlowsheetViewTerm;
begin
  NewCPFlowsheetViewTerm(nil, '').GetInterface(ICPFlowsheetViewTerm, Result);
end;

procedure TDBController_CPConsole.DeleteConsoleItem(aCPConsoleItem: ICPConsoleItem);
var
  i: integer;
begin
  try
    case aCPConsoleItem.getItemType of
      ciShift:
        begin
          i := fShifts.IndexOf(aCPConsoleItem);
          if i < 0 then
            raise ECPDeleteFailure.Create('No such shift found in CPConsole.');
          DBConnection.ExecCommand('DeleteShift', [aCPConsoleItem.getID]);
          ACLDelete(aCPConsoleItem, nil);
          fShifts.Delete(i);
        end;
      ciFlowsheet:
        begin
          i := fFlowsheets.IndexOf(aCPConsoleItem);
          if i < 0 then
            raise ECPDeleteFailure.Create('No such flowsheet found in CPConsole.');
          DBConnection.ExecCommand('DeleteFlowsheetTotalPairs', [aCPConsoleItem.getID]);
          DBConnection.ExecCommand('DeleteFlowsheetPagePairs', [aCPConsoleItem.getID]);
          DBConnection.ExecCommand('DeleteFlowsheet', [aCPConsoleItem.getID]);
          ACLDelete(aCPConsoleItem, nil);
          fFlowsheets.Delete(i);
        end;
      ciFlowsheetTotal:
        begin
          i := fFlowsheetTotals.IndexOf(aCPConsoleItem);
          if i < 0 then
            raise ECPDeleteFailure.Create('No such flowsheet total found in CPConsole.');
          DBConnection.ExecCommand('DeleteTotalFlowsheetPairs', [aCPConsoleItem.getID]);
          DBConnection.ExecCommand('DeleteTotalTerms', [aCPConsoleItem.getID]);
          DBConnection.ExecCommand('DeleteTotal', [aCPConsoleItem.getID]);
          ACLDelete(aCPConsoleItem, nil);
          fFlowsheetTotals.Delete(i);
        end;
    else
      raise ECPException.Create('This item cannot be deleted.');
    end;
  except
    raise;
  end;
end;

procedure TDBController_CPConsole.RenameConsoleItem(aCPConsoleItem: ICPConsoleItem; aNewName: string);
//var
//  i: integer;
begin
  try
    case aCPConsoleItem.getItemType of
      ciShift:
        begin
          CheckForDuplicateName(fShifts, aNewName);
          if fShifts.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such shift found in CPConsole.');
          DBConnection.ExecCommand('RenameShift', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciSchedule:
        begin
          CheckForDuplicateName(fSchedules, aNewName);
          if fSchedules.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such schedule found in CPConsole.');
          DBConnection.ExecCommand('RenameSchedule', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciProcedure:
        begin
          CheckForDuplicateName(fProcedures, aNewName);
          if fProcedures.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such procedure found in CPConsole.');
          DBConnection.ExecCommand('RenameProcedure', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciInstrument:
        begin
          CheckForDuplicateName(fInstruments, aNewName);
          if fInstruments.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such instrument found in CPConsole.');
          DBConnection.ExecCommand('RenameInstrument', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciFlowsheetView:
        begin
          CheckForDuplicateName(fFlowsheetViews, aNewName);
          if fFlowsheetViews.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such flowsheet found in CPConsole.');
          DBConnection.ExecCommand('RenameFlowsheetView', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciFlowsheet:
        begin
          CheckForDuplicateName(fFlowsheets, aNewName);
          if fFlowsheets.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such flowsheet found in CPConsole.');
          DBConnection.ExecCommand('RenameFlowsheet', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
      ciFlowsheetTotal:
        begin
          CheckForDuplicateName(fFlowsheetTotals, aNewName);
          if fFlowsheetTotals.IndexOf(aCPConsoleItem) < 0 then
            raise ECPRenameFailure.Create('No such flowsheet found in CPConsole.');
          DBConnection.ExecCommand('RenameFlowsheetTotal', [aCPConsoleItem.getID, aNewName]);
          aCPConsoleItem.setName(aNewName);
        end;
    else
      raise ECPException.Create('This item cannot be renamed.');
    end;
  except
    on E: Exception do
      raise ECPRenameFailure.Create('Error renaming CP Console Item.' + #13 + e.Message);
  end;
end;

procedure TDBController_CPConsole.DiscardHL7Message(aMessageID: string);
begin
  DBConnection.ExecCommand('ReSubmitHL7Message', [aMessageID, lsProcessed]);
end;

procedure TDBController_CPConsole.ResubmitHL7Message(aMessageID: string); stdcall;
begin
  DBConnection.ExecCommand('ReSubmitHL7Message', [aMessageID, lsAwaitingProcessing]);
end;

procedure TDBController_CPConsole.SaveConsoleItem(aItem: IInterface);
var
  aSchedule: ICPSchedule;
begin
  if aItem.QueryInterface(ICPSchedule, aSchedule) = 0 then
    with aSchedule do
      begin
        DBConnection.ExecCommand(
          'UpdateSchedule',
          [getID, getScheduleInterval, getDisplayName, getActive, getComment, getCustomSchedule])
      end
  else
    raise Exception.Create('CP Service unable to save item. Interface not accepted.');
end;

function TDBController_CPConsole.ExecuteDBCommand(aName: string; aParams: array of variant): boolean;
begin
  with DBConnection do
    try
      BeginTransaction;
      ExecCommand(aName, aParams);
      CommitTransaction;
      Result := True;
    except
      on E: Exception do
        begin
          RollbackTransaction;
          raise;
        end;
    end;
end;

function TDBController_CPConsole.ExecuteDBQuery(aName: string; aParams: array of variant; aXMLDocument: IXMLDocument): boolean;
begin
  with DBConnection do
    try
      BeginTransaction;
      ExecQuery(aName, aParams, aXMLDocument);
      CommitTransaction;
      Result := True;
    except
      on E: Exception do
        begin
          RollbackTransaction;
          raise;
        end;
    end;
end;

function TDBController_CPConsole.getAccessParameters: ICPParameterFile;
begin
  if DBConnection = nil then
    raise ECPLoadControllerFail.Create('DBConnection in DBController not defined.');
  if not Assigned(fAccessParameters) then
  begin
    fAccessParameters := NewCPParameterFile(DBConnection, USER_ACCESS);
  end;
  result := fAccessParameters;
end;

procedure TDBController_CPConsole.DBTransactionBegin;
begin
  DBConnection.BeginTransaction;
end;

procedure TDBController_CPConsole.DBTransactionCommit;
begin
  DBConnection.CommitTransaction;
end;

procedure TDBController_CPConsole.DBTransactionRollback;
begin
  DBConnection.RollbackTransaction;
end;

function TDBController_CPConsole.CPBackgroundTask(aIndex: integer): ICPBackgroundTask;
begin
  try
    fBackgroundTasks[aIndex].QueryInterface(ICPBackgroundTask, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.ACLAccessLevel(aItem: ICPConsoleItem; aPerson: IPerson): TCPAccessLevel;
var
  aUser: IUser;
begin
  if aPerson.QueryInterface(IUser, aUser) = 0 then
    if aUser.HasRole('MD ADMINISTRATOR') then
      Result := alFullControl
    else
      Result := fACL.ItemAccessLevel(aItem.getID, aUser.getDUZ)
  else
    Result := fACL.ItemAccessLevel(aItem.getID, aPerson.getID);
end;

procedure TDBController_CPConsole.ACLDelete(aItem: ICPConsoleItem; aPerson: IPerson);
begin
  if Assigned(aPerson) then
    fACL.Delete(aItem.getID, aPerson.getID)
  else
    fACL.DeleteAll(aItem.getID);
end;

procedure TDBController_CPConsole.ACLDelete(aItem: ICPAccessControlListItem);
begin
  fACL.Delete(aItem.getItemID, aItem.getUserID);
end;

procedure TDBController_CPConsole.ACLLoadList(aItem: ICPConsoleItem; aList: TStrings);
begin
  fACL.LoadList(aItem.getID, aList);
end;

procedure TDBController_CPConsole.ACLUpdate(aItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel);
begin
  fACL.Update(aItem.getItemID, aItem.getUserID, aAccessLevel);
end;

procedure TDBController_CPConsole.ACLUpdate(aItem: ICPConsoleItem; aPerson: IPerson; aAccessLevel: TCPAccessLevel);
begin
  fACL.Update(aItem.getID, aPerson.getID, aAccessLevel);
end;

procedure TDBController_CPConsole.CheckForDuplicateName(AList: IInterfaceList;
  ANewName: string);
var
  I: Integer;
  Item: ICPConsoleItem;
begin
  for I := 0 to AList.Count - 1 do
  begin
    if AList[I].QueryInterface(ICPConsoleItem, Item) = 0 then
      if Item.getName = ANewName then
        raise ECPConsoleCreate.CreateFmt('Name %S is a duplicate.', [ANewName]);
  end;
end;

function TDBController_CPConsole.CPBackgroundTask(aID: string): ICPBackgroundTask;
var
  i: integer;
begin
  for i := 0 to fBackgroundTasks.Count - 1 do
    if fBackgroundTasks[i].QueryInterface(ICPBackgroundTask, Result) = 0 then
      if Result.getTaskID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPBackgroundTaskCount: integer;
begin
  Result := fBackgroundTasks.Count;
end;

function TDBController_CPConsole.CPFlowsheet(aID: string): ICPFlowsheet;
var
  i: integer;
begin
  for i := 0 to fFlowsheets.Count - 1 do
    if fFlowsheets[i].QueryInterface(ICPFlowsheet, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPFlowsheet(aIndex: integer): ICPFlowsheet;
begin
  try
    fFlowsheets[aIndex].QueryInterface(ICPFlowsheet, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPFlowsheetCount: integer;
begin
  Result := fFlowsheets.Count;
end;

function TDBController_CPConsole.CPFlowsheetTotal(aIndex: integer): ICPFlowsheetTotal;
begin
  try
    fFlowsheetTotals[aIndex].QueryInterface(ICPFLowsheetTotal, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPFlowsheetTotal(aID: string): ICPFlowsheetTotal;
var
  i: integer;
begin
  for i := 0 to fFlowsheetTotals.Count - 1 do
    if CPFlowsheetTotal(i).getID = aID then
      begin
        CPFlowsheetTotal(i).QueryInterface(ICPFlowsheetTotal, Result);
        Exit;
      end;
  Result := nil;
end;

function TDBController_CPConsole.CPFlowsheetTotalCount: integer;
begin
  Result := fFlowsheetTotals.Count;
end;

function TDBController_CPConsole.CPFlowsheetView(aIndex: integer): ICPFlowsheetView;
begin
  try
    fFlowsheetViews[aIndex].QueryInterface(ICPFlowsheetView, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPFlowsheetView(aID: string): ICPFlowsheetView;
var
  i: integer;
begin
  for i := 0 to fFlowsheetViews.Count - 1 do
    if fFlowsheetViews[i].QueryInterface(ICPFlowsheetView, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPFlowsheetViewCount: integer;
begin
  Result := fFlowsheetViews.Count;
end;

function TDBController_CPConsole.CPInstrument(aIndex: integer): ICPInstrument;
begin
  try
    fInstruments[aIndex].QueryInterface(ICPInstrument, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPInstrument(aID: string): ICPInstrument;
var
  i: integer;
begin
  for i := 0 to fFlowsheets.Count - 1 do
    if fInstruments[i].QueryInterface(ICPInstrument, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPInstrumentCount: integer;
begin
  Result := fInstruments.Count;
end;

function TDBController_CPConsole.CPProcedure(aIndex: integer): ICPProcedure;
begin
  try
    fProcedures[aIndex].QueryInterface(ICPProcedure, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPProcedure(aID: string): ICPProcedure;
var
  i: integer;
begin
  for i := 0 to fProcedures.Count - 1 do
    if fProcedures[i].QueryInterface(ICPProcedure, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPProcedureCount: integer;
begin
  Result := fProcedures.Count;
end;

function TDBController_CPConsole.CPShift(aIndex: integer): ICPShift;
begin
  try
    fShifts[aIndex].QueryInterface(ICPShift, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPSchedule(aIndex: integer): ICPSchedule;
begin
  try
    fSchedules[aIndex].QueryInterface(ICPSchedule, Result);
  except
    Result := nil;
  end;
end;

function TDBController_CPConsole.CPSchedule(aID: string): ICPSchedule;
var
  i: integer;
begin
  for i := 0 to fSchedules.Count - 1 do
    if fSchedules[i].QueryInterface(ICPSchedule, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPScheduleCount: integer;
begin
  Result := fSchedules.Count;
end;

function TDBController_CPConsole.CPShift(aID: string): ICPShift;
var
  i: integer;
begin
  for i := 0 to fShifts.Count - 1 do
    if fShifts[i].QueryInterface(ICPShift, Result) = 0 then
      if Result.getID = aID then
        Exit;
  Result := nil;
end;

function TDBController_CPConsole.CPShiftCount: integer;
begin
  Result := fShifts.Count;
end;

procedure TDBController_CPConsole.Load;
var
  aItem: IInterface;
  aXMLDoc: IXMLDocument;
  aXMLNode: IXMLNode;
begin
  aXMLDoc := NewXMLDocument;

  {Get the instruments}
  OnNotifyControllerStatus('Loading Instruments', 0, 0);
  if not DBConnection.ExecQuery('GetInstrumentList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting instrument list');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPInstrument(aXMLNode, self).GetInterface(ICPInstrument, aItem) then
        fInstruments.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the procedure listing}
  OnNotifyControllerStatus('Loading Procedures', 0, 0);
  if not DBConnection.ExecQuery('GetProcedureList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting procedure list');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPProcedure(aXMLNode, Self).GetInterface(ICPProcedure, aItem) then
        fProcedures.Add(aItem)
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Background Tasks}
  OnNotifyControllerStatus('Loading Background Tasks', 0, 0);
  if not DBConnection.ExecQuery('GetBackgroundTaskList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting background task list');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPBackgroundTask(aXMLNode, DBConnection).QueryInterface(ICPConsoleItem, aItem) = 0 then
        fBackgroundTasks.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Flowsheets}
  OnNotifyControllerStatus('Loading Flowsheets', 0, 0);
  if not DBConnection.ExecQuery('GetFlowsheetList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting flowsheet list');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      NewCPFlowsheet(aXMLNode, Self, fTerminology).GetInterface(ICPConsoleItem, aItem);
      fFlowsheets.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Flowsheet Totals}
  OnNotifyControllerStatus('Loading Flowsheet Total Definitions', 0, 0);
  if not DBConnection.ExecQuery('GetTotalList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting Flowsheet Total List');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPFlowsheetTotal(aXMLNode, Self, fTerminology).GetInterface(ICPConsoleItem, aItem) then
        fFlowsheetTotals.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Flowsheet Views}
  OnNotifyControllerStatus('Loading Flowsheet View Definitions', 0, 0);
  if not DBConnection.ExecQuery('GetFlowsheetViewList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting Flowsheet View List');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPFlowsheetView(aXMLNode, Self, fTerminology).GetInterface(ICPConsoleItem, aItem) then
        fFlowsheetViews.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Shifts}
  OnNotifyControllerStatus('Loading Shifts', 0, 0);
  if not DBConnection.ExecQuery('GetShiftList', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting shift list');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPShift(aXMLNode, Self).GetInterface(ICPConsoleItem, aItem) then
        fShifts.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Get the Schedules}
  OnNotifyControllerStatus('Loading Schedules', 0, 0);
  if not DBConnection.ExecQuery('GetSchedules', [], aXMLDoc) then
    raise ECPLoadControllerFail.Create('Error getting schedules');

  aXMLDoc.Active := True;
  aXMLNode := XMLDocumentFirstRecord(aXMLDoc);
  while Assigned(aXMLNode) do
    try
      if NewCPSchedule(aXMLNode, Self).GetInterface(ICPConsoleItem, aItem) then
        fSchedules.Add(aItem);
    finally
      aXMLNode := XMLDocumentNextRecord(aXMLNode);
    end;

  {Load up the Access Control List}
  fACL := NewAccessControlList(DBConnection, fPersonService);
end;

procedure TDBController_CPConsole.LoadACL;
begin
  fACL.Load;
end;

procedure TDBController_CPConsole.LoadActiveSupplementalViews(aPatientID: string; aList: IInterfaceList);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aView: ICPFlowsheetView;
begin
  aList.Clear;
  try
    aXMLDoc := NewXMLDocument;
    DBConnection.ExecQuery('GetFlowsheetSupplementalPageListActive', [aPatientID], aXMLDoc);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while aXMLRec <> nil do
      try
        //ShowMessage('Loading Supplemental Page' + #13 + aXMLRec.XML);
        aView := NewCPFlowsheetView(aXMLRec, Self, fTerminology);
        aList.Add(aView);
        //ShowMessage('Loaded with Supp ID: ' + aView.getSupplementalID);
      finally
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;
    aView := nil; {Removes any danglers}
  except
    on E: Exception do
      raise ECPLoadFailure.Create('Error loading supplemental pages. Error: ' + E.Message)
  end;
end;

procedure TDBController_CPConsole.LoadAllSupplementalViews(aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aList: IInterfaceList);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
  aView: ICPFlowsheetView;
  i: integer;
  aAlreadyInList: boolean;
begin
  aList.Clear;
  try
    aXMLDoc := NewXMLDocument;
    DBConnection.ExecQuery('GetFlowsheetSupplementalPageListAll', [aPatientID, aStartDateTime, aStopDateTime], aXMLDoc);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    while aXMLRec <> nil do
      try
        //ShowMessage('Loading Supplemental Page' + #13 + aXMLRec.XML);
        aView := NewCPFlowsheetView(aXMLRec, Self, fTerminology);
        if aView.getPageType = ptOptional then {Only add optional view once}
          begin
            aAlreadyInList := False;
            for i := 0 to aList.Count - 1 do
              if (aList[i] as ICPFlowsheetView).getPageType = ptOptional then {Filter out supplementals drp/2011-07-18 10:19am}
                if (aList[i] as ICPFlowsheetView).getID = aView.getID then
                  begin
                    aAlreadyInList := True;
                    Break;
                  end;
            if not aAlreadyInList then {Note, it MUST be in the list as an optional now DRP/2011-07-18 10:19am}
              aList.Add(aView);
          end
        else
          aList.Add(aView);
      finally
        aXMLRec := XMLDocumentNextRecord(aXMLRec);
      end;
    aView := nil; {Removes any danglers}
  except
    on E: Exception do
      raise ECPLoadFailure.Create('Error loading supplemental pages. Error: ' + E.Message)
  end;
end;

procedure TDBController_CPConsole.LoadItem(aItem: ICPConsoleItem; aForceLoad: Boolean = False);
begin
  if (not aItem.getIsLoaded) or (aForceLoad) then
    begin
      // Load this item
    end;
end;

procedure TDBController_CPConsole.LoadList(aTreeNode: TTreeNode; aType: TCPItemType; aActiveOnly: boolean);
var
  i: integer;
  fList: IInterfaceList;
begin
  case aType of
    ciBackgroundTask: fList := fBackgroundTasks;
    ciFlowsheet: fList := fFlowsheets;
    ciFlowsheetTotal: fList := fFlowsheetTotals;
    ciFlowsheetView: fList := fFlowsheetViews;
    ciInstrument: fList := fInstruments;
    ciProcedure: fList := fProcedures;
    ciShift: fList := fShifts;
    ciSchedule: fList := fSchedules;
  else
    begin
    end;
  end;

  if Assigned(fList) then
    for i := 0 to fList.Count - 1 do
      with (fList[i] as ICPConsoleItem) do
        if (aActiveOnly and getActive) or (not aActiveOnly) then
          AddToTreeView(TTreeView(aTreeNode.TreeView), aTreeNode);
end;

function TDBController_CPConsole.MatchHL7MessageToMappingTable(aMessageID, aNewMappingTableSourceID: string): boolean;
begin
  with DBConnection do
    try
      BeginTransaction;
      ExecCommand('UpdateHL7MessageMappingTable', [aMessageID, aNewMappingTableSourceID]);
      CommitTransaction;
      Result := True;
    except
      begin
        RollbackTransaction;
        Result := False;
      end;
    end;
end;

function TDBController_CPConsole.MatchHL7MessageToPatient(aMessageID, aNewPTID: string): boolean;
begin
  with DBConnection do
    try
      BeginTransaction;
      ExecCommand('UpdateHL7MessagePatientID', [aMessageID, aNewPTID]);
      CommitTransaction;
      Result := True;
    except
      begin
        RollbackTransaction;
        Result := False;
      end;
    end;
end;

procedure TDBController_CPConsole.LoadList(aList: TStrings; aType: TCPItemType; aActiveOnly: boolean);
var
  i: integer;
  fList: IInterfaceList;
begin
  case aType of
    ciProcedure: fList := fProcedures;
    ciInstrument: fList := fInstruments;
    ciFlowsheet: fList := fFlowsheets;
    ciFlowsheetTotal: fList := fFlowsheetTotals;
    ciFlowsheetView: fList := fFlowsheetViews;
    ciBackgroundTask: fList := fBackgroundTasks;
    ciShift: fList := fShifts;
    ciSchedule: fList := fSchedules;
  else
    begin
    end;
  end;

  if Assigned(fList) then
    for i := 0 to fList.Count - 1 do
      with (fList[i] as ICPConsoleItem) do
        if (aActiveOnly and getActive) or (not aActiveOnly) then
          AddToList(aList);
end;

procedure TDBController_CPConsole.LoadList(aListView: TListView; aType: TCPItemType; aActiveOnly: boolean);
var
  i: integer;
  fList: IInterfaceList;
begin
  case aType of
    ciProcedure: fList := fProcedures;
    ciInstrument: fList := fInstruments;
    ciFlowsheet: fList := fFlowsheets;
    ciFlowsheetTotal: fList := fFlowsheetTotals;
    ciFlowsheetView: fList := fFlowsheetViews;
    ciBackgroundTask: fList := fBackgroundTasks;
    ciShift: fList := fShifts;
  else
    begin
    end;
  end;

  if aType = ciShift then
    aListView.Columns.Clear;

  if Assigned(fList) then
    for i := 0 to fList.Count - 1 do
      with (fList[i] as ICPConsoleItem) do
        if (aActiveOnly and getActive) or (not aActiveOnly) then
          AddToListView(aListView);
end;

end.

