unit oDBController_Fileman;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 10/05/06 9:55a $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Default Fileman style controller
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oDBController_Fileman.pas $
*
* $History: oDBController_Fileman.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:07p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Classes,
  Controls,
  Graphics,
  SysUtils,
  StdCtrls,
  ExtCtrls,
  oInterfaces,
  oDBController,
  oFilemanField;

type
  TFileManNotificationMethod = procedure (aMessage: string) of object;
  TDBController_FileMan = class (TDBController)
  private
    fCurrentField: TFilemanField;
    fDDNumber: string;
    fFieldColor: TColor;
    fIEN: string;
    fListOfComponents: TStringList;
    fLookupButton: TButton;
    OnNotifyProgress: TFileManNotificationMethod;
    procedure btnLookupOnClick(Sender: TObject);
    procedure DoNotifyProgress(aMessage: string);
    function getFileManField(aControl: TControl): TFilemanField;
  protected
    procedure TCheckBoxOnClick(Sender: TObject);
    procedure TComboBoxOnChange(Sender: TObject);
    procedure TEditOnChange(Sender: TObject);
    procedure TEditOnEnter(Sender: TObject);
    procedure TEditOnExit(Sender: TObject);
    procedure TRadioGroupOnClick(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    function RegisterComponent(aTCheckBox: TCheckBox; aFldNumber: string; aFldType: TFileManFieldType): Boolean; 
            overload; stdcall;
    function RegisterComponent(aTComboBox: TComboBox; aFldNumber: string; aFldType: TFileManFieldType): Boolean; 
            overload; stdcall;
    function RegisterComponent(aTEdit: TEdit; aFldNumber: string; aFldType: TFileManFieldType): Boolean; overload; 
            stdcall;
    function RegisterComponent(aTRadioGroup: TRadioGroup; aFldNumber: string; aFldType: TFileManFieldType): Boolean; 
            overload; stdcall;
    function RegisterForNotification(aFileManNotificationMethod: TFileManNotificationMethod): Boolean; stdcall;
    function SaveCurrentRecord: Boolean;
    procedure SetDDNumber(aDDNumber: string);
    procedure setFieldColor(const aValue: TColor); stdcall;
    procedure setRecord(aIEN: string); stdcall;
    procedure ShowWhoIsRegistered; stdcall;
    function UnRegisterAllComponents: Boolean; stdcall;
    function UnRegisterComponent(aControl: TControl): Boolean; stdcall;
  end;
  
function NewFilemanController: TDBController_FileMan;

implementation

uses
  fFileManLookup;

//**************************************************************************
//
//                          RECORD MANAGER FACTORY
//
//**************************************************************************

function NewFilemanController: TDBController_FileMan;
begin
  Result := TDBController_FileMan.Create;
end;

{ TFileManRecordManager }

{
************************************************ TDBController_FileMan *************************************************
}
constructor TDBController_FileMan.Create;
begin
  fFieldColor := clWindow;
  fListOfComponents := TStringList.Create;
  fLookupButton := TButton.Create(nil);
  fLookupButton.OnClick := btnLookupOnClick;
  OnNotifyProgress := DoNotifyProgress;
end;

destructor TDBController_FileMan.Destroy;
begin
  while fListOfComponents.Count > 0 do
    begin
      if Assigned(fListOfComponents.Objects[0]) then
        TFileManField(fListOfComponents.Objects[0]).Free;
      fListOfComponents.Objects[0] := nil;
      fListOfComponents.Delete(0);
    end;
  FreeAndNil(fListOfComponents);
  inherited;
end;

procedure TDBController_FileMan.btnLookupOnClick(Sender: TObject);
begin
  LookupPointer(DBConnection, fCurrentField, TEdit(fCurrentField.getControl).Text);
  fCurrentField.setDisplayValue(fCurrentField.getExternalValue);
end;

procedure TDBController_FileMan.DoNotifyProgress(aMessage: string);
begin
  // Placed here to prevent null pointers :)
end;

function TDBController_FileMan.getFileManField(aControl: TControl): TFilemanField;
var
  i: Integer;
begin
  i := fListOfComponents.IndexOf(aControl.Name);
  if i > -1 then
    Result := TFileManField(fListOfComponents.Objects[i])
  else
    Result := nil;
end;

function TDBController_FileMan.RegisterComponent(aTCheckBox: TCheckBox; aFldNumber: string; aFldType: 
        TFileManFieldType): Boolean;
begin
  if fListOfComponents.IndexOf(aTCheckBox.Name) < 1 then
    begin
      aTCheckBox.Color := fFieldColor;
      fListOfComponents.AddObject(aTCheckBox.Name, NewFileManField(aTCheckBox, fDDNumber, aFldNumber, aFldType));
      getFileManField(aTCheckBox).Init(DBConnection);
      aTCheckBox.OnClick := TCheckBoxOnClick;
    end;
  Result := True;
end;

function TDBController_FileMan.RegisterComponent(aTComboBox: TComboBox; aFldNumber: string; aFldType: 
        TFileManFieldType): Boolean;
begin
  if fListOfComponents.IndexOf(aTComboBox.Name) < 1 then
    begin
      aTComboBox.Color := fFieldColor;
      aTComboBox.Style := csDropDownList;
      aTComboBox.Sorted := True;
      fListOfComponents.AddObject(aTComboBox.Name, NewFileManField(aTComboBox, fDDNumber, aFldNumber, aFldType));
      getFileManField(aTComboBox).Init(DBConnection);
      aTComboBox.OnChange := TComboBoxOnChange;
    end;
  Result := True;
end;

function TDBController_FileMan.RegisterComponent(aTEdit: TEdit; aFldNumber: string; aFldType: TFileManFieldType): 
        Boolean;
begin
  if fListOfComponents.IndexOf(aTEdit.Name) < 1 then
    begin
      aTEdit.Color := fFieldColor;
      aTEdit.OnEnter := TEditOnEnter;
      aTEdit.OnChange := TEditOnChange;
      aTEdit.OnExit := TEditOnExit;
      fListOfComponents.AddObject(aTEdit.Name, NewFileManField(aTEdit, fDDNumber, aFldNumber, aFldType));
    end;
  Result := True;
end;

function TDBController_FileMan.RegisterComponent(aTRadioGroup: TRadioGroup; aFldNumber: string; aFldType: 
        TFileManFieldType): Boolean;
begin
  if fListOfComponents.IndexOf(aTRadioGroup.Name) < 1 then
    begin
      aTRadioGroup.Color := fFieldColor;
      fListOfComponents.AddObject(aTRadioGroup.Name, NewFileManField(aTRadioGroup, fDDNumber, aFldNumber, aFldType));
      getFileManField(aTRadioGroup).Init(DBConnection);
      aTRadioGroup.OnClick := TRadioGroupOnClick;
    end;
  Result := True;
end;

function TDBController_FileMan.RegisterForNotification(aFileManNotificationMethod: TFileManNotificationMethod): Boolean;
begin
  try
    OnNotifyProgress := aFileManNotificationMethod;
    Result := True;
  except
    Result := False;
  end;
end;

function TDBController_FileMan.SaveCurrentRecord: Boolean;
var
  i: Integer;
begin
  with DBConnection do
    try
      {Call without params to clear the decks}
      ExecStoredProc('ZZDRP FMAN', ['SETFDA'], nil);
  
      {Loop through the record and send down the values}
      for i := 0 to fListOfComponents.Count - 1 do
        with TFileManField(fListOfComponents.Objects[i]) do
          DBConnection.ExecStoredProc(
            'ZZDRP FMAN',
            ['SETFDA', getDDNumber, getIENs, getFieldNumber, getInternalValue], nil);
  
      Result := ExecStoredProc('ZZDRP FMAN', ['SAVEFDA'], nil);
    except
      Result := False;
    end;
end;

procedure TDBController_FileMan.SetDDNumber(aDDNumber: string);
begin
  fDDNumber := aDDNumber;
  setRecord('');
end;

procedure TDBController_FileMan.setFieldColor(const aValue: TColor);
begin
  fFieldColor := aValue;
end;

procedure TDBController_FileMan.setRecord(aIEN: string);
var
  i: Integer;
begin
  fIEN := aIEN;
  for i := 0 to fListOfComponents.Count - 1 do
    begin
      TFileManField(fListOfComponents.Objects[i]).setIEN(fIEN);
      TFileManField(fListOfComponents.Objects[i]).Load(DBConnection);
    end;
end;

procedure TDBController_FileMan.ShowWhoIsRegistered;
var
  i: Integer;
  txt: string;
begin
  for i := 0 to fListOfComponents.Count - 1 do
    txt := Format('%s' + #13 + '%s', [txt, TFileManField(fListOfComponents.Objects[i]).getDescription]);
  OnNotifyProgress('Current Component Listing' + #13 + txt);
end;

procedure TDBController_FileMan.TCheckBoxOnClick(Sender: TObject);
var
  aFld: TFilemanField;
begin
  OnNotifyProgress('Check Box Change');
  aFld := getFileManField(TControl(Sender));
  if Assigned(aFld) then
    with TCheckBox(Sender) do
      case Checked of
        True:
          begin
            aFld.setInternalValue('1');
            aFld.setExternalValue('1');
          end;
        False:
          begin
            aFld.setInternalValue('0');
            aFld.setExternalValue('0');
          end;
      end;
end;

procedure TDBController_FileMan.TComboBoxOnChange(Sender: TObject);
var
  aFld: TFilemanField;
begin
  OnNotifyProgress('Combo Box Change');
  aFld := getFileManField(TControl(Sender));
  if Assigned(aFld) then
    with TComboBox(Sender) do
      begin
        aFld.setInternalValue(TLookupValue(Items.Objects[ItemIndex]).InternalValue);
        aFld.setExternalValue(TLookupValue(Items.Objects[ItemIndex]).ExternalValue);
      end;
end;

procedure TDBController_FileMan.TEditOnChange(Sender: TObject);
begin
  if Assigned(fCurrentField) then
    begin
      case fCurrentField.getFieldType of
        ftText:
          begin
          end;
        ftPointer:
          begin
            if (not fLookupButton.Visible) or (fLookupButton.Parent <> TEdit(Sender)) then
              with fLookupbutton do
                begin
                  {Someday we need to deal with anchors}
                  Parent := TEdit(Sender).Parent;
                  BringToFront;
                  Height := TEdit(Sender).ClientHeight;
                  Width := Height;
                  Top := TEdit(Sender).Top + ((TEdit(Sender).Height - TEdit(Sender).ClientHeight) div 2);
                  Left := TEdit(Sender).Left + TEdit(Sender).Width - Width - ((TEdit(Sender).Height - TEdit(Sender).ClientHeight) div 2);
                  Caption := '...';
                  Visible := True;
                end;
          end;
        ftSet:
          begin
          end;
      end;
      fCurrentField.setChangeMade(True);
    end;
end;

procedure TDBController_FileMan.TEditOnEnter(Sender: TObject);
begin
  fCurrentField := getFileManField(TEdit(Sender));
  fCurrentField.setChangeMade(False);
  OnNotifyProgress('Entering ' + fCurrentField.getDescription);
end;

procedure TDBController_FileMan.TEditOnExit(Sender: TObject);
begin
  with fCurrentField do
    if getChangeMade then
      if getFieldType = ftPointer then
        begin
          if IsValid(DBConnection, TEdit(Sender).Text) then
            TEdit(Sender).Text := getExternalValue
          else
            begin
              btnLookupOnClick(Sender);
              TEdit(Sender).Text := getExternalValue;
              TEdit(Sender).SetFocus;
            end;
        end
      else
        if IsValid(DBConnection, TEdit(Sender).Text) then
          begin {Valid pointer value}
            TEdit(Sender).Text := getExternalValue;
            OnNotifyProgress(Format('You are now leaving %s', [getControl.ClassName]));
            OnNotifyProgress(Format('Internal: %s, External %s', [getInternalValue, getExternalValue]));
          end
        else
          begin {Invalid non-pointer}
            TEdit(Sender).Text := getExternalValue;
            TEdit(Sender).SetFocus;
          end;
  fLookupButton.Visible := False;
end;

procedure TDBController_FileMan.TRadioGroupOnClick(Sender: TObject);
var
  aFld: TFilemanField;
begin
  OnNotifyProgress('Combo Box Change');
  aFld := getFileManField(TControl(Sender));
  if Assigned(aFld) then
    with TRadioGroup(Sender) do
      begin
        aFld.setInternalValue(TLookupValue(Items.Objects[ItemIndex]).InternalValue);
        aFld.setExternalValue(TLookupValue(Items.Objects[ItemIndex]).ExternalValue);
      end;
end;

function TDBController_FileMan.UnRegisterAllComponents: Boolean;
begin
  while fListOfComponents.Count > 0 do
    begin
      TFileManField(fListOfComponents.Objects[0]).Free;
      fListOfComponents.Objects[0] := nil;
      fListOfComponents.Delete(0);
    end;
  Result := True;
end;

function TDBController_FileMan.UnRegisterComponent(aControl: TControl): Boolean;
var
  i: Integer;
begin
  i := fListOfComponents.IndexOf(aControl.Name);
  if i > -1 then
    begin
      TFileManField(fListOfComponents.Objects[i]).Free;
      fListOfComponents.Objects[i] := nil;
      fListOfComponents.Delete(i);
    end;
  Result := True;
end;

//**************************************************************************
//
//                        REGISTER/UNREGISTER
//
//**************************************************************************

//******************************************************************************
//                                                                            **
//                               TEDIT                                        **
//                                                                            **
//******************************************************************************

//******************************************************************************
//                                                                            **
//                               TComboBox                                    **
//                                                                            **
//******************************************************************************

//******************************************************************************
//                                                                            **
//                               TRadioGroup                                  **
//                                                                            **
//******************************************************************************

//******************************************************************************
//                                                                            **
//                               TRadioGroup                                  **
//                                                                            **
//******************************************************************************

end.

