unit oInterfaces;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 56 $  $Modtime: 11/02/11 4:32p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Primary interface collection for CliO based applications.
*
*       Notes:        NEEDS TO BE RENAMED SOMEDAY - MAJOR IMPACT ON LOTS OF
*                     CODE.
*
*
================================================================================
*       $Archive: /CP Flowsheets/CliO/source/Common/objects/oInterfaces.pas $
================================================================================
}
interface

uses
  oCliOTypes,
  oCPTypes,
  uCCOW,
  ComCtrls,
  Controls,
  Graphics,
  SysUtils,
  Classes,
  Grids,
  XMLDoc,
  XMLIntf,
  IniFiles;

type
  ICPBackgroundTask = interface;
  ICPConsoleItem = interface;
  ICPConsoleCopy = interface; // zzzzzzandria 2011-10-03
  ICPAccessControlList = interface;
  ICPAccessControlListItem = interface;
  ICPEditor = interface;
  ICPEMailMessage = interface;

  ICPFlowsheet = interface;
  ICPFlowsheetPage = interface;
  ICPFlowsheetTotal = interface;
  ICPFlowsheetView = interface;
  ICPFlowsheetViewTerm = interface;

  ICPGatewayConfig = interface;
  ICPInstrument = interface;
  ICPParameterFile = interface;
  ICPUserOptions = interface;
  ICPProcedure = interface;
  ICPTransaction = interface;
  ICPLocation = interface;
  ICPShift = interface;
  ICPSchedule = interface;
  ICPTIUDocument = interface;
  ICPReport = interface;

  ICPService = interface;
  ICPKardexService = interface;
  ICPKardexAction = interface;
  ICPKardexEvent = interface;
  ICPKardexAudit = interface;

  ICPGUILoader = interface;

  IDBConnection = interface;

  IDocumentor = interface;
  ILegacyBroker = interface;
  ILookupItem = interface;

  IObservation = interface;

  IPatient = interface;
  IPerson = interface;
  IRangeCheck = interface;

  ITerm = interface;
  ITermChild = interface;
  ITermConversion = interface;
  ITermPair = interface;
  ITermUpdate = interface;
  ITermEditorFrame = interface;
  ITerminology = interface;
  ITerminologyEditor = interface;
  ITermLoader = interface;
  ITermMapper = interface;
  ITermRangeCheck = interface;
  ITermUnit = interface;
  IUser = interface;
  IVistASystem = interface;

  ETermValidationError = class(Exception);
  ETermConversionError = class(Exception);

  //TManagedInterfacedObject = class;
  //TUnmanagedInterfacedObject = class;

  TLookupItem = class;

  //TCPConsoleRename = procedure(aNewName: string) of object;
  TNotifyOnProgress = procedure(const aMsg: string; aProgress, aProgressMax: integer) of object;

  ICPTransaction = interface(IInterface)
    ['{A6248E7F-7D91-420D-8FD9-DC0E943AC05B}']
    procedure getConsultID; stdcall;
    function getCPProcedure: ICPProcedure; stdcall;
    function getCreatedBy: string; stdcall;
    function getCreatedDateTime: TDateTime; stdcall;
    function getInstrument: ICPInstrument; stdcall;
    function getPatient: IPatient; stdcall;
    function getTIUNoteID: string; stdcall;
    function getTIUNoteTitle: string; stdcall;
  end;

  ICPLocation = interface(IInterface)
    ['{6779189C-5991-4FC9-9447-E327D0012671}']
    function getID: string; stdcall;
    function getType: TCPLocationType; stdcall;
    function getName: string; stdcall;
    function getDivisionID: string; stdcall;
    function getDivisionNumber: string; stdcall;
    function getDivisionName: string; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
  end;

  ICPConsoleItem = interface(IInterface)
    ['{D6E60FBF-1F00-4F0D-B6C3-798A3D985D49}']
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getCanDelete: boolean; stdcall;
    function getCanRename: boolean; stdcall;
    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;

    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    procedure Load; stdcall;
  end;

    ICPConsoleCopy  = interface(ICPConsoleItem) // zzzzzzandria 2011-10-03
    ['{828AC757-32AC-4A70-962C-C56A1644277B}']
      procedure CopyFrom(aID: string); stdcall;
      procedure Save; stdcall;
    end;

  ICPAccessControlList = interface(IInterface)
    ['{A2CA1793-26D3-4C9C-AE20-C01A8198B40B}']
    function Item(aIndex: Integer): ICPAccessControlListItem; stdcall;
    function ItemCount: integer; stdcall;
    function ItemAccessLevel(aItemID, aUserID: string): TCPAccessLevel; stdcall;

    procedure Load; stdcall;
    procedure LoadList(aItemID: string; aList: TStrings); stdcall;
    procedure LoadListView(aItemID: string; aListView: TListView); stdcall;

    procedure Update(aItemID, aUserID: string; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure Update(aACLItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure Delete(aItemID, aUserID: string); stdcall;
    procedure DeleteAll(aItemID: string); stdcall;
  end;

  ICPAccessControlListItem = interface(IInterface)
    ['{B9FBFD5E-D341-442D-9AE2-C2CF59F534E7}']
    function getItemID: string; stdcall;
    function getUserID: string; stdcall;
    function getAccessLevel: TCPAccessLevel; stdcall;

    procedure setItemID(const aValue: string); stdcall;
    procedure setUserID(const aValue: string); stdcall;
    procedure setAccessLevel(const aValue: TCPAccessLevel); stdcall;

    function AddToList(aList: TStrings; aCaption: string): integer; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
  end;

  ICPGUILoader = interface(IInterface)
    ['{4F9F3084-9CC9-4BB2-98D2-EAD430A18B15}']
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
  end;

  ICPEditor = interface(IInterface)
    ['{CF6AEF6C-7D94-4018-83AA-35F9E03A156F}']
    procedure Initialize(aItem: IInterface); stdcall;
    procedure Save; stdcall;
    procedure Validate(var aErrorLog: TStringList); stdcall;
    procedure SetReadOnly(const aValue: boolean); stdcall;
  end;

  ICPEMailMessage = interface(IInterface)
    ['{F357F776-2117-45FC-B73F-1AA516C78A28}']
    procedure SetSubject(const aValue: string); stdcall;
    procedure AddRecipient(const aValue: string); stdcall;
    procedure AddText(const aValue: string); overload; stdcall;
    procedure AddText(const aValue: TStrings); overload; stdcall;
    procedure AddList(const aValue: TStrings); stdcall;

    procedure Send; stdcall;
  end;

  ICPBackgroundTask = interface(IInterface)
    ['{7F5B7312-3B27-4DE2-9D24-35DF973E7C00}']
    function getTaskName: string; stdcall;
    function getTaskID: string; stdcall;
    function getTaskStatus: string; stdcall;
    function getTaskRecurrence: integer; stdcall;
    function getDescription: string; stdcall;
    function getIsScheduled: boolean; stdcall;
    function getScheduledAt: TDateTime; stdcall;
    function getSystemID: string; stdcall;
    procedure Load; stdcall;
    procedure Start(aStartTime: TDateTime); stdcall;
    procedure Stop; stdcall;
  end;

  ICPInstrument = interface(ICPConsoleItem)
    ['{7FEC2FAF-6E08-4323-A35E-2C3EEAFF6195}']
    function getPrintableName: string; stdcall;
    function getSerialNumber: string; stdcall;
    function getNotificationMailgroup: string; stdcall;
    function getNotificationMailgroupID: string; stdcall;
    function getDeleteWhenSubmitted: boolean; stdcall;
    function getDefaultFileExtension: string; stdcall;
    function getProcessingRoutine: string; stdcall;
    function getProcessingCode: string; stdcall;
    function getProcessUNC: boolean; stdcall;
    function getProcessText: boolean; stdcall;
    function getProcessURL: boolean; stdcall;
    function getProcessDLL: boolean; stdcall;
    function getProcessUUEncode: boolean; stdcall;
    function getProcessXML: boolean; stdcall;
    function getProcessXMS: boolean; stdcall;
    function getBiDirectional: boolean; stdcall;
    function getIPAddress: string; stdcall;
    function getPort: string; stdcall;
    function getHL7InstrumentID: string; stdcall;
    function getHL7UniversalServiceID: string; stdcall;
    function getHL7LogicalLink: string; stdcall;
    function getHL7LogicalLinkID: string; stdcall;

    procedure setPrintableName(const aValue: string); stdcall;
    procedure setSerialNumber(const aValue: string); stdcall;
    procedure setNotificationMailgroup(const aValue: string); stdcall;
    procedure setNotificationMailgroupID(const aValue: string); stdcall;
    procedure setDeleteWhenSubmitted(const aValue: boolean); stdcall;
    procedure setDefaultFileExtension(const aValue: string); stdcall;
    procedure setProcessingRoutine(const aValue: string); stdcall;
    procedure setProcessingCode(const aValue: string); stdcall;
    procedure setProcessUNC(const aValue: boolean); stdcall;
    procedure setProcessText(const aValue: boolean); stdcall;
    procedure setProcessURL(const aValue: boolean); stdcall;
    procedure setProcessDLL(const aValue: boolean); stdcall;
    procedure setProcessUUEncode(const aValue: boolean); stdcall;
    procedure setProcessXML(const aValue: boolean); stdcall;
    procedure setProcessXMS(const aValue: boolean); stdcall;
    procedure setBiDirectional(const aValue: boolean); stdcall;
    procedure setIPAddress(const aValue: string); stdcall;
    procedure setPort(const aValue: string); stdcall;
    procedure setHL7InstrumentID(const aValue: string); stdcall;
    procedure setHL7UniversalServiceID(const aValue: string); stdcall;
    procedure setHL7LogicalLink(const aValue: string); stdcall;
    procedure setHL7LogicalLinkID(const aValue: string); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  end;

  ICPGatewayConfig = interface(IInterface)
    ['{836FACF0-0233-4DFF-9284-1308FE03173D}']
    // Accessor functions
    // Stored locally in the registry of the machine running
    // CP Console.
    function getBrokerServerIP: string; stdcall;
    function getBrokerServerPort: integer; stdcall;
    function getAccessCode: string; stdcall;
    function getVerifyCode: string; stdcall;
    function getIsGatewayInstalled: boolean; stdcall;
    // Stored in the KERNEL SITE PARAMETERS file
    function getGatewayServerIP: string; stdcall;
    function getGatewayServerPort: integer; stdcall;
    function getGatewayServerHint: string; stdcall;
    function getDaysToKeepData: integer; stdcall;
    function getLogLevel: integer; stdcall;
    function getLogFileDirectory: string; stdcall;
    function getSharedDirectory: string; stdcall;
    function getEncryptedAccess: string; stdcall;
    function getEncryptedVerify: string; stdcall;
    function getDataRetrievalType: TDataRetrievalType; stdcall;
    function getPollingInterval: integer; stdcall;

    // Mutator functions
    // Stored locally in the registry of the machine running
    // CP Console.
    procedure setAccessCode(const AValue: string); stdcall;
    procedure setVerifyCode(const AValue: string); stdcall;
    procedure setBrokerServerIP(const AValue: string); stdcall;
    procedure setBrokerServerPort(const AValue: integer); stdcall;
    // Stored in the KERNEL SITE PARAMETERS file
    procedure setGatewayServerIP(const AValue: string); stdcall;
    procedure setGatewayServerPort(const AValue: integer); stdcall;
    procedure setGatewayServerHint(const AValue: string); stdcall;
    procedure setDaysToKeepData(const aValue: integer); stdcall;
    procedure setLogLevel(const AValue: integer); stdcall;
    procedure setLogFileDirectory(const AValue: string); stdcall;
    procedure setSharedDirectory(const AValue: string); stdcall;
    procedure setEncryptedAccess(const AValue: string); stdcall;
    procedure setEncryptedVerify(const AValue: string); stdcall;
    procedure setDataRetrievalType(const AValue: TDataRetrievalType); stdcall;
    procedure setPollingInterval(const AValue: integer); stdcall;
    procedure Save; stdcall;
    procedure Load; stdcall;

    function getDBConnection: IDBConnection; stdcall;
  end;

  ICPProcedure = interface(ICPConsoleItem)
    ['{F915FFC8-47EE-439E-A67D-E9FCCB18C22F}']
    function getTreatingSpecialty: string; stdcall;
    function getTreatingSpecialtyID: string; stdcall;
    function getHospitalLocation: string; stdcall;
    function getHospitalLocationID: string; stdcall;
    function getDefaultTIUNote: string; stdcall;
    function getDefaultTIUNoteID: string; stdcall;
    function getDescription: string; stdcall;
    function getRequireExternalData: boolean; stdcall;
    function getAutoSubmit: boolean; stdcall;
    function getHighVolume: boolean; stdcall;
    function getProcessingApplication: TCPProcessingApplication; stdcall;
    function getProcessedResult: TCPProcessedResult; stdcall;
    function getExternalDataDirectory: string; stdcall;

    function getInstrumentID(aIndex: integer): string; stdcall;
    function getInstrumentIDCount: integer; stdcall;
    function getInstrumentAssigned(aID: string): boolean; stdcall;
    procedure AddInstrument(const aInstrumentID: string); stdcall;
    procedure ClearInstruments; stdcall;

    procedure setTreatingSpecialty(const aValue: string); stdcall;
    procedure setTreatingSpecialtyID(const aValue: string); stdcall;
    procedure setHospitalLocation(const aValue: string); stdcall;
    procedure setHospitalLocationID(const aValue: string); stdcall;
    procedure setDefaultTIUNote(const aValue: string); stdcall;
    procedure setDefaultTIUNoteID(const aValue: string); stdcall;
    procedure setDescription(const aValue: string); stdcall;
    procedure setRequireExternalData(const aValue: boolean); stdcall;
    procedure setAutoSubmit(const aValue: boolean); stdcall;
    procedure setHighVolume(const aValue: boolean); stdcall;
    procedure setProcessingApplication(const aValue: TCPProcessingApplication); stdcall;
    procedure setProcessedResult(const aValue: TCPProcessedResult); stdcall;
    procedure setExternalDataDirectory(const aValue: string); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  end;

  ICPParameterFile = interface(IInterface)
    ['{E4C7580F-BAFD-4223-9959-96F608F2F85C}']
    function getApplicationKey: string; stdcall;
    procedure setApplicationKey(aKey: string); stdcall;
    procedure LoadValues(aKey: TCPParamKey; aList: TStrings); stdcall;
    procedure ClearValues(aKey: TCPParamKey); stdcall;

    function ReadDateTime(aKey: TCPParamKey; aParam: string; aDefault: TDateTime): TDateTime; stdcall;
    function ReadBoolean(aKey: TCPParamKey; aParam: string; aDefault: boolean = False): Boolean; stdcall;
    function ReadInteger(aKey: TCPParamKey; aParam: string; aDefault: integer = 0): integer; stdcall;
    function ReadDouble(aKey: TCPParamKey; aParam: string; aDefault: double = 0.0): double; stdcall;
    function ReadString(aKey: TCPParamKey; aParam: string; aDefault: string = ''): string; overload; stdcall;
    function ReadString(aKey: string; aParam: string; aDefault: string = ''): string; overload; stdcall;

    procedure WriteDateTime(aKey: TCPParamKey; aParam: string; aValue: TDateTime); stdcall;
    procedure WriteBoolean(aKey: TCPParamKey; aParam: string; aValue: boolean); stdcall;
    procedure WriteInteger(aKey: TCPParamKey; aParam: string; aValue: integer); stdcall;
    procedure WriteDouble(aKey: TCPParamKey; aParam: string; aValue: double); stdcall;
    procedure WriteString(aKey: TCPParamKey; aParam: string; aValue: string); overload; stdcall;
    procedure WriteString(aKey: string; aParam: string; aValue: string); overload; stdcall;
  end;

  ICPService = interface(IInterface)
    ['{73A2B841-F74C-4D34-969B-E57AD970B051}']
    function Initialize(aName: string; aDBConnection: IDBConnection): Boolean; overload; stdcall;
    function Initialize(aName: string; aDBType: integer; aParams: array of Variant): Boolean; overload; stdcall;
    function Initialize(aName: string; aXMLDocument: IXMLDocument): Boolean; overload; stdcall;

    function ExecuteDBCommand(aName: string; aParams: array of variant): boolean; stdcall;
    function ExecuteDBQuery(aName: string; aParams: array of variant; aXMLDocument: IXMLDocument): boolean; stdcall;

    procedure DBTransactionBegin; stdcall;
    procedure DBTransactionCommit; stdcall;
    procedure DBTransactionRollback; stdcall;

    function CPInstrument(aIndex: integer): ICPInstrument; overload; stdcall;
    function CPInstrument(aID: string): ICPInstrument; overload; stdcall;
    function CPInstrumentCount: integer; stdcall;

    function CPProcedure(aIndex: integer): ICPProcedure; overload; stdcall;
    function CPProcedure(aID: string): ICPProcedure; overload; stdcall;
    function CPProcedureCount: integer; stdcall;

    function CPBackgroundTask(aIndex: integer): ICPBackgroundTask; overload; stdcall;
    function CPBackgroundTask(aID: string): ICPBackgroundTask; overload; stdcall;
    function CPBackgroundTaskCount: integer; stdcall;

    function CPFlowsheet(aIndex: integer): ICPFlowsheet; overload; stdcall;
    function CPFlowsheet(aID: string): ICPFlowsheet; overload; stdcall;
    function CPFlowsheetCount: integer; stdcall;

    function CPShift(aIndex: integer): ICPShift; overload; stdcall;
    function CPShift(aID: string): ICPShift; overload; stdcall;
    function CPShiftCount: integer; stdcall;

    function CPSchedule(aIndex: integer): ICPSchedule; overload; stdcall;
    function CPSchedule(aID: string): ICPSchedule; overload; stdcall;
    function CPScheduleCount: integer; stdcall;

    function CPFlowsheetTotal(aIndex: integer): ICPFlowsheetTotal; overload; stdcall;
    function CPFlowsheetTotal(aID: string): ICPFlowsheetTotal; overload; stdcall;
    function CPFlowsheetTotalCount: integer; stdcall;

    function CPFlowsheetView(aIndex: integer): ICPFlowsheetView; overload; stdcall;
    function CPFlowsheetView(aID: string): ICPFlowsheetView; overload; stdcall;
    function CPFlowsheetViewCount: integer; stdcall;

    procedure Load; stdcall;
    (*
    procedure LoadACL; stdcall;
    *)
    procedure LoadList(aList: TStrings; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;
    procedure LoadList(aListView: TListView; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;
    procedure LoadList(aTreeNode: TTreeNode; aType: TCPItemType; aActiveOnly: boolean = True); overload; stdcall;
    procedure LoadActiveSupplementalViews(aPatientID: string; aList: IInterfaceList); stdcall;
    procedure LoadAllSupplementalViews(aPatientID: string; aStartDateTime, aStopDateTime: TDateTime; aList: IInterfaceList); stdcall;

    function CreateNewCPFlowsheetPage(aFlowsheet: ICPFlowsheet): ICPFlowsheetPage; stdcall;
    function CreateNewCPFlowsheetViewTerm(aView: ICPFLowsheetView): ICPFlowsheetViewTerm; stdcall;
    function CreateNewSet(aUserID, aComment: string): string; stdcall;
    function CreateNewSupplementalPage(aViewID, aPatientID, aMetID, aPosID,
      aLocID, aProID, aDisplayName, aUserID, aComment: string;
      aPageType: TCPFlowsheetPageType; aDateTimeOpened: TDateTime): string; stdcall;
    function DeactivateSupplementalPage(aID, aUserID, aDeactivationComment: string; aDateTimeClosed: TDateTime): boolean; stdcall;

    function NewConsoleItem(aItemName: string; aCPItemType: TCPItemType; aUser: IUser): ICPConsoleItem; stdcall;
    procedure DeleteConsoleItem(aCPConsoleItem: ICPConsoleItem); stdcall;
    procedure RenameConsoleItem(aCPConsoleItem: ICPConsoleItem; aNewName: string); stdcall;
    procedure SaveConsoleItem(aItem: IInterface); stdcall;

    function NewCPEmailMessage: ICPEMailMessage; stdcall;

    {HL7 interface stuff}
    procedure ResubmitHL7Message(aMessageID: string); stdcall;
    procedure DiscardHL7Message(aMessageID: string); stdcall;
    function MatchHL7MessageToPatient(aMessageID: string; aNewPTID: string): boolean; stdcall;
    function MatchHL7MessageToMappingTable(aMessageID: string; aNewMappingTableSourceID: string): boolean; stdcall;

    (*
    blj 30 September 2011: CQ 680 - We're removing the ACL functionality completely.
      Instead, we'll be storing parameters in XPAR USER that will allow access to
      individual folders within CP Console.  This access will be all or nothing:
      there will be no read only/read write.  If you can see it, you own it.

    procedure ACLUpdate(aItem: ICPConsoleItem; aPerson: IPerson; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure ACLUpdate(aItem: ICPAccessControlListItem; aAccessLevel: TCPAccessLevel); overload; stdcall;
    procedure ACLDelete(aItem: ICPConsoleItem; aPerson: IPerson); overload; stdcall;
    procedure ACLDelete(aItem: ICPAccessControlListItem); overload; stdcall;
    procedure ACLLoadList(aItem: ICPConsoleItem; aList: TStrings); stdcall;
    function ACLAccessLevel(aItem: ICPConsoleItem; aPerson: IPerson): TCPAccessLevel; stdcall;
    *)
    {Folder permissions stuff}
    function getAccessParameters: ICPParameterFile;
  end;

  ICPKardexService = interface(IInterface)
    ['{06755E5E-0E5D-4DC9-96FA-D7148D6489B4}']
    function Initialize(aName: string; aDBConnection: IDBConnection): Boolean; overload; stdcall;
    function Initialize(aName: string; aDBType: integer; aParams: array of Variant): Boolean; overload; stdcall;
    function Initialize(aName: string; aXMLDocument: IXMLDocument): Boolean; overload; stdcall;

    function getAction(aIndex: integer): ICPKardexAction; overload; stdcall;
    function getAction(aID: string): ICPKardexAction; overload; stdcall;
    function getActionCount: integer; stdcall;
    function getNewAction(aPatient: IPatient): ICPKardexAction; stdcall;

    function getError(aIndex: integer): string; stdcall;
    function getErrorCount: integer; stdcall;

    procedure DiscontinueAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aComment: string); stdcall;
    procedure HoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aHoldStartDate, aHoldStopDate: TDateTime; aComment: string); stdcall;
    procedure HoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aHoldDate: TDateTime; aComment: string); stdcall;
    procedure UnHoldAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aUnHoldDate: TDateTime; aComment: string); stdcall;
    procedure UnHoldEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aUnHoldDate: TDateTime; aComment: string); stdcall;
    procedure CompleteEvent(aCPKardexAction: ICPKardexAction; aUser: IUser; aDateTime: TDateTIme; aComment: string); overload; stdcall;
    procedure CompleteEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTIme; aComment: string); overload; stdcall;

    procedure Load(aPatient: IPatient; aStartDateTime, aStopDateTime: TDateTime); stdcall;
    procedure LoadActions(aListView: TListView); overload; stdcall;
    procedure LoadActions(aList: TStrings; aActiveOnly: boolean); overload; stdcall;
    procedure LoadAudits(aAction: ICPKardexAction; aList: IInterfaceList); overload; stdcall;
    procedure LoadAudits(aEvent: ICPKardexEvent; aList: IInterfaceList); overload; stdcall;

    procedure InsertAction(aCPKardexAction: ICPKardexAction); stdcall;
    procedure UpdateAction(aCPKardexAction: ICPKardexAction); stdcall;
    procedure RenewAction(aCPKardexAction: ICPKardexAction; aUser: IUser; aNewStopDateTime: TDateTime); stdcall;

    procedure UpdateEvent(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDateTime: TDateTime; aStatus: TCPKardexEventStatus; aComment: string); stdcall;
    procedure UpdateAudit(aCPKardexAction: ICPKardexAction; aUser: IUser; aDescription: string; aComment: string); overload; stdcall;
    procedure UpdateAudit(aCPKardexEvent: ICPKardexEvent; aUser: IUser; aDescription: string; aComment: string); overload; stdcall;
  end;

  ICPKardexAction = interface(IInterface)
    ['{D349F4EC-0134-40D4-BF07-97EEA68271D0}']
    function getID: string; stdcall;
    function getCareAction: ITerm; stdcall;
    function getCareActionID: string; stdcall;
    function getSchedule: ICPSchedule; stdcall;
    function getScheduleID: string; stdcall;
    function getScheduleDisplay: string; stdcall;
    function getPatient: IPatient; stdcall;
    function getStartDateTime: TDateTime; stdcall;
    function getStopDateTime: TDateTime; stdcall;
    function getFirstTime: string; stdcall;
    function getFirstEvent: ICPKardexEvent; stdcall;
    function getScheduleType: TCPScheduleType; stdcall;
    function getSchedulePRNType: TCPScheduleTypePRN; stdcall;
    function getStatus: TCPKardexActionStatus; stdcall;
    function getSpecialInstructions: string; stdcall;
    function getPRNNTE: integer; stdcall;
    function getScheduleTimes: string; stdcall;
    function getDuration: integer; stdcall;
    function getActive: boolean; stdcall;

    function getEventCount: integer; stdcall;
    function getEvent(aIndex: integer): ICPKardexEvent; stdcall;
    function getLastCompletedEvent: ICPKardexEvent; stdcall;

    procedure AddEvent(aCPKardexEvent: ICPKardexEvent); stdcall;
    procedure ClearEvents; stdcall;

    procedure setTask(const aValue: ITerm); stdcall;
    procedure setSchedule(const aValue: ICPSchedule); stdcall;
    procedure setStartDateTime(const aValue: TDateTime); stdcall;
    procedure setStopDateTime(const aValue: TDateTime); stdcall;
    procedure setFirstTime(const aValue: string); stdcall;
    procedure setScheduleType(const aValue: TCPScheduleType); stdcall;
    procedure setSchedulePRNType(const aValue: TCPScheduleTypePRN); stdcall;
    procedure setStatus(const aValue: TCPKardexActionStatus); stdcall;
    procedure setSpecialInstructions(const aValue: string); stdcall;
    procedure setPRNNTE(const aValue: integer); stdcall;
    procedure setScheduleTimes(const aValue: string); stdcall;

    procedure DebugDisplay; stdcall;

    procedure BuildSchedule; stdcall;
    procedure ExtendSchedule(aNewStopDate: TDateTime); stdcall;
    procedure LoadSchedule(aList: TStrings); stdcall;
    procedure AddToList(aList: TStrings); stdcall;
  end;

  ICPKardexEvent = interface(IInterface)
    ['{B837F863-8B61-4A0C-84B8-91CA4F424962}']
    function getEventID: string; stdcall;
    function getActionID: string; stdcall;
    function getScheduledDateTime: TDateTime; stdcall;
    function getStatus: TCPKardexEventStatus; stdcall;
    function getComment: string; stdcall;
    function getCompletedDateTime: TDateTime; stdcall;
    function getCompletedByID: string; stdcall;
    function getHasAudits: boolean; stdcall;

    procedure setScheduledDateTime(const aValue: TDateTime); stdcall;
    procedure setStatus(const aValue: TCPKardexEventStatus); stdcall;
    procedure setComment(const aValue: string); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
  end;

  ICPKardexAudit = interface(IInterface)
    ['{0DAC996E-A383-4465-87BA-765577C4F856}']
    function getEventID: string; stdcall;
    function getActionID: string; stdcall;
    function getAuditDateTime: TDateTime; stdcall;
    function getAuditByID: string; stdcall;
    function getDescription: string; stdcall;
    function getComment: string; stdcall;
  end;

  ICPParameters = interface(IInterface)
    ['{0C98E0B2-DEDB-4079-95CA-E8F452AFA770}']
    function getAllowExternalAttachments: boolean; stdcall;
    function getByPassCRC: boolean; stdcall;
    function getAllowCachedQueries: boolean; stdcall;
    function getCPHomePage: string; stdcall;
    function getOffLine: Boolean; stdcall;
    function getOfflineMessage: string; stdcall;
    function getBatchLoadSize: integer; stdcall;
    function getUnverifiedRetention: integer; stdcall;
    function getInstrumentDataRetention: integer; stdcall;
    function getImagingXferDirectory: string; stdcall;
    function getRegisteredApplicationVersion: string; stdcall;
    function getCurrentApplicationVersion: string; stdcall;
    function getCurrentApplicationCompileDateTime: string; stdcall;
    function getCurrentApplicationCRCValue: string; stdcall;
    function getApplicationRegistered: boolean; stdcall;
    function getNotificationList(const aList: TCPNotificationList): string; stdcall;

    function getUseKardex(aLocationName: string): boolean; stdcall;
    function getUseTIUNoteWriter(aLocationName: string): boolean; stdcall;

    function getUpdateFramework: string; stdcall;
    function getUpdateMaster: string; stdcall;
    function getUpdateSource: string; stdcall;

    function getTerminologyVersion: integer; stdcall;
    function getTerminologyDescription: string; stdcall;
    function getTerminologyCacheConnectionString: string; stdcall;
    function getTerminologyCached: boolean; stdcall;
    function getTerminologyPath: string; stdcall;

    function getVersionCRCValue(aVersion: string): string; stdcall;
    function getVersionCompatible(aVersion: string): boolean; stdcall;

    procedure setAllowExternalAttachments(const aValue: boolean); stdcall;
    procedure setByPassCRC(const aValue: boolean); stdcall;
    procedure setAllowCachedQueries(const aValue: boolean); stdcall;
    procedure setCPHomePage(const aValue: string); stdcall;
    procedure setOffLine(const aValue: boolean); stdcall;
    procedure setOfflineMessage(const aValue: string); stdcall;
    procedure setBatchLoadSize(const aValue: integer); stdcall;
    procedure setUnverifedRetention(const aValue: integer); stdcall;
    procedure setInstrumentDataRetention(const aValue: integer); stdcall;
    procedure setImagingXferDirectory(const aValue: string); stdcall;
    procedure setNotificationList(const aList: TCPNotificationList; const aValue: string); stdcall;

    procedure setTerminologyVersion(const aValue: integer); stdcall;
    procedure setTerminologyDescription(const aValue: string); stdcall;
    procedure setTerminologyCacheConnectionString(const aValue: string); stdcall;
    procedure setTerminologyCached(const aValue: boolean); stdcall;
    procedure setTerminologyPath(const aValue: string); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
    procedure SaveList(const aList: TCPNotificationList); stdcall;
  end;

  ICPUserOptions = interface(IInterface)
    ['{C8913981-4F41-4D8D-97BE-DE79E7AEB2F0}']
    function getConfirmOnClose: boolean; stdcall;
    function getFlowsheetItemFont(aRange: TObservationRange): TFont; stdcall;
    function getFlowsheetItemColor(aRange: TObservationRange): TColor; stdcall;
    function getAutoRefreshEnabled: boolean; stdcall;
    function getAutoRefreshInterval: integer; stdcall;
    function getDefaultFlowsheetID: string; stdcall;
    function getUsePatientLocation: boolean; stdcall;
    function getDefaultLocationID: string; stdcall;
    function getLookAndFeel: TCPLookAndFeel; stdcall;
    function getUseFlowsheetExplorer: boolean; stdcall;

    procedure setConfirmOnClose(aValue: boolean); stdcall;
    procedure setFlowsheetItemFont(aRange: TObservationRange; aFont: TFont); stdcall;
    procedure setFLowsheetItemColor(aRange: TObservationRange; aColor: TColor); stdcall;
    procedure setAutoRefreshEnabled(aValue: boolean); stdcall;
    procedure setAutoRefreshInterval(aValue: integer); stdcall;
    procedure setDefaultFlowsheetID(aValue: string); stdcall;
    procedure setUsePatientLocation(aValue: boolean); stdcall;
    procedure setDefaultLocationID(aValue: string); stdcall;
    procedure setLookAndFeel(aValue: TCPLookAndFeel); stdcall;
    procedure setUseFlowsheetExplorer(aValue: boolean); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  end;

  ICPShift = interface(ICPConsoleItem)
    ['{EB79E739-8B9B-4117-9EDC-263F188836D0}']
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getStartTime: TDateTime; stdcall;
    function getStopTime: TDateTime; stdcall;
    function getLength: integer; stdcall;
    function getMultiDay: boolean; stdcall;
    function getActive: boolean; stdcall;
    function getDisplayName: string; stdcall;
    function getComment: string; stdcall;

    procedure setStartTime(const aValue: TDateTime); stdcall;
    procedure setStopTime(const aValue: TDateTime); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setComment(const aValue: string); stdcall;

    procedure Save; stdcall;
  end;

  ICPSchedule = interface(IInterface)
    ['{128361E7-EE9C-4C2E-961A-669BD45628B2}']
    function getID: string; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;
    function getScheduleInterval: integer; stdcall;
    function getScheduleType: TCPScheduleType; stdcall;
    function getMultiDay: boolean; stdcall;
    function getScheduledTimes(aStartDateTime: TDateTime = 0): string; stdcall;
    function getSchedule: string; stdcall;
    function getCustomSchedule: string; stdcall;

    procedure setDisplayName(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setCustomSchedule(const aValue: string); stdcall;
    procedure setScheduleInterval(const aValue: integer); stdcall;
  end;

  IDBConnection = interface(IInterface)
    ['{1C736B30-1188-48F6-A407-52C684F19121}']
    procedure BeginTransaction; stdcall;
    procedure CommitTransaction; stdcall;
    function Connect(aParams: array of Variant): boolean; stdcall;
    procedure Disconnect; stdcall;
    function ExecCommand(aCommandName: string; aParams: array of variant): boolean; stdcall;
    function ExecQuery(aQueryName: string; aParams: array of variant; aResults: IXMLDocument): boolean; stdcall;
    function ExecResults: TStringList; stdcall;
    function ExecStoredProc(aStoredProcName: string; aParams: array of variant; aParamList: TStringList): boolean; stdcall;
    function getDescription: string; stdcall;
    function InitCommands(aCommandFileName: string): boolean; stdcall;
    function IsConnected: boolean; stdcall;
    procedure RollbackTransaction; stdcall;
    procedure SaveCommandList(aFileName: string); stdcall;
  end;

  IDocumentor = interface(IInterface)
    ['{A6F92BE4-A229-4B29-83E5-5FA004C14605}']
    function getHTML(aXMLDocument: IXMLDocument): boolean; stdcall;
    function getText: string; stdcall;
    function getXML: string; stdcall;
  end;

  ILegacyBroker = interface(IInterface)
    ['{C6EC52B8-A20F-410A-BA97-0A386F224DD3}']
    function CallServer(aRPCName: string; aParams: array of variant; aMultlist:
      TStringList): boolean;
    function CreateContext(strContext: string): boolean; stdcall;
    function GetConnected: boolean; stdcall;
    function GetListenerPort: Integer; stdcall;
    function GetResults: TStringList; stdcall;
    function GetRPCTimeLimit: Integer; stdcall;
    function GetServer: string;
    procedure SetConnected(const Value: boolean); stdcall;
    procedure SetListenerPort(const Value: Integer); stdcall;
    procedure SetRPCTimeLimit(const Value: Integer); stdcall;
    procedure SetServer(const Value: string);
    property Connected: boolean read GetConnected write SetConnected;
    property ListenerPort: Integer read GetListenerPort write SetListenerPort;
    property Results: TStringList read GetResults;
    property RPCTimeLimit: Integer read GetRPCTimeLimit write SetRPCTimeLimit;
    property Server: string read GetServer write SetServer;
  end;

  ICCOWContextor = interface(IInterface)
    ['{194F2FFF-C8D5-4489-B83B-F27CE1641FC8}']
    procedure SetCCOWNotifyNewPatientID(aProcedure: TNotifyNewPatientID); stdcall;
    procedure SetCCOWNotifyNewLinkStatus(aProcedure: TNotifyNewLinkStatus); stdcall;
    procedure SetCCOWNotifyAbleToChange(aProcedure: TNotifyAbleToChange); stdcall;
    procedure SetCCOWPatientContextItem(aContextItem: string); stdcall;
    procedure SetCCOWPatientContextID(const aPtId: string); stdcall;
    procedure CCOWContextRefresh; stdcall;
    procedure CCOWContextSuspend; stdcall;
    procedure CCOWContextResume(const aRejoinMode: TVACCOW_RejoinMode); stdcall;

    function CCOWLinkStatus: TVACCOW_LinkStatus; stdcall;
  end;

  IVistaSystem = interface(IInterface)
    ['{963714BA-461C-4175-803C-29C62F5781EE}']
    function Station_ID: string; stdcall;
    function Station_Name: string; stdcall;
    function DUZ: string; stdcall;
  end;

  ILookupItem = interface(IInterface)
    ['{3E09F8C0-D468-47F6-A652-7C38F5188C04}']
    function getExternalID: string; stdcall;
    function getInternalID: string; stdcall;
    procedure setExternalID(aValue: string); stdcall;
    procedure setInternalID(aValue: string); stdcall;
  end;

  IObservation = interface(IInterface)
    ['{F2222085-9125-4903-8552-512AFC0D3151}']
    function getID: string; stdcall;
    function getParent: IObservation; stdcall;
    function getParentID: string; stdcall;
    function getBatteryID: string; stdcall;
    function getLocationID: string; stdcall;
    function getFacilityID: string; stdcall;

    function getPatientName: string; stdcall;
    function getPatientID: string; stdcall;

    function getObservedDateTime: TDateTime; stdcall;
    function getObservedBy: string; stdcall;
    function getObservedByID: string; stdcall;
    function getObservedByInitials: string; stdcall;

    function getEnteredDateTime: TDateTime; stdcall;
    function getEnteredBy: string; stdcall;
    function getEnteredByID: string; stdcall;

    function getObsTerm: ITerm; stdcall;
    function getObsUnit: ITerm; stdcall;

    function getSValue: string; stdcall;
    function getDisplayValue: string; stdcall;
    function getNumericValue: double; stdcall;

    function getStatus: TObservationRecordStatus; stdcall;
    function getStatusValue: string; stdcall;

    function getRange: TObservationRange; stdcall;
    function getRangeValue: string; stdcall;

    function getSetCount: integer; stdcall;
    function getSetID(aIndex: integer): string; stdcall;

    function getComment: string; stdcall;
    function getHasAuditLog: boolean; stdcall;
    function getIsCorrectionForID: string; stdcall;

    function getSourceID: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceDataItemID: string; stdcall;
    function getSourceComments: string; stdcall;

    function getTerm(aTermType: TTermType): ITerm;
    function getTermID(aTermType: TTermType): string; stdcall;
    function getTermName(aTermType: TTermType): string; stdcall;
    function getTermDisplayName(aTermType: TTermType): string; stdcall;

    function getChildCount: integer; stdcall;
    function getChildID(aIndex: integer): string; stdcall;
    function getFirstChild: IObservation; stdcall;
    function getNextChild(aObservation: IObservation): IObservation; stdcall;

    function getHasAttribute(aID: string): boolean; stdcall;
    function IsPartOfSet(aSetID: string): boolean; stdcall;
    procedure addChildObservation(aObservation: IObservation); stdcall;
    procedure setParent(aObservation: IObservation); stdcall;

    function AddToListView(aListView: TListView): TListItem; stdcall;
    {
    blj 15 Nov 2010 - Added to more cleanly support vitals.
    }
    function isReadOnly: boolean; stdcall;

  end;

  ICPFlowsheetView = interface(IInterface)
    ['{04FDB017-8FCF-46A4-B683-D66118273E48}']
    function getID: string; stdcall;
    function getActive: boolean; stdcall;
    function getName: string; stdcall;
    function getDisplayName: string; stdcall;
    function getComment: string; stdcall;
    function getIsNationalTitle: boolean; stdcall;
    function getAllowPivot: boolean; stdcall;
    function getDisplayInterval: Integer; stdcall;
    function getIncludeAverages: boolean; stdcall;
    function getIncludeCounts: boolean; stdcall;
    function getIncludeTotals: boolean; stdcall;
    function getViewStyle: TObservationViewStyle; stdcall;

    function getSupplementalID: string; stdcall;
    function getSupplementalTitle: string; stdcall;
    function getDefaultSetID: string; stdcall;
    function getDefaultMethod: ITerm; stdcall;
    function getDefaultPosition: ITerm; stdcall;
    function getDefaultLocation: ITerm; stdcall;
    function getDefaultProduct: ITerm; stdcall;
    function getPageType: TCPFlowsheetPageType; stdcall;
    function getActivatedDateTime: TDateTime; stdcall;
    function getActivatedByID: string; stdcall;
    function getActivatedByName: string; stdcall;
    function getActivatedComment: string; stdcall;
    function getDeactivatedDateTime: TDateTime; stdcall;
    function getDeactivatedByID: string; stdcall;
    function getDeactivatedByName: string; stdcall;
    function getDeactivatedComment: string; stdcall;

    function getIndex(aList: TStrings): integer; stdcall;

    function getViewTermByIndex(aIndex: integer): ICPFlowsheetViewTerm; stdcall;
    function getViewTermByOrder(aTermOrder: integer): ICPFlowsheetViewTerm; stdcall;
    function getViewTermCount: integer; stdcall;
    function getViewTermIndex(aItem: IInterface): integer; stdcall;

//    procedure CopyFrom(aID: string); stdcall;
    procedure Load; stdcall;
    procedure ForcedLoad; stdcall;
    procedure Save; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aParentNode: TTreeNode): TTreeNode; stdcall;

    procedure setAllowPivot(const aValue: boolean); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayInterval(const aValue: integer); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setIncludeAverages(const aValue: boolean); stdcall;
    procedure setIncludeCounts(const aValue: boolean); stdcall;
    procedure setIncludeTotals(const aValue: boolean); stdcall;
    procedure setViewStyle(const aValue: TObservationViewStyle); stdcall;

    procedure InsertViewTerm(aViewTerm: ICPFlowsheetViewTerm); stdcall;
    procedure DeleteViewTerm(aViewTerm: ICPFlowsheetViewTerm); stdcall;
  end;

  ICPFlowsheetViewTerm = interface(IInterface)
    ['{1E66DB49-78C6-4ACA-88BA-AB07F1886B36}']
    function getAttributeAllowed(Index: TTermType): boolean; stdcall;
    function getAttributeMandatory(Index: TTermType): boolean; stdcall;
    function getAttributeID(Index: TTermType): string; stdcall;

    function getDisplayName: string; stdcall;
    function getDisplayOrder: integer; stdcall;
    function getDisplayWidth: integer; stdcall;
    function getDisplayColumns: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayOnly: boolean; stdcall;
    function getDefaultSetID: string; stdcall;
    function getDisplayInCellTotal: boolean; stdcall;
    function getDisplayMinValue: boolean; stdcall;
    function getDisplayMaxValue: boolean; stdcall;
    function getRequiredTerm: boolean; stdcall;
    function getDisplayInDropdown: boolean; stdcall;

    procedure setAttribute(aTermType: TTermType; aAllowed: boolean; aMandatory: boolean; aID: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setDisplayOrder(const aValue: integer); stdcall;
    procedure setDisplayWidth(const aValue: integer); stdcall;
    procedure setDisplayColumns(const aValue: integer); stdcall;
    procedure setDisplayTotal(const aValue: boolean); stdcall;
    procedure setDisplayCount(const aValue: boolean); stdcall;
    procedure setDisplayAverage(const aValue: boolean); stdcall;
    procedure setDisplayOnly(const aValue: boolean); stdcall;
    procedure setDisplayInCellTotal(const aValue: boolean); stdcall;
    procedure setDisplayMinValue(const aValue: boolean); stdcall;
    procedure setDisplayMaxValue(const aValue: boolean); stdcall;
    procedure setRequiredTerm(const aValue: boolean); stdcall;
    procedure setDisplayInDropdown(const aValue: boolean); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function ContainsObservation(aObservation: IObservation): boolean; stdcall;
  end;

  IPatient = interface(IInterface)
    ['{7CCE3675-9F89-4073-B509-88789D8D8266}']
    function getAge: Integer; stdcall;
    function getDFN: string; stdcall;
    function getDOB: TDateTime; stdcall;
    function getDateOfBirth: string; stdcall;
    function getICN: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getRace: string; stdcall;
    function getRoomBed: string; stdcall;
    function getSex: string; stdcall;
    function getSSN: string; stdcall;
    function getWardLocation: string; stdcall;
    function getSensitive: boolean; stdcall;
    function getDisplayLine(aLine: integer): string; stdcall;
    function getLastMessageID: integer; stdcall;
    procedure setLastMessageID(const AID: integer); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
  end;

  ISensitivePatient = interface(IInterface)
    ['{3B532FE6-EAAB-4588-9A39-DFCD3114DFA0}']
    procedure Unlock; stdcall;
  end;

  ITerminology = interface(IInterface)
    ['{12230E98-91E4-40EE-A3C7-358D365AF551}']
    function GetConvertedValue(aValue: Double; aFromUnit, aToUnit: ITerm): Double; stdcall;
    function GetTerm(aID: string): ITerm; stdcall;
    function GetTermCount: Integer; stdcall;
    function GetTermIndex(aList: TStrings; aID: string): Integer; stdcall;
    function GetTermMapper(aID: string): ITermMapper; overload; stdcall;
    function GetTermMapper(aIndex: integer): ITermMapper; overload; stdcall;
    function GetTermMapper(aSourceID, aSourceApplication, aSourceVersion: string): ITermMapper; overload; stdcall;
    function GetTermMapperCount: integer; stdcall;
    function GetTermMapperHL7(aSendingApplication: string): ITermMapper; stdcall;
    function GetNewTermMapper: ITermMapper; stdcall;
    function Initialize(aName: string; aDBConnection: IDBConnection): boolean; overload; stdcall;
    function Initialize(aName: string; aDBType: integer; aParams: array of Variant): boolean; overload; stdcall;
    function DBConnection: IDBConnection; stdcall;

    function GetCached: boolean; stdcall;
    function GetCachedPathName: string; stdcall;
    function CacheAllQueries(const aPath: string): boolean; stdcall;

    procedure CacheQuery(const aQueryName, aPath: string);
    procedure SetCaching(const aSettingsString: string); stdcall;

    procedure Load; stdcall;
    procedure LoadConversionObjects(aList: TStrings); stdcall;
    procedure LoadItem(aList: TStrings; aTermID: string); stdcall;
    procedure LoadList(aList: TStrings; aTermType: TTermType; aValueType: TTermValueType = vtUnspecified; aDisplayAs: TTermDisplay = tdTerm); stdcall;
    procedure LoadTermMappers(aListView: TListView); overload; stdcall;
    procedure LoadTermMappers(aList: TStrings); overload; stdcall;
    procedure LoadListView(aListView: TListView; aTermType: TTermType); overload; stdcall;
    procedure LoadTreeView(aTreeView: TTreeView); overload; stdcall;
    procedure LoadTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aTermType: TTermType); overload; stdcall;

    procedure DebugMe; stdcall;
  end;

  ITerminologyEditor = interface(ITerminology)
    ['{EE93A905-E803-4C4E-B12A-34B9E8B47F91}']
    function CreateTerm(aTermType: TTermType; aDataType: TTermDataType; aID: string = ''): ITermUpdate; stdcall;
    function UpdateTerm(aTermUpdate: ITermUpdate): boolean; stdcall;

    procedure ClearClinicalDataModel(aTerm: ITerm); stdcall;

    procedure AddTermChild(aTermChild: ITermChild); stdcall;
    procedure AddTermPair(aTermPair: ITermPair); stdcall;
    procedure AddTermUnit(aTermUnit: ITermUnit); stdcall;
    procedure AddConversion(aTerm: ITermConversion); stdcall;

    procedure LoadTermChildren(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadTermPairs(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadTermUnits(aTerm: ITerm; aList: IInterfaceList); stdcall;
    procedure LoadUnitConversions(aUnit: ITerm; aList: IInterfaceList); stdcall;

    procedure ImportTerminology(aXMLDoc: IXMLDocument; aNotifyOnProgress: TNotifyOnProgress = nil); stdcall;
  end;

  ITermConversion = interface(IInterface)
    ['{D6A40A09-A232-4586-AF21-ED288D4957F0}']
    function getID: string; stdcall;
    function getTermPre: ITerm; stdcall;
    function getTermPost: ITerm; stdcall;
    function getOffsetPre: Double; stdcall;
    function getOffsetPost: Double; stdcall;
    function getFactor: Double; stdcall;
    function getDecimal: integer; stdcall;
    function getDescription: string; stdcall;
    function Execute(aValue: Double): Double; stdcall;
  end;

  ITerm = interface(IInterface)
    ['{34691C0A-EFCF-4EDE-BA39-D4F60671A7C0}']
    function getAbbreviation: string; stdcall;
    function getActive: boolean; stdcall;
    function getIndex(aList: TStrings): integer; stdcall;

    function getChildCount: integer; stdcall;
    function getChildTerm(aIndex: integer): ITerm; stdcall;
    function getChildValue(aIndex: integer; aValue: string): string; stdcall;
    function getChildUnit(aIndex: integer): ITerm; stdcall;

    function getDataType: TTermDataType; stdcall;
    function getDescription: string; stdcall;
    function getDisplayName: string; stdcall;
    function getHelpText: string; stdcall;
    function getPickListValue(aIndex: integer): string; stdcall;
    function getHasQualifiers(aTermType: TTermType): boolean; stdcall;
    function getTerm: string; stdcall;
    function getTermType: TTermType; stdcall;
    function getValueFormat: string; stdcall;
    function getID: string; stdcall;
    function getSiblingTerm(aID: string): ITerm; stdcall;
    function getBooleanValueTrue: string; stdcall;
    function getBooleanValueFalse: string; stdcall;
    function getMultiSelectPickList: boolean; stdcall;

    function LoadListOfPairs(aTermType: TTermType; aList: TStrings; aDisplayFormat: TTermDisplay = tdTerm): boolean; stdcall;

    function AddToList(aList: TStrings; aDisplayFormat: TTermDisplay): integer; stdcall;
    function AddToListView(aListView: TListView; aDisplayFormat: TTermDisplay): TListItem; stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aDisplayFormat: TTermDisplay): TTreeNode; stdcall;

    function Validate(aUnit: ITerm; aValue: Double): TObservationRange; overload; stdcall;
    function Validate(aUnit: ITerm; aValue: string): TObservationRange; overload; stdcall;
    function getUnitCount: integer; stdcall;
    function getIsValidUnit(aUnit: ITerm): boolean; stdcall;

    procedure DebugMe(aList: TStringList); stdcall;
  end;

  ITermLoader = interface(ITerm)
    ['{CD3ABDF8-2816-4B47-AC8D-7F38BE3B8A91}']
    procedure ClearClinicalDataModel; stdcall;
    procedure AddTermPair(aTermPair: ITermPair); stdcall;
    procedure AddTermUnit(aTermUnit: ITermUnit); stdcall;
    procedure AddTermChild(aTermChild: ITermChild); stdcall;
    procedure SetOwnerList(aList: IInterfaceList); stdcall;
    function GetObject: TObject; stdcall;
  end;

  ITermPair = interface(IInterface)
    ['{146B8379-8ACF-495C-AEDE-02029AD5C4D7}']
    function getTerm: ITerm; stdcall;
    function getQualifier: ITerm; stdcall;
    function getType: TTermType; stdcall;
    function getValue: Double; stdcall;
    function getOrder: integer; stdcall;
  end;

  ITermUpdate = interface(ITerm)
    ['{BAA20271-E9F8-45AC-8B96-A4DA9A3C514A}']
    procedure setTerm(aValue: string); stdcall;
    procedure setAbbreviation(aValue: string); stdcall;
    procedure setActive(aValue: boolean); stdcall;
    procedure setDataType(aValue: TTermDataType); stdcall;
    procedure setDescription(aValue: string); stdcall;
    procedure setDisplayName(aValue: string); stdcall;
    procedure setHelpText(aValue: string); stdcall;
    procedure setTermType(aValue: TTermType); stdcall;
    procedure setValueType(aValue: TTermValueType); stdcall;
    procedure setBooleanValueTrue(aValue: string); stdcall;
    procedure setBooleanValueFalse(aValue: string); stdcall;
    procedure setMultiSelectPickList(aValue: boolean); stdcall;
  end;

  ITermChild = interface(IInterface)
    ['{0E72CCDA-C6EA-48B4-A50B-7E962C1EE011}']
    function getParentTerm: ITerm; stdcall;
    function getChildTerm: ITerm; stdcall;
    function getChildOrder: integer; stdcall;
    function getValue(aParentValue: string): string; stdcall;
    function getValueType: TTermChildValueType; stdcall;
    function getValueDelimiter: char; stdcall;
    function getValueStartPosition: integer; stdcall;
    function getValueStopPosition: integer; stdcall;
    function getValueUnit: ITerm; stdcall;
    function getEvaluation: string; stdcall;
    function getDescription: string; stdcall;
  end;

  ITermUnit = interface(IInterface)
    ['{78A316EB-B21B-4A76-99E8-727DB2AFCBB6}']
    function getFormattedValidMin: string; stdcall;
    function getFormattedValidMax: string; stdcall;
    function getFormattedReferenceMin: string; stdcall;
    function getFormattedReferenceMax: string; stdcall;

    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getValidMin: Double; stdcall;
    function getValidMax: Double; stdcall;
    function getReferenceMin: Double; stdcall;
    function getReferenceMax: Double; stdcall;
    function getValidDec: integer; stdcall;
    function getParameters: string; stdcall;
  end;

  ITermRangeCheck = interface(IInterface)
    ['{EBBBBE20-7096-4C31-9F29-D5EC8587BADB}']
    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getAgeSpecific: boolean; stdcall;
    function getSexSpecific: boolean; stdcall;
    function getSex: string; stdcall;
    function getAgeMin: integer; stdcall;
    function getAgeMax: integer; stdcall;
    function getNormalMin: Double; stdcall;
    function getNormalMax: Double; stdcall;
    function getAbnormalMin: Double; stdcall;
    function getAbnormalMax: Double; stdcall;
    function getCriticalMin: Double; stdcall;
    function getCriticalMax: Double; stdcall;
    function getAllRanges: string; stdcall;
    function getDescription: string; stdcall;
    function Execute(aValue: Double; aUnit: ITerm): boolean; stdcall;
  end;

  IRangeCheck = interface(IInterface)
    ['{A9BB853C-73DD-4957-9A76-2EC09EB89BE3}']
    function getUnit: ITerm; stdcall;
    function getNormalMin: Double; stdcall;
    function getNormalMax: Double; stdcall;
    function getAbnormalMin: Double; stdcall;
    function getAbnormalMax: Double; stdcall;
    function getCriticalMin: Double; stdcall;
    function getCriticalMax: Double; stdcall;
    function Execute(aValue: Double; aUnit: ITerm): boolean; stdcall;
  end;

  ITermEditorFrame = interface(IInterface)
    ['{B8A8EB51-1EF3-466F-9865-3E665D344F84}']
    procedure Initialize(aTerm: ITerm); stdcall;
    procedure Save; stdcall;
  end;

  ITermMapper = interface(IInterface)
    ['{66B5FDE0-F003-4912-A74D-725E7B34C0D2}']
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getHL7SendingApplication: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceID: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getTableID: string; stdcall;
    function getTerm(aTermType: TTermType; aVendorKey: string): string; stdcall;
    function getValueFormat(aTermType: TTermType; aVendorKey: string): string; stdcall;
    procedure Load; stdcall;
  end;

  IPerson = interface(IInterface)
    ['{CF233213-3479-4395-8683-382E0A19E919}']
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getInitials: string; stdcall;
    function getSex: string; stdcall;
    function getSSN: string; stdcall;
    function getDOB: TDateTime; stdcall;
    function getTitle: string; stdcall;
    procedure AddToList(aList: TStrings); stdcall;
  end;

  IPersonService = interface(IInterface)
    ['{02AD4D7B-FBAB-48D5-B6FA-6D1EF0E0C182}']
    function ExecuteLookup(aLookupValue: string; aList: IInterfaceList): integer; stdcall;
    function getPerson(aID: string): IPerson; stdcall;
  end;

  IUser = interface(IPerson)
    ['{D956CDFC-7152-422B-A5DC-D48E602B74EC}']
    function getDUZ: string; stdcall;
    function getInternalID: string;
    function getName: string; stdcall;
    function getInitials: string; stdcall;
    function getESIGName: string; stdcall;
    function getESIGTitle: string; stdcall;
    function getStation_ID: string; stdcall;
    function getStation_Name: string; stdcall;
    function getCCOWDFNItemName: string; stdcall;
    function getTimeOut: integer; stdcall;
    function HasRole(aRoleName: string): boolean; stdcall;
    function IsThisExecutableValid: boolean; stdcall;
    function IsValidElectronicSignature(aSignature: string): boolean; stdcall;
    function IsProductionSignOn: boolean; stdcall;
    function GetLastError: string; stdcall;
    procedure RefreshAccess; stdcall;
  end;

  ICPFlowsheet = interface(ICPConsoleItem)
    ['{D41BF6E1-7A4A-411D-BDB0-87084D122667}']
    function getDefaultTIUNoteID: string; stdcall;
    function getDefaultTIUNote: string; stdcall;
    procedure setDefaultTIUNoteID(const aValue: string); stdcall;
    procedure setDefaultTIUNote(const aValue: string); stdcall;

    function getDisplayKardex: boolean; stdcall;
    procedure setDisplayKardex(const aValue: boolean); stdcall;

    function Page(aIndex: integer): ICPFlowsheetPage; stdcall;
    function PageCount: integer; stdcall;
    function Total(aIndex: integer): ICPFlowsheetTotal; stdcall;
    function TotalCount: integer; stdcall;

    function LoadPages(aList: TStrings): integer; overload; stdcall;
    function LoadPages(aListView: TListView): integer; overload; stdcall;
    procedure InsertPage(aPage: ICPFLowsheetPage); stdcall;
    procedure RemovePage(aPage: ICPFlowsheetPage); stdcall;

    function LoadTotals(aList: TStrings): integer; overload; stdcall;
    function LoadTotals(aListView: TListView): integer; overload; stdcall;
    procedure InsertTotal(aTotal: ICPFlowsheetTotal); stdcall;
    procedure RemoveTotal(aTotal: ICPFlowsheetTotal); stdcall;
    procedure ClearTotals; stdcall;
    procedure ClearPages; stdcall;
    procedure Save; stdcall;
  end;

  ICPFlowsheetPage = interface(IInterface)
    ['{8189EFB7-EE91-43E7-870A-83C71270CD76}']
    function getDisplayName: string; stdcall;
    function getSpecialInstructions: string; stdcall;
    function getPageType: TCPFlowsheetPageType; stdcall;
    function getPageOrder: integer; stdcall;

    function getFlowsheet: ICPFLowsheet; stdcall;
    function getFlowsheetID: string; stdcall;
    function getFlowsheetName: string; stdcall;

    function getFlowsheetView: ICPFlowsheetView; stdcall;
    function getFlowsheetViewID: string; stdcall;
    function getFlowsheetViewName: string; stdcall;

    procedure setDisplayName(aValue: string); stdcall;
    procedure setSpecialInstructions(aValue: string); stdcall;
    procedure setPageType(aValue: TCPFlowsheetPageType); stdcall;
    procedure setPageOrder(aValue: integer); stdcall;
    procedure setFlowsheetView(aValue: ICPFlowsheetView); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
  end;

  ICPFlowsheetTotal = interface(ICPConsoleItem)
    ['{E3DBB2C8-C643-41C5-824E-A7B2DD9FF759}']

    function getDisplayName: string; stdcall;
    function getDefaultUnit: ITerm; stdcall;
    function getDefaultUnitAbbv: string; stdcall;
    function getDecimalPrec: integer; stdcall;
    function getDisplayTotal: boolean; stdcall;
    function getDisplayCount: boolean; stdcall;
    function getDisplayAverage: boolean; stdcall;
    function getDisplayMinimum: boolean; stdcall;
    function getDisplayMaximum: boolean; stdcall;
    function getDisplayRatio: boolean; stdcall;
    function getDisplayDifference: boolean; stdcall;
    function getFlowsheetID: string; stdcall;
    function getTermCount: integer; stdcall;
    function getTerm(aIndex: integer): ITerm; stdcall;

    procedure setDisplayName(const aValue: string); stdcall;
    procedure setDefaultUnit(const aValue: ITerm); stdcall;
    procedure setDecimalPrec(const aValue: integer); stdcall;
    procedure setDisplayTotal(const aValue: boolean); stdcall;
    procedure setDisplayCount(const aValue: boolean); stdcall;
    procedure setDisplayAverage(const aValue: boolean); stdcall;
    procedure setDisplayMinimum(const aValue: boolean); stdcall;
    procedure setDisplayMaximum(const aValue: boolean); stdcall;
    procedure setDisplayRatio(const aValue: boolean); stdcall;
    procedure setDisplayDifference(const aValue: boolean); stdcall;
    procedure setFlowsheetID(const aValue: string); stdcall;
    procedure ClearAllTerms; stdcall;
    procedure AddTerm(aTerm: ITerm); stdcall;

    function Execute(aList: IInterface): boolean; stdcall;
    function getTotal: Double; stdcall;
    function getCount: integer; stdcall;
    function getAverage: Double; stdcall;
    function getRatio: string; stdcall;

    function getTotalByTerm(aIndex: integer): Double; stdcall;
    function getCountByTerm(aIndex: integer): integer; stdcall;
    function getAverageByTerm(aIndex: integer): Double; stdcall;
    procedure AppendToGrid(aStringGrid: TStringGrid); stdcall;
    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;

    procedure Save; stdcall;
  end;

  //blj 13 August 2007:  Protocol/Location stuff

  ICPWard = interface(IInterface)
    ['{022C84C8-5F96-4A01-9FB5-3AF735377A1A}']
    function getIFN: integer; stdcall;
    function getWard: string; stdcall;
  end;

  ICPDivision = interface(IInterface)
    ['{9EEDFE08-6008-41F8-BA91-73B4089C7192}']
    function getIFN: integer; stdcall;
    function getName: string; stdcall;
    function getWardList: IInterfaceList; stdcall;
  end;

  ICPProtocol = interface(IInterface)
    ['{28BBDF68-C7F5-44FC-B4BF-6962ED50DE91}']
    function getIFN: integer; stdcall;
    function getName: string; stdcall;
  end;

  ICPADTTarget = interface(IInterface)
    ['{F24C3FFA-2E84-4F85-AD90-956CDB95F168}']
    function getDivision: ICPDivision; stdcall;
    function getSubscriber: ICPProtocol; stdcall;
    function getWard: ICPWard; stdcall;
    function getMessageType: string; stdcall;
    function getEventType: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
  end;

  ICPTIUDocument = interface(IInterface)
    ['{5F05BBFF-C6E8-43F5-BAA8-DBACFF214A8E}']
    function getID: string; stdcall;
    function getLocation: string; stdcall;
    function getLocationID: string; stdcall;
    function getTitle: string; stdcall;
    function getTitleID: string; stdcall;
    function getPatient: IPatient; stdcall;
    function getUser: IUser; stdcall;
    function getDateTime: TDateTime; stdcall;

    procedure setLocation(const aValue: string); stdcall;
    procedure setLocationID(const aValue: string); stdcall;
    procedure setTitle(const aValue: string); stdcall;
    procedure setTitleID(const aValue: string); stdcall;
    procedure setPatient(const aPatient: IPatient); stdcall;
    procedure setCosignerID(const aValue: string);stdcall;
    procedure setDateTime(const aValue: TDateTime); stdcall;

    procedure ClearText; stdcall;
    procedure AppendText(aText: TStrings); stdcall; overload;
    procedure AppendText(aText: string); stdcall; overload;

    function Sign(aESIG: string): boolean; stdcall;
  end;

  ICPReport = interface(IInterface)
    ['{A8A3E8BD-37E8-4DE9-8906-B22B997781A7}']
    function getPage(aPageNumber: integer; aLines: TStrings): boolean; stdcall;
    function getPageCount: integer; stdcall;

    procedure Refresh(
      aStartDateTime, aStopDateTime: TDateTime;
      aPatient: IPatient;
      aFlowsheet: ICPFlowsheet;
      //aObsList: ICliOList;
      aIncludeComments: boolean = False);

  end;

  ICPADTTargetList = interface(IInterfaceList)
    ['{AAD685DD-8CC1-490E-8514-D40A75517A2D}']
    procedure DeleteItem(AItem: ICPADTTarget); stdcall;
    procedure Load; stdcall;
    procedure Save; stdcall;
    function NewTarget(ASubscriber: ICPProtocol; ADivision: ICPDivision; AWard: ICPWard;
      AName: string; AMessageType: string; AEventType: string): ICPADTTarget; stdcall;
  end;

  ICPDivisionList = interface(IInterfaceList)
    ['{21267F57-B39E-4F74-A3F5-BF8547C84694}']
    // Holds ICPLocation interfaces.
    procedure Load; stdcall;
  end;

  ICPProtocolList = interface(IInterfaceList)
    ['{4B94C582-CCA0-4164-9142-9FDE3765CD84}']
    procedure Load; stdcall;
  end;

  ICPProtocolLocation = interface(IInterface)
    ['{E30C6096-A020-4A49-85BA-9EF81E864581}']
    function getProtocolList: IInterfaceList;
    function getADTTargetList: IInterfaceList;
    function getDivisionList: IInterfaceList;
  end;

  IDBExport = interface(IInterface)
    ['{74C45547-FFD4-481E-BD24-04DC9BB32AAA}']
    procedure ClearLastExport; stdcall;
    procedure AddExportTable(aTableName: string; aQueryName: string; aParams: array of Variant);
    procedure SaveExport(aFilename: string; aHeaderFilename: string = ''); stdcall;
    procedure DisplayExport(aList: TStrings); stdcall;
    procedure SetExportDescription(aValue: string); stdcall;
    procedure SetExportFramework(aValue: string); stdcall;
    procedure SetExportVersion(aValue: integer); stdcall;
  end;

  IDBImport = interface(IInterface)
    ['{2322BF59-1BFB-4D0B-8273-BC40B2799F98}']
    procedure ExecuteImport(aImportDocument: IXMLDocument); stdcall;
    procedure setNotifyImportProgress(aNotifyOnProgress: TNotifyOnProgress); stdcall;
  end;

  (*
  TManagedInterfacedObject = class(TInterfacedObject);

  TUnmanagedInterfacedObject = class(TInterfacedObject)
  public
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  end;
  *)

  TLookupItem = class(TInterfacedObject, ILookupItem)
  private
    fExternalID: string;
    fInternalID: string;
  public
    constructor Create(aInternalID, aExternalID: string);
    function getExternalID: string; stdcall;
    function getInternalID: string; stdcall;
    procedure setExternalID(aValue: string); stdcall;
    procedure setInternalID(aValue: string); stdcall;
  end;

function NewDBConnection_MSAccess: IDBConnection;
function NewDBConnection_VistA: IDBConnection;
function NewDBConnection_VistANoCCOW: IDBConnection;

function NewDBConnection_CacheSQL: IDBConnection;
function NewDBConnection_J2EE: IDBConnection;

function NewTerminology: ITerminology;
function NewCPService(aTerminology: ITerminology; aPersonService: IPersonService): ICPService;
function NewCPKardexService(aTerminology: ITerminology; aCPService: ICPService): ICPKardexService;
function NewPersonService(aDBConnection: IDBConnection): IPersonService;
function NewAccessControlList(aDBConnection: IDBConnection; aPersonService: IPersonService): ICPAccessControlList;

function NewTestPatient: IPatient;
function NewPatient(aXMLNode: IXMLNode): IPatient;
function NewUser(aUserID: string; aDBConnection: IDBConnection): IUser;
function NewGUID(aStrip: Boolean = False): string;

function NewCPBackgroundTask(aXMLNode: IXMLNode; aDBConnection: IDBConnection): ICPBackgroundTask;
function NewCPParameters(aDBConnection: IDBConnection): ICPParameters;
function NewCPParameterFile(aDBConnection: IDBConnection; aApplicationKey: string): ICPParameterFile;
function NewCPUserOptions(aDBConnection: IDBConnection): ICPUserOptions;

function NewCPGatewayConfig(fDBConnection: IDBConnection): ICPGatewayConfig;
function NewCPProtocolLocation(fDBConnection: IDBConnection): ICPProtocolLocation;
function NewCPLocation(aXMLNode: IXMLNode): ICPLocation;
function NewCPTIUDocument(aDBConnection: IDBConnection; aUser: IUser): ICPTIUDocument;

implementation

uses
  oDBController_Terminology,
  oDBController_CPConsole,
  oDBController_CPKardex,
  oDBController_Person,

  oCPAccessControlList,

  oDBConnection_MSAccess,
  oDBConnection_VistA,
  oDBConnection_Cache,
  oDBConnection_J2EE,

  oDBController_Patient,

  oCliO,
  oCPGatewayConfig,
  oCPPatient,
  oCPUser,
  oCPUserOptions,
  oCPBackgroundTask,
  oCPParameterFile,
  oCPParameters,
  oCPProtocolLocation,
  oCPLocation,
  oCPTIUDocument,
  StrUtils,
  Variants,
  oCP_Exceptions;

{---------------- Factories for primary interfaces objects --------------------}

function NewDBConnection_MSAccess: IDBConnection;
begin
  TDBConnection_MSAccess.Create.GetInterface(IDBConnection, Result);
end;

function NewDBConnection_VistA: IDBConnection;
begin
  TDBConnection_VistA.Create.GetInterface(IDBConnection, Result);
end;

function NewDBConnection_VistANoCCOW: IDBConnection;
begin
  TDBConnection_VistA.CreateNoCCOW.GetInterface(IDBConnection, Result);
end;

function NewDBConnection_CacheSQL: IDBConnection;
begin
  TDBConnection_Cache.Create.GetInterface(IDBConnection, Result);
end;

function NewDBConnection_J2EE: IDBConnection;
begin
  TDBConnection_J2EE.Create.GetInterface(IDBConnection, Result);
end;

function NewTerminology: ITerminology;
begin
  NewTerminologyController.GetInterface(ITerminology, Result);
end;

function NewCPService(aTerminology: ITerminology; aPersonService: IPersonService): ICPService;
begin
  TDBController_CPConsole.Create(aTerminology, aPersonService).GetInterface(ICPService, Result);
end;

function NewCPKardexService(aTerminology: ITerminology; aCPService: ICPService): ICPKardexService;
begin
  TDBController_CPKardex.Create(aTerminology, aCPService).GetInterface(ICPKardexService, Result);
end;

function NewPersonService(aDBConnection: IDBConnection): IPersonService;
begin
//  TDBController_Person.Create(aDBConnection).GetInterface(IPersonService, Result);
  Result := TDBController_Person.Create(aDBConnection);
end;

function NewAccessControlList(aDBConnection: IDBConnection; aPersonService: IPersonService): ICPAccessControlList;
begin
  TCPAccessControlList.Create(aDBConnection, aPersonService).GetInterface(ICPAccessControlList, Result);
end;

function NewTestPatient: IPatient;
begin
  NewCPTestPatient.GetInterface(IPatient, Result);
end;

function NewPatient(aXMLNode: IXMLNode): IPatient;
begin
  NewCPPatient(aXMLNode).GetInterface(IPatient, Result);
end;

function NewUser(aUserID: string; aDBConnection: IDBConnection): IUser;
begin
  NewCPUser(aUserID, aDBConnection).GetInterface(IUser, Result);
end;

{
function NewCPFlowsheetView(aXMLNode: IXMLNode; aDBConnection: IDBConnection; aTerminology: ITerminology): IObservationView;
begin
  TObservationView.Create(aXMLNode, aDBConnection, aTerminology).GetInterface(IObservationView, Result);
end;

function NewCPFlowsheetViewTerm(aXMLNode: IXMLNode; aDefaultSetID: string): IObservationViewTerm;
begin
  TObservationViewTerm.Create(aXMLNode, aDefaultSetID).GetInterface(IObservationViewTerm, Result);
end;
}
{
function NewCPInstrument(aXMLNode: IXMLNode; aDBConnection: IDBConnection): ICPInstrument;
begin
  TCPInstrument.Create(aXMLNode, aDBConnection).GetInterface(ICPInstrument, Result);
end;
}

{
function NewCPProcedure(aXMLNode: IXMLNode; aDBConnection: IDBConnection): ICPProcedure;
begin
  TCPProcedure.Create(aXMLNode, aDBConnection).GetInterface(ICPProcedure, Result);
end;
}

function NewCPBackgroundTask(aXMLNode: IXMLNode; aDBConnection: IDBConnection): ICPBackgroundTask;
begin
  TCPBackGroundTask.Create(aXMLNode, aDBConnection).GetInterface(ICPBackgroundTask, Result);
end;

{
function NewCPReport(aCliOService: TDBController_Observations; aTerminology: ITerminology): ICPReport;
begin
  TCPReport.Create(aCliOService, aTerminology).GetInterface(ICPReport, Result);
end;
}

{
function NewCPFlowsheet(aXMLNode: IXMLNode; aDBConnection: IDBConnection; aTerminology: ITerminology): ICPFlowsheet;
begin
  TCPFlowsheet.Create(aXMLNode, aDBConnection, aTerminology).GetInterface(ICPFlowsheet, Result);
end;
}

{
function NewCPFlowsheetPage(aXMLNode: IXMLNode): ICPFlowsheetPage;
begin
  TCPFlowsheetPage.Create(aXMLNode).GetInterface(ICPFLowsheetPage, Result);
end;
}

{
function NewCPFlowsheetTotal(aXMLNode: IXMLNode; aDBConnection: IDBConnection; aTerminology: ITerminology): ICPFlowsheetTotal;
begin
  TCPFlowsheetTotal.Create(aXMLNode, aDBConnection, aTerminology).GetInterface(ICPFlowsheetTotal, Result);
end;
}

function NewCPParameterFile(aDBConnection: IDBConnection; aApplicationKey: string): ICPParameterFile;
begin
  TCPParameterFile.Create(aDBConnection, aApplicationKey).GetInterface(ICPParameterFile, Result);
end;

function NewCPParameters(aDBConnection: IDBConnection): ICPParameters;
begin
  TCPParameters.Create(aDBConnection).GetInterface(ICPParameters, Result);
end;

function NewCPUserOptions(aDBConnection: IDBConnection): ICPUserOptions;
begin
  TCPUserOptions.Create(aDBConnection).GetInterface(ICPUserOptions, Result);
end;

function NewCPGatewayConfig(fDBConnection: IDBConnection): ICPGatewayConfig;
begin
  TCPGatewayConfig.Create(fDBConnection).GetInterface(ICPGatewayConfig, Result);
end;

function NewCPTIUDocument(aDBConnection: IDBConnection; aUser: IUser): ICPTIUDocument;
begin
  TCPTIUDocument.Create(aDBConnection, aUser).GetInterface(ICPTIUDocument, Result);
end;

function NewGUID(aStrip: Boolean = False): string;
var
  aGUID: TGUID;
begin
  CreateGUID(aGUID);
  Result := GUIDToString(aGUID);
  if aStrip then
    begin
      Result := AnsiReplaceStr(Result, '{', '');
      Result := AnsiReplaceStr(Result, '}', '');
      Result := AnsiReplaceStr(Result, '-', '');
    end;
end;

function NewCPProtocolLocation(fDBConnection: IDBConnection): ICPProtocolLocation;
begin
  Result := oCPProtocolLocation.NewCPProtocolLocation(FDBConnection);
end;

function NewCPLocation(aXMLNode: IXMLNode): ICPLocation;
begin
  TCPLocation.Create(aXMLNode).GetInterface(ICPLocation, Result);
end;

{------------------------------- TLookupItem ----------------------------------}

constructor TLookupItem.Create(aInternalID, aExternalID: string);
begin
  inherited Create;
  fInternalID := aInternalID;
  fExternalID := aExternalID;
end;

function TLookupItem.getExternalID: string;
begin
  Result := fExternalID;
end;

function TLookupItem.getInternalID: string;
begin
  Result := fInternalID;
end;

procedure TLookupItem.setExternalID(aValue: string);
begin
  fExternalID := aValue;
end;

procedure TLookupItem.setInternalID(aValue: string);
begin
  fInternalID := aValue;
end;

{------------------------- TUnmanagedInterfacedObject -------------------------}

(*
function TUnmanagedInterfacedObject._AddRef: Integer;
begin
  Result := -1;
end;

function TUnmanagedInterfacedObject._Release: Integer;
begin
  Result := -1;
end;
*)

end.

