unit oObservation_Audit;

{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 11/10/07 12:39p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Primary object for management of observations.
*
*       Notes:        Observations external to the TDBController_Observations
*                     should ALWAYS be accessed via the IObservation interface.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oObservation_Audit.pas $
*
* $History: oObservation_Audit.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/10/07   Time: 1:18p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 7/06/07    Time: 10:11a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 4/10/07    Time: 3:29p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Revamp of terminology to all interfaces.
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 2/23/07    Time: 12:55p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 12/15/06   Time: 4:04p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * End of year check-in.
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 11/24/06   Time: 4:26p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Implementation of non-modal input in the main form.
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 10/23/06   Time: 4:04p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Finished Pivots, Parser Key, and Totals.
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:10p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Classes,
  SysUtils,
  oInterfaces,
  iCliOGrid,
  oCliOTypes,
  Variants,
  uCPXML,
  XMLintf;

type
  TObservation_Audit = class(TObject)
  private
    fID: string;
    fType: TObservationAuditType;
    fObservationID: string;
    fAuditDateTime: TDateTime;
    fAuditAuthor: string;
    fAuditAuthorID: string;
    fStatusOriginal: TObservationRecordStatus;
    fStatusNew: TObservationRecordStatus;
    fComment: string;
  public
    procedure Display(aStrings: TStrings);
    property ID: string read fID;
    property AuditDateTime: TDateTime read fAuditDateTime;
  end;

function NewAuditEntry(aXMLNode: IXMLNode): TObservation_Audit;

implementation

function NewAuditEntry(aXMLNode: IXMLNode): TObservation_Audit;
begin
  Result := TObservation_Audit.Create;
  try
    with Result do
      begin
        fID := XMLValueAsStr(aXMLNode, 'AUDIT_ID');
        fType := TObservationAuditType(XMLValueAsInt(aXMLNode, 'AUDIT_TYPE'));
        fObservationID := XMLValueAsStr(aXMLNode, 'AUDIT_OBSERVATION');
        fAuditDateTime := XMLValueAsDateTime(aXMLNode, 'AUDIT_DATE_TIME');
        fAuditAuthor := XMLValueAsStr(aXMLNode, 'AUDIT_AUTHOR');
        fAuditAuthorID := XMLValueAsStr(aXMLNode, 'AUDIT_AUTHOR_ID');
        fStatusOriginal := TObservationRecordStatus(XMLValueAsInt(aXMLNode, 'STATUS_ORIGINAL'));
        fStatusNew := TObservationRecordStatus(XMLValueAsInt(aXMLNode, 'STATUS_NEW'));
        fComment := XMLValueAsStr(aXMLNode, 'COMMENT');
      end;
  except
    FreeAndNil(Result);
    raise Exception.Create('Error Creating Audit: ' + #13 + aXMLNode.XML);
  end;
end;

{ TObservation_Audit }

procedure TObservation_Audit.Display(aStrings: TStrings);
begin
  with aStrings do
    begin
      Clear;
      Values['ID'] := fID;
      Values['Audit Type'] := OBSERVATION_AUDIT_TYPE[fType];
      Values['Observation ID'] := fObservationID;
      Values['Audit Date/Time'] := FormatDateTime('MM-DD-YYYY hh:mm:ss', fAuditDateTime);
      Values['Author'] := Format('%s (%s)', [fAuditAuthor, fAuditAuthorID]);
      Values['Original Status'] := OBSERVATION_STATUS[fStatusOriginal];
      Values['New Status'] := OBSERVATION_STATUS[fStatusNew];
      Values['Comment'] := fComment;
    end;
end;

end.

