unit oTermMapper;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 8 $  $Modtime: 5/22/09 12:54p $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Utilized by incoming messages to map vendor specific data
*                     elements to an actual ID.
*
*       Notes:        Will raise ETerminologyMapperError exception if there
*                     is no mapping.
*
*
================================================================================
* $Archive: /CP Modernization/CliO/source/Common/objects/oTermMapper.pas $
================================================================================
}
interface

uses
  Dialogs,
  SysUtils,
  Windows,
  Messages,
  Classes,
  Controls,
  ComCtrls,
  oCliO,
  oInterfaces,
  oCPTypes,
  oCliOTypes,
  XMLDoc,
  XMLIntf;

type
  ETerminologyMapperError = class(Exception);

  TTermMapper = class(TInterfacedObject, ITermMapper, ICliOSource, ICPCOnsoleItem)
  protected
    fDBConnection: IDBConnection;
    fCPService: ICPService;
    fXMLDocument: IXMLDocument;
    fTableID: string;
    fTableName: string;
    fHL7SendingApplication: string;
    fSourceID: string;
    fSourceApplication: string;
    fSourceVersion: string;
    fSourceTrusted: boolean;
    fLoaded: boolean;

    {ICPConsoleItem Interface}
    function getID: string; stdcall;
    function getItemType: TCPItemType; stdcall;

    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getDisplayName: string; stdcall;
    function getName: string; stdcall;

    function getCanDelete: boolean; stdcall;
    function Delete: boolean; stdcall;

    function getCanRename: boolean; stdcall;
    function Rename(const aNewName: string): boolean; stdcall;

    function getIsLoaded: boolean; stdcall;

    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setService(const aCPService: ICPService); stdcall;

    procedure AddToList(aList: TStrings); stdcall;
    procedure AddToListView(aListView: TListView); stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;

    procedure Load; stdcall;
    function getTerm(aTermType: TTermType; aVendorKey: string): string; stdcall;
    function getValueFormat(aTermType: TTermType; aVendorKey: string): string; stdcall;
    function getTableID: string; stdcall;
    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
    function getHL7SendingApplication: string; stdcall;
  public
    constructor Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection);
    destructor Destroy; override;
  end;

  //function NewTerminologyMapper(aXMLNode: IXMLNode): TTermMapper;

implementation

uses
  uCPXML;

{
function NewTerminologyMapper(aXMLNode: IXMLNode): TTermMapper;
begin
  Result := TTermMapper.Create;
  with Result do
    try
      fTableID := XMLValueAsStr(aXMLNode, 'ID');
      fTableName := XMLValueAsStr(aXMLNode, 'NAME');
      fSourceID := XMLValueAsStr(aXMLNode, 'SOURCE_ID');
      fSourceApplication := XMLValueAsStr(aXMLNode, 'SOURCE_APPLICATION');
      fSourceVersion := XMLValueAsStr(aXMLNode, 'SOURCE_VERSION');
      fSourceTrusted := (XMLValueAsInt(aXMLNode, 'SOURCE_TRUSTED') = 1);
    except
      Free;
    end;
end;
}

{ TTerminology_Mapper }

constructor TTermMapper.Create(aXMLNode: IXMLNode; aDBConnection: IDBConnection);
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fXMLDocument := NewXMLDocument;
  fLoaded := False;
  if Assigned(aXMLNode) then
    begin
      fTableID := XMLValueAsStr(aXMLNode, 'ID');
      fTableName := XMLValueAsStr(aXMLNode, 'NAME');
      fHL7SendingApplication := XMLValueAsStr(aXMLNode, 'HL7_SENDING_APPLICATION');
      fSourceID := XMLValueAsStr(aXMLNode, 'SOURCE_ID');
      fSourceApplication := XMLValueAsStr(aXMLNode, 'SOURCE_APPLICATION');
      fSourceVersion := XMLValueAsStr(aXMLNode, 'SOURCE_VERSION');
      fSourceTrusted := (XMLValueAsInt(aXMLNode, 'SOURCE_TRUSTED') = 1);
    end
  else
    begin
      fTableID := NewGUID;
      fTableName := 'NEW MAPPING TABLE';
      fHL7SendingApplication := '';
      fSourceID := '';
      fSourceApplication := '';
      fSourceVersion := '';
      fSourceTrusted := False;
      fDBConnection.ExecCommand('AddMappingTable', [fTableID, fTableName, 0, 'New Mapping Table', '', '', '', 0]);
    end;
end;

function TTermMapper.Delete: boolean;
begin
  Result := False;
end;

destructor TTermMapper.Destroy;
begin
  fXMLDocument := nil;
  inherited;
end;

procedure TTermMapper.AddToList(aList: TStrings);
begin
  aList.AddObject(fTableName, Self);
end;

procedure TTermMapper.AddToListView(aListView: TListView);
begin
  //
end;

function TTermMapper.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fTableName, Self);
end;

function TTermMapper.getActive: boolean;
begin
  Result := True;
end;

function TTermMapper.getCanDelete: boolean;
begin
  Result := False;
end;

function TTermMapper.getCanRename: boolean;
begin
  Result := True;
end;

function TTermMapper.getComment: string;
begin
  Result := fTableName;
end;

function TTermMapper.getDisplayName: string;
begin
  Result := fTableName;
end;

function TTermMapper.getHL7SendingApplication: string;
begin
  Result := fHL7SendingApplication;
end;

function TTermMapper.getID: string;
begin
  Result := fTableID;
end;

function TTermMapper.getIsLoaded: boolean;
begin
  Result := fLoaded;
end;

function TTermMapper.getItemType: TCPItemType;
begin
  Result := ciMappingTable;
end;

function TTermMapper.getName: string;
begin
  Result := fTableName;
end;

function TTermMapper.getSourceApplication: string;
begin
  Result := fSourceApplication;
end;

function TTermMapper.getSourceID: string;
begin
  Result := fSourceID;
end;

function TTermMapper.getSourceTrusted: boolean;
begin
  Result := fSourceTrusted;
end;

function TTermMapper.getSourceVersion: string;
begin
  Result := fSourceVersion;
end;

function TTermMapper.getTableID: string;
begin
  Result := fTableID;
end;

function TTermMapper.getTerm(aTermType: TTermType; aVendorKey: string): string;
var
  aNode: IXMLNode;
begin
  aNode := fXMLDocument.DocumentElement.ChildNodes.FindNode('RECORD');
  while aNode <> nil do
    begin
      if XMLValueAsStr(aNode, 'VENDOR_KEY') = aVendorKey then
        if XMLValueAsInt(aNode, 'TERM_TYPE') = Ord(aTermType) then
          if XMLValueAsInt(aNode, 'TERM_ACTIVE', 0) <> 1 then
            raise ETerminologyMapperError.CreateFmt(
              'Error Vendor Key ''%s'' in Mapping Table %s is mapped to inactive %s: %s.',
              [aVendorKey, fTableName, QUALNAME[aTermType], XMLValueAsStr(aNode, 'TERM_ID')])
          else
            begin
              Result := XMLValueAsStr(aNode, 'TERM_ID');
              Exit;
            end;
      aNode := aNode.NextSibling;
    end;

  raise ETerminologyMapperError.CreateFmt(
    'Error Locating Vendor Key ''%s'' in Mapping Table %s - Type: %s, SourceID: %s, SourceApplication: %s, SourceVersion: %s)',
    [aVendorKey, fTableName, QUALNAME[aTermType], fSourceID, fSourceApplication, fSourceVersion]);
end;

function TTermMapper.getValueFormat(aTermType: TTermType; aVendorKey: string): string;
var
  aNode: IXMLNode;
begin
  aNode := fXMLDocument.DocumentElement.ChildNodes.FindNode('RECORD');
  while aNode <> nil do
    begin
      if XMLValueAsStr(aNode, 'VENDOR_KEY') = aVendorKey then
        if XMLValueAsInt(aNode, 'TERM_TYPE') = Ord(aTermType) then
          begin
            Result := XMLValueAsStr(aNode, 'PARSER_KEY');
            Exit;
          end;
      aNode := aNode.NextSibling;
    end;
end;

procedure TTermMapper.Load;
begin
  //fLoaded := False; {Uncomment to force a reload of the table on every call}
  if not fLoaded then
    try
      fXMLDocument := NewXMLDocument;
      if fDBConnection.ExecQuery('GetMappingTableTermsByID', [fTableID], fXMLDocument) then
        fXMLDocument.Active := True;
      fLoaded := True;
    except
      fLoaded := False;
    end;
end;

function TTermMapper.Rename(const aNewName: string): boolean;
begin
  if fDBConnection.ExecCommand('UpdateMappingTableName', [fTableID, aNewName]) then
    begin
      fTableName := aNewName;
      Result := True;
    end
  else
    Result := False;
end;

procedure TTermMapper.setActive(const aValue: boolean);
begin
  //
end;

procedure TTermMapper.setComment(const aValue: string);
begin
  //
end;

procedure TTermMapper.setDisplayName(const aValue: string);
begin
  //
end;

procedure TTermMapper.setName(const aValue: string);
begin
  fTableName := aValue;
end;

procedure TTermMapper.setService(const aCPService: ICPService);
begin
  fCPService := aCPService;
end;

end.

