unit oTermPair;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 2 $  $Modtime: 4/08/08 9:52a $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Default terminology object.
*
*       Notes:        NEVER ACCESS DIRECTLY, ALWAYS USE ITERM.
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oTermPair.pas $
================================================================================
}
interface

uses
  SysUtils,
  Classes,
  ComCtrls,
  oCliOTypes,
  oInterfaces,
  Variants,
  XMLIntf;

type
  TTermPair = class(TInterfacedObject, ITermPair)
  private
    [weak] fTerm: ITerm;
    [weak] fQualifier: ITerm;
    fPairOrder: integer;
    fPairValue: Double;
  public
    destructor Destroy; override;
    function getTerm: ITerm; stdcall;
    function getQualifier: ITerm; stdcall;
    function getType: TTermType; stdcall;
    function getValue: Double; stdcall;
    function getOrder: integer; stdcall;

    procedure setValue(aValue: Double);
    procedure setQualifier(aValue: ITerm);
    procedure setOrder(aValue: integer);
  end;

function NewTermPair(aTerm: ITerm; aXMLNode: IXMLNode): ITermPair;

implementation

uses
  uCPXML;

function NewTermPair(aTerm: ITerm; aXMLNode: IXMLNode): ITermPair;
begin
  Result := TTermPair.Create;
  TTermPair(Result).fTerm := aTerm;
  if Assigned(aXMLNode) then
    with TTermPair(Result) do
      begin
        fQualifier := aTerm.getSiblingTerm(XMLValueAsStr(aXMLNode, 'QUALIFIER_ID'));
        fPairOrder := XMLValueAsInt(aXMLNode, 'PAIR_ORDER');
        fPairValue := XMLValueAsDouble(aXMLNode, 'PAIR_VALUE');
      end;
end;

{ TTermPair }

destructor TTermPair.Destroy;
begin
  fTerm := nil;
  fQualifier := nil;
  inherited;
end;

function TTermPair.getOrder: integer;
begin
  Result := fPairOrder;
end;

function TTermPair.getQualifier: ITerm;
begin
  Result := fQualifier;
end;

function TTermPair.getTerm: ITerm;
begin
  Result := fTerm;
end;

function TTermPair.getType: TTermType;
begin
  Result := fQualifier.getTermType;
end;

function TTermPair.getValue: Double;
begin
  Result := fPairValue
end;

procedure TTermPair.setOrder(aValue: integer);
begin
  fPairOrder := aValue;
end;

procedure TTermPair.setQualifier(aValue: ITerm);
begin
  fQualifier := aValue;
end;

procedure TTermPair.setValue(aValue: Double);
begin
  fPairValue := aValue;
end;

end.

