unit oTermUnit;

interface

uses
  Classes,
  SysUtils,
  XMLIntf,
  XMLDoc,
  oInterfaces;

type
  TTermUnit = class(TInterfacedObject, ITermUnit)
  private
    fTerm: ITerm;
    fUnit: ITerm;
    fValidMin: Double;
    fValidMax: Double;
    fReferenceMin: Double;
    fReferenceMax: Double;
    fValidDec: integer;
  public
    constructor Create;
    destructor Destroy; override;

    function getFormattedValidMin: string; stdcall;
    function getFormattedValidMax: string; stdcall;
    function getFormattedReferenceMin: string; stdcall;
    function getFormattedReferenceMax: string; stdcall;
    function getTerm: ITerm; stdcall;
    function getUnit: ITerm; stdcall;
    function getValidMin: Double; stdcall;
    function getValidMax: Double; stdcall;
    function getReferenceMin: Double; stdcall;
    function getReferenceMax: Double; stdcall;
    function getValidDec: integer; stdcall;
    function getParameters: string; stdcall;

    procedure setUnit(aUnit: ITerm);
    procedure setValidDec(aDec: integer);
    procedure setValidMin(aValidMin: Double);
    procedure setValidMax(aValidMax: Double);
    procedure setReferenceMin(aReferenceMin: Double);
    procedure setReferenceMax(aReferenceMax: Double);
  end;

function NewTermUnit(aTerm: ITerm; aXMLNode: IXMLNode): TTermUnit;

implementation

uses
  uCPXML;

function NewTermUnit(aTerm: ITerm; aXMLNode: IXMLNode): TTermUnit;
begin
  Result := TTermUnit.Create;
  Result.fTerm := aTerm;
  if Assigned(aXMLNode) then
    with Result do
      begin
        fUnit := aTerm.getSiblingTerm(XMLValueAsStr(aXMLNode, 'UNIT_ID'));
        fValidDec := XMLValueAsInt(aXMLNode, 'DEC_PRECISION');
        fValidMin := XMLValueAsDouble(aXMLNode, 'MIN_VALUE');
        fValidMax := XMLValueAsDouble(aXMLNode, 'MAX_VALUE');
        fReferenceMin := XMLValueAsDouble(aXMLNode, 'REFERENCE_LOW');
        fReferenceMax := XMLValueAsDouble(aXMLNode, 'REFERENCE_HIGH');
      end;
end;

{ TTerm_Unit }

constructor TTermUnit.Create;
begin
  inherited Create;
  fUnit := nil;
  fValidMin := 0;
  fValidMax := 0;
  fReferenceMin := 0;
  fReferenceMax := 0;
  fValidDec := 0;
end;

destructor TTermUnit.Destroy;
begin
  fUnit := nil;
  inherited;
end;

function TTermUnit.getFormattedReferenceMax: string;
begin
  Result := Format(Format('%%0.%df', [fValidDec]), [fReferenceMax]);
end;

function TTermUnit.getFormattedReferenceMin: string;
begin
  Result := Format(Format('%%0.%df', [fValidDec]), [fReferenceMin]);
end;

function TTermUnit.getFormattedValidMax: string;
begin
  Result := Format(Format('%%0.%df', [fValidDec]), [fValidMax]);
end;

function TTermUnit.getFormattedValidMin: string;
begin
  Result := Format(Format('%%0.%df', [fValidDec]), [fValidMin]);
end;

function TTermUnit.getParameters: string;
var
  aFmt: string;
begin
  aFmt := Format('%%0.%df', [fValidDec]);
  Result :=
    Format(aFmt, [fValidMin]) + ', ' +
    Format(aFmt, [fReferenceMin]) + ', ' +
    Format(aFmt, [fReferenceMax]) + ', ' +
    Format(aFmt, [fValidMax]) + ', ' +
    Format('%d', [fValidDec]);
end;

function TTermUnit.getReferenceMax: Double;
begin
  Result := fReferenceMax;
end;

function TTermUnit.getReferenceMin: Double;
begin
  Result := fReferenceMin;
end;

function TTermUnit.getTerm: ITerm;
begin
  Result := fTerm;
end;

function TTermUnit.getUnit: ITerm;
begin
  Result := fUnit;
end;

function TTermUnit.getValidDec: integer;
begin
  Result := fValidDec;
end;

function TTermUnit.getValidMax: Double;
begin
  Result := fValidMax;
end;

function TTermUnit.getValidMin: Double;
begin
  Result := fValidMin;
end;

procedure TTermUnit.setReferenceMax(aReferenceMax: Double);
begin
  fReferenceMax := aReferenceMax;
end;

procedure TTermUnit.setReferenceMin(aReferenceMin: Double);
begin
  fReferenceMin := aReferenceMin;
end;

procedure TTermUnit.setUnit(aUnit: ITerm);
begin
  fUnit := aUnit;
end;

procedure TTermUnit.setValidDec(aDec: integer);
begin
  fValidDec := aDec;
end;

procedure TTermUnit.setValidMax(aValidMax: Double);
begin
  fValidMax := aValidMax;
end;

procedure TTermUnit.setValidMin(aValidMin: Double);
begin
  fValidMin := aValidMin;
end;

end.

