unit oTerminology_Mapper;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 6 $  $Modtime: 4/14/10 8:45a $
*       Developer:    dan.petit@domain
*       Site:         Hines OIFO
*
*       Description:  Utilized by incoming messages to map vendor specific data
*                     elements to an actual VUID.
*
*       Notes:        Will raise ETerminologyMapperError exception if there
*                     is no mapping.
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/Common/objects/oTerminology_Mapper.pas $
*
* $History: oTerminology_Mapper.pas $
 * 
 * *****************  Version 6  *****************
 * User: Zzzzzzjuergb Date: 4/14/10    Time: 8:50a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Updated TObservationView, TObservationViewTerm, and TTermMapper to
 * derive from TInterfacedObject instead of TManagedInterfacedObject
 * 
 * *****************  Version 5  *****************
 * User: Zzzzzzpetitd Date: 3/21/07    Time: 1:52p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 12/05/06   Time: 1:51p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Added getValueFormat to oTerminology_Mapper and it's interface.
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzjuergb Date: 12/04/06   Time: 8:56a
 * Updated in $/CP Modernization/CliO/source/Common/objects
 * Getting Rid of a ShowMessage in TTerminology_Mapper.Load().
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 10/27/06   Time: 3:01p
 * Updated in $/CP Modernization/CliO/source/Common/objects
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 10/05/06   Time: 12:13p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  Dialogs,
  SysUtils,
  Windows,
  Messages,
  Classes,
  Controls,
  ComCtrls,
  oInterfaces,
  oCliO_Types,
  XMLDoc,
  XMLIntf;

type
  ETerminologyMapperError = class(Exception);

  TTerm_Mapper = class(TInterfacedObject, ITermMapper)
  private
    fXMLDocument: IXMLDocument;
    fTableID: string;
    fTableName: string;
    fSourceID: string;
    fSourceApplication: string;
    fSourceVersion: string;
    function Load(aDBConnection: IDBConnection): boolean; stdcall;
    function getTerm(aTermType: TTermType; aVendorKey: string): string; stdcall;
    function getValueFormat(aTermType: TTermType; aVendorKey: string): string; stdcall;
    function getTableID: string; stdcall;
    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
  public
    constructor Create;
    destructor Destroy;
  end;

function NewTerminologyMapper(aTableID, aTableName, aSourceID, aSourceApplication, aSourceVersion: string): ITermMapper;

implementation

function NewTerminologyMapper(aTableID, aTableName, aSourceID, aSourceApplication, aSourceVersion: string): ITermMapper;
begin
  with TTerm_Mapper.Create do
    try
      fTableID := aTableID;
      fTableName := aTableName;
      fSourceID := aSourceID;
      fSourceApplication := aSourceApplication;
      fSourceVersion := aSourceVersion;
      QueryInterface(ITermMapper, Result);
    except
      Result := nil;
    end;
end;

{ TTerminology_Mapper }

constructor TTerm_Mapper.Create;
begin
  fXMLDocument := NewXMLDocument;
end;

destructor TTerm_Mapper.Destroy;
begin
  fXMLDocument := nil;
end;

function TTerm_Mapper.getSourceApplication: string;
begin
  Result := fSourceApplication;
end;

function TTerm_Mapper.getSourceID: string;
begin
  Result := fSourceID;
end;

function TTerm_Mapper.getSourceVersion: string;
begin
  Result := fSourceVersion;
end;

function TTerm_Mapper.getTableID: string;
begin
  Result := fTableID;
end;

function TTerm_Mapper.getTerm(aTermType: TTermType; aVendorKey: string): string;
var
  aNode: IXMLNode;
begin
  aNode := fXMLDocument.DocumentElement.ChildNodes.FindNode('RECORD');
  while aNode <> nil do
    begin
      if XMLValueAsStr(aNode, 'VENDOR_KEY') = aVendorKey then
        if XMLValueAsInt(aNode, 'TERM_TYPE') = Ord(aTermType) then
          if XMLValueAsStr(aNode, 'TERM_VUID') <> '' then
            begin
              Result := XMLValueAsStr(aNode, 'TERM_VUID');
              Exit;
            end;
      aNode := aNode.NextSibling;
    end;

  if Result = '' then
    raise ETerminologyMapperError.CreateFmt(
      'Error Locating Vendor Key ''%s'' in Mapping Table %s - Type: %s, SourceID: %s, SourceApplication: %s, SourceVersion: %s)',
      [aVendorKey, fTableName, QUALNAME[aTermType], fSourceID, fSourceApplication, fSourceVersion]);
end;

function TTerm_Mapper.getValueFormat(aTermType: TTermType; aVendorKey: string): string;
var
  aNode: IXMLNode;
begin
  aNode := fXMLDocument.DocumentElement.ChildNodes.FindNode('RECORD');
  while aNode <> nil do
    begin
      if XMLValueAsStr(aNode, 'VENDOR_KEY') = aVendorKey then
        if XMLValueAsInt(aNode, 'TERM_TYPE') = Ord(aTermType) then
          begin
            Result := XMLValueAsStr(aNode, 'PARSER_KEY');
            Exit;
          end;
      aNode := aNode.NextSibling;
    end;
end;

function TTerm_Mapper.Load(aDBConnection: IDBConnection): boolean;
begin
  fXMLDocument := nil; {Remove this before production release}
  if fXMLDocument = nil then
    try
      fXMLDocument := NewXMLDocument;
      if aDBConnection.ExecQuery('GetMappingTableTermsByID', [fTableID], fXMLDocument) then
        begin
          fXMLDocument.Active := True;
          {
            blj 4 Dec 2006: doing a ShowMessage() in a service is contraindicated. :)
          }
          //ShowMessage('Loaded term mapper ' + fTableID + #13 + fXMLDocument.XML.Text);
          Result := True;
        end
      else
        Result := False;
    except
      Result := False;
    end;
end;

end.

